﻿using System.Threading;
using Dss.Api.Examples.Internal;
using System;
using System.IO;
using System.Linq;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.ExtractionRequests;
using DataScope.Select.Api.Extractions.ReportTemplates;
using DataScope.Select.Api.Extractions.SubjectLists;
using DataScope.Select.Core.RestApi;
using System.Text;

namespace Dss.Api.Examples.Extractions.OnDemand
{
    [Example("5. On Demand Extractions")]
    public class OnDemandExamples
    {
        protected ExtractionsContext ExtractionsContext { get; set; }

        public OnDemandExamples()
        {
            ExtractionsContext = ContextHelper.CreateExtractionsContext();
        }

        #region Scenarios

        [Example("Features: Pre-Validate")]
        public void Features_PreValidate()
        {

            Status.Notify(ExtractionsContext, "InstrumentList", "ValidateIdentifiers", MethodType.Operation, Publish.Secondary);
            //Pre-Validate the instruments: If the same instruments are to be used for multiple extractions for the same or
            //other different extraction types, there is a significant performance benefit to pre-validating the instruments
            //and invoking the extraction with the pre-validated instruments
            var validatedIdentifiers = ExtractionsContext.InstrumentListOperations.ValidateIdentifiers(
                new[]
                {
                    new InstrumentIdentifier {Identifier = "191216100", IdentifierType = IdentifierType.Cusip}
                }, true);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                //Use pre-validated instruments to reduce the extraction time
                IdentifierList = InstrumentIdentifierList.Create(validatedIdentifiers.ValidatedInstruments.ToArray(), null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                    "Ask Price", "High Price", "Low Price", "Volume", "Net Asset Value", "Official Close Price",
                    "Open Price", "Security Description" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                Status.WriteLine("# Rows with errors: {0}", extractedRows.Count(er => er.Error != null));
                var udf1 = extractedRows.First();
                foreach (var field in udf1.DynamicProperties)
                    Status.WriteLine(field.Key + ": " + field.Value);
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Features: User Defined Identifier")]
        public void Features_UserDefinedIdentifiers()
        {
            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip, UserDefinedIdentifier = @"_ :;@#&?=-+^,.!|abCD123\/~*%$`()[]{}'""<>"},
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol, UserDefinedIdentifier = "UDF-2" },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric, UserDefinedIdentifier = "UDF-3" }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                "CUSIP", "File Code", "Ask Price", "High Price", "Low Price", "Mid Price", "Volume", "Net Asset Value",
                "Offer Price", "Official Close Price", "Open Price", "Previous Close Price", "RIC",
                "Security Description", "SEDOL", "Ticker", "Trade Date", "User Defined Identifier" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                Status.WriteLine("# Rows with errors: {0}", extractedRows.Count(er => er.Error != null));
                var udf1 = extractedRows.First(r => r.UserDefinedIdentifier == @"_ :;@#&?=-+^,.!|abCD123\/~*%$`()[]{}'""<>");
                foreach (var field in udf1.DynamicProperties)
                    Status.WriteLine(field.Key + ": " + field.Value);
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Features: Pre-Validate with User Defined Identifiers")]
        public void Features_PreValidateWithUserDefinedIdentifiers()
        {

            Status.Notify(ExtractionsContext, "InstrumentList", "ValidateIdentifiers", MethodType.Operation, Publish.Secondary);
            //Pre-Validate the instruments
            var validatedIdentifiers = ExtractionsContext.InstrumentListOperations.ValidateIdentifiers(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip, UserDefinedIdentifier = "Group1"},
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol, UserDefinedIdentifier = "Group1" },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric, UserDefinedIdentifier = "Group2" }
                }, true);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                //Only use the instruments in Group1
                IdentifierList = InstrumentIdentifierList.Create(validatedIdentifiers.ValidatedInstruments.ToArray(), null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                    "Ask Price", "High Price", "Low Price", "Volume", "Net Asset Value", "Official Close Price",
                    "Open Price", "Security Description" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                Status.WriteLine("# Rows with errors: {0}", extractedRows.Count(er => er.Error != null));
                foreach (var row in extractedRows)
                    Status.WriteLine("{0} ({1}): {2} / Status = {3}",
                        row.Identifier, row.IdentifierType, row.UserDefinedIdentifier, row.DynamicProperties["Asset Status"]);
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Error Handling: Invalid Identifier")]
        public void ErrorHandling_InvalidIdentifier()
        {

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                //Use pre-validated instruments to reduce the extraction time
                IdentifierList = InstrumentIdentifierList.Create(new[]
                {
                    new InstrumentIdentifier {Identifier = "BAD_CUSIP", IdentifierType = IdentifierType.Cusip }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                    "Ask Price", "High Price", "Low Price", "Volume", "Net Asset Value", "Official Close Price",
                    "Open Price", "Security Description" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                Status.WriteLine("# Rows with errors: {0}", extractedRows.Count(er => er.Error != null));
                foreach (var row in extractedRows)
                    Status.WriteLine("{0} ({1}): Error: '{2}'", row.Identifier, row.IdentifierType, row.Error);
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Validation Options")]
        public void ValidationOptions()
        {
            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                },
                new InstrumentValidationOptions
                {
                    UseExchangeCodeInsteadOfLipper = true,
                    AllowHistoricalInstruments = false,
                    AllowOpenAccessInstruments = false,
                    ExcludeFinrAsPricingSourceForBonds = false,
                    UseConsolidatedQuoteSourceForCanada = false,
                    UseConsolidatedQuoteSourceForUsa = false,
                    UseUsQuoteInsteadOfCanadian = true
                },
                false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                "CUSIP", "File Code", "Ask Price", "High Price", "Low Price", "Mid Price", "Volume", "Net Asset Value",
                "Offer Price", "Official Close Price", "Open Price", "Previous Close Price", "RIC",
                "Security Description", "SEDOL", "Ticker", "Trade Date" },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechanism will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Entity Validation Options")]
        public void EntityValidationOptions()
        {

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new LegalEntityDetailExtractionRequest
            {
                IdentifierList = EntityIdentifierListWithValidationOptions.Create(
                    new[]
                    {
                        new EntityIdentifier { Identifier = "18228", IdentifierType = IdentifierType.OrgId }
                    }
                    , new EntityValidationOptions { AllowDuplicateInstruments = true, AllowUnmanagedOrUnverifiedEntities = true, IncludeParentAndUltimateParent = false }
                    , false
                    ),
                ContentFieldNames = new[] {
                    "CIK", "Company/Registration Authority", "Company/Registration Number", "Country of Incorporation",
                    "Country of Incorporation Description", "Date of Incorporation", "Domicile", "Domicile Description",
                    "Entity URL", "Event Date", "Event Type", "FATF Jurisdiction", "GICS Industry Code",
                    "ID", "ID Type", "Immediate Parent Official Name", "Immediate Parent RCP ID", "Immediate Parent Reuters Editorial RIC",
                    "Is Issuer", "Jurisdiction of Incorporation", "Official Name", "Primary Regulator", "Primary Regulator ID",
                    "RCP ID", "RCP Reuters Editorial RIC", "Record Active Flag", "Registered City/Town", "Registered Country",
                    "Registered Country Description", "Registered Fax", "Registered Postal/Zip Code", "Registered State/County/Region",
                    "S&P Issuer Long Rating", "S&P Issuer Long Rating - F", "S&P Issuer Long Rating Date", "S&P Issuer Long Rating Date - F",
                    "S&P Issuer Long Rating Rank", "S&P Issuer Long Rating Rank - F", "S&P Issuer Short Rating",
                    "S&P Issuer Short Rating - F", "S&P Issuer Short Rating Date", "S&P Issuer Short Rating Date - F",
                    "S&P Issuer Short Rating Rank", "S&P Issuer Short Rating Rank - F"},
                Condition = new LegalEntityDetailCondition
                {
                    DeltaDays = 100
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechanism will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);

        }


        #endregion

        #region End of Day

        [Example("Create: End of Day")]
        public void EndOfDay()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EndOfDayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                "CUSIP", "File Code", "Ask Price", "High Price", "Low Price", "Mid Price", "Volume", "Net Asset Value",
                "Offer Price", "Official Close Price", "Open Price", "Previous Close Price", "RIC",
                "Security Description", "SEDOL", "Ticker", "Trade Date" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Intraday

        [Example("Create: Intraday")]
        public void Intraday()
        {
            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);

            //Create the request
            var extractionRequest = new IntradayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                Condition = new IntradayPricingCondition { ScalableCurrency = true },
                ContentFieldNames = new[] {
                    "RIC", "Ask Price", "Asset Type", "Bid Price", "Currency Code",
                    "Exchange Code", "High Price", "Instrument ID", "Instrument ID Type", "Low Price", "Open Price",
                    "Previous Close Date", "Previous Close Price", "Security Description", "Settlement Price",
                    "Trade Date", "User Defined Identifier", "Volume" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechanism will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Intraday with OnlyNonEmbargoedData")]
        public void IntradayWithOnlyNonEmbargoedData()
        {
            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);

            //Create the request
            var extractionRequest = new IntradayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                            new InstrumentIdentifier { Identifier = "AAPL.O", IdentifierType = IdentifierType.Ric },
                            new InstrumentIdentifier { Identifier = "AMD.OQ", IdentifierType = IdentifierType.Ric },
                    }, null, false),
                Condition = new IntradayPricingCondition { ScalableCurrency = false, OnlyNonEmbargoedData = true },
                ContentFieldNames = new[] {
                    "RIC", "Ask Price", "Asset Type", "Bid Price", "Currency Code",
                    "Exchange Code", "High Price", "Instrument ID", "Instrument ID Type", "Low Price", "Open Price",
                    "Previous Close Date", "Previous Close Price", "Security Description", "Settlement Price",
                    "Trade Date", "User Defined Identifier", "Volume" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechanism will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Fixed Income Analytics

        [Example("Create: Fixed Income Analytics")]
        public void FixedIncomeAnalytics()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.FixedIncomeAnalytics);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the request
            var extractionRequest = new FixedIncomeAnalyticsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[]
                    {
                        new InstrumentIdentifier { Identifier = "912810RJ9", IdentifierType = IdentifierType.Cusip},
                    }, null, false),
                ContentFieldNames = new[] {
                    "Instrument ID", "Econometric Corp Convexity", "Econometric Corp Modified Duration", "Econometric Corp Yield"
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Technical Indicators

        [Example("Create: Technical Indicators")]
        public void TechnicalIndicators()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TechnicalIndicators);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new TechnicalIndicatorsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Alpha - 3 Year Weekly", "Alpha - 3 Year Weekly - Down", "Alpha - 3 Year Weekly - Up", "Asset Category",
                    "Asset Category Description", "Asset ID", "Asset Status", "Asset Status Description", "Asset SubType",
                    "Asset SubType Description", "Asset Type", "Asset Type Description", "Average Volume - 10 Days", "Average Volume - 100 Days",
                    "Average Volume - 50 Days", "Average Volume - 60 Days", "Average Volume - 80 Days", "Average Volume - 90 Days",
                    "Average Volume - Monthly 13 Weeks", "Benchmark RIC", "Beta - 3 Year Weekly", "CFI Code", "Close Price - 1 Day",
                    "Close Price - 13 Weeks", "Close Price - 2 Days", "Close Price - 52 Week High", "Close Price - 52 Week Low"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Legal Entity Audit

        [Example("Create: Legal Entity Audit")]
        public void LegalEntityAudit()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityAudit);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new LegalEntityAuditExtractionRequest
            {
                IdentifierList = EntityIdentifierList.Create(
                    new[] { new EntityIdentifier { Identifier = "18228", IdentifierType = IdentifierType.OrgId } }),
                ContentFieldNames = new[] {
                    "Audit Modified Date", "Audit Source", "Audit Source Reference", "Audited Field Name", "ID", "ID Type",
                    "Official Name", "RCP ID", "User Defined Identifier"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Composite

        [Example("Create: Composite")]
        public void Composite()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.Composite);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new CompositeExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Daily Beta - 180 Days", "Volatility - 30 days", "Market Capitalization", "Average Volume - 30 Days",
                    "Volume", "Total Shares - Listed"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region MBS

        [Example("Create: MBS")]
        public void CreateMbsTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.MBSFactorHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new MBSFactorHistoryExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset ID", "April Factor", "Asset SubType", "Asset SubType Description", "Asset Type", "Asset Type Description",
                    "August Factor", "CUSIP", "December Factor", "Factor Year", "February Factor", "Instrument ID",
                    "Instrument ID Type", "January Factor", "July Factor", "June Factor", "March Factor", "May Factor",
                    "November Factor", "October Factor", "Security Description", "September Factor", "Usage Instrument Type"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Tranche Factor History

        [Example("Create: Tranche Factor History")]
        public void CreateTrancheTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TrancheFactorHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new TrancheFactorHistoryExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "CUSIP", "Factor", "Factor Date", "Asset ID", "Asset SubType", "Asset SubType Description", "Asset Type",
                    "Asset Type Description", "Common Code", "ISIN",
                    "Instrument ID", "Instrument ID Type", "SEDOL", "Security Description", "Usage Instrument Type",
                    "Wertpapier", "User Defined Identifier"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Ownership

        [Example("Create: Ownership")]
        public void CreateOwnershipTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.Owners);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new OwnershipExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset ID", "Asset Status", "Asset Status Description",
                    "Change Sign", "Common Code", "Currency Code", "Currency Code Description", "CUSIP", "File Code",
                    "Issuer ID", "Issuer Name", "Issuer OrgID", "RCP ID", "Issuer Short Name", "Legal Entity OrgID",
                    "Holdings Previous Report Date", "Holdings Report Date", "Owner Country", "Owner ID", "Owner Name", "Owner Type",
                    "Owner Type Description", "Percent of Shares Outstanding", "Primary Issue Flag", "Security Description",
                    "Shares Held", "Value Held", "Shares Changed", "Value Changed"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows.Take(10))
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Terms and Conditions

        [Example("Create: Terms and Conditions")]
        public void TermsAndConditions()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TermsAndConditions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create new report template with conditions
            var extractionRequest = new TermsAndConditionsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Conversion Terms End Date", "Index Principal Flag", "Conversion Terms Start Date", "RIC", "Round Lot Size",
                    "Redemption Value", "Original Issue Amount", "Issue Price", "First Coupon Date", "Issue Date", "Par Value",
                    "Accrual Date", "Base Index", "Factor", "Original Issue Discount Flag", "End Of Month Payment Flag", "ISIN",
                    "Index Linked Bond Base Index", "Coupon Frequency Description", "Total Amount Outstanding", "Maturity Date",
                    "Total Amount Issued", "Capitilization Flag", "Currency Code", "Trading Status"},
                Condition = new TermsAndConditionsCondition
                {
                    IssuerAssetClassType = IssuerAssetClassType.AllSupportedAssets,
                    ExcludeWarrants = false,
                    //DaysAgo = 3, //Use either DaysAgo or StartDate
                    StartDate = new DateTimeOffset(new DateTime(2015, 1, 1)),
                    FixedIncomeRatingSources = FixedIncomeRatingSource.StandardAndPoors,
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Time Series

        [Example("Create: Time Series")]
        public void TimeSeries()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TimeseriesPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new TimeSeriesExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] {
                    "File Code", "RIC", "Trade Date", "Close Price", "Alternate Close Price", "Split Factor", "High Price",
                    "Low Price", "Open Price", "VWAP Price", "Turnover", "Volume", "Bid Price", "Asset Type", "Quote ID",
                    "Bid Yield", "Universal Close Price", "Exchange Code", "Currency Code"},
                Condition = new TimeSeriesCondition
                {
                    StartDate = DateTime.UtcNow.Date.AddDays(-7),
                    EndDate = DateTime.UtcNow.Date
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Bond Schedule

        [Example("Create: Bond Schedule")]
        public void BondSchedule()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.BondSchedules);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetBondScheduleTypes", MethodType.Operation, Publish.Primary);
            //Get the list of bond schedule types
            var bondScheduleTypes = ExtractionsContext.ReportTemplateOperations.GetBondScheduleTypes();

            //Output the list of codes
            Status.WriteLine(String.Join(", ", bondScheduleTypes.Select(bst => bst.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);


            //Status.Notify(ExtractionsContext, "BondScheduleReportTemplate", "Create");
            //Create the new report template
            var extractionRequest = new BondScheduleExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Instrument ID", "Instrument ID Type", "Asset ID", "Asset Type", "CUSIP",
                    "Common Code", "Extend Holders Option Flag", "MIC",
                    "RIC", "SEDOL", "SICC", "Schedule Type", "Security Description", "User Defined Identifier"},
                Condition = new BondScheduleCondition
                {
                    BondScheduleTypeCodes = new[] { "CALS", "PUTS" }
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Ratings

        [Example("Create: Ratings")]
        public void Ratings()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.BondRatings);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetRatingsSourceTypes", MethodType.Operation, Publish.Primary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var ratings = ExtractionsContext.ReportTemplateOperations.GetRatingsSourceTypes();
            Status.WriteLine(String.Join(", ", ratings.Select(lbp => lbp.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);

            //Create the new report template
            var extractionRequest = new RatingsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset ID", "Asset Status", "Asset Status Description", "Asset Status Effective Date", "Asset SubType",
                    "Asset Subtype Country Code", "Asset SubType Description", "Asset Type", "Asset Type Description",
                    "Bond Type", "Bond Type Description", "Country of Issuance", "Country of Issuance Description",
                    "CUSIP", "Domicile", "Domicile Description", "Entity ID", "FirmID", "Identifier Segment Source",
                    "Identifier Segment Source Description", "Instrument ID", "Instrument ID Type", "Instrument Type Code",
                    "Instrument Type Code Description", "ISIN", "ISO Country Code", "ISO Currency", "Local Code", "Rating Type",
                    "RIC", "Security Description", "Series", "SIC Classification Code", "SIC Description1", "SICC", "Sicovam",
                    "Ticker", "Usage Instrument SubType", "Usage Instrument Type"},
                Condition = new RatingsCondition
                {
                    DaysAgo = 7,
                    ReportDateRangeType = ReportDateRangeType.Range,
                    QueryStartDate = DateTime.Now.AddDays(-30),
                    QueryEndDate = DateTime.Now,
                    IncludeInstrumentsWithNoRatings = false,
                    RatingLevel = new RatingLevel { IsIssue = false, IsIssuer = true, IssuerOutlookScopeCodeFilter = "F" },
                    RatingSourcesCodes = new[] { "ASI", "CAR", "SSU" }
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Standard Events (Corporate Actions)

        [Example("Create: Standard Events (Corporate Actions)")]
        public void CorporateActionsStandard()
        {
            Status.Notify(ExtractionsContext, null, "GetValidContentFieldTypes", MethodType.Operation, Publish.Primary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.WriteLine(String.Join(", ", availableFields.Select(lbp => lbp.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetValidContentFieldTypesForTemplateCode", MethodType.Operation, Publish.Primary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFieldsForTempl = ExtractionsContext.GetValidContentFieldTypesForTemplateCode("COR");
            Status.WriteLine(String.Join(", ", availableFieldsForTempl.Select(lbp => lbp.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetValidExtractionFieldNames", MethodType.Operation, Publish.Primary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var fieldNames = ExtractionsContext.GetValidExtractionFieldNames(ReportTemplateTypes.CorporateActions);
            foreach (var field in fieldNames)
            {
                Status.WriteLine("{0})", field);
            }
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetValidExtractionFieldNamesForTemplateCode", MethodType.Operation, Publish.Primary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var fieldNmTemplCode = ExtractionsContext.GetValidExtractionFieldNamesForTemplateCode("COR");
            foreach (var field in fieldNmTemplCode)
            {
                Status.WriteLine("{0})", field);
            }
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new CorporateActionsStandardExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                }, null, false),
                ContentFieldNames = new[] {
                    "Capital Change Announcement Date", "Capital Change Ex Date", "Effective Date", "Record Date",
                    "Capital Change Event Type", "Old Shares Terms", "New Shares Terms", "New Shares Currency" },
                Condition = new CorporateActionsStandardCondition
                {
                    PreviousDays = 30,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    CorporateActionsCapitalChangeType = CorporateActionsCapitalChangeType.CapitalChangeExDate,
                    CorporateActionsDividendsType = CorporateActionsDividendsType.DividendPayDate,
                    CorporateActionsEarningsType = CorporateActionsEarningsType.PeriodEndDate,
                    CorporateActionsEquityOfferingsType = CorporateActionsEquityOfferingsType.AllPendingDeals,
                    CorporateActionsMergersAcquisitionsType = CorporateActionsMergersAcquisitionsType.DealAnnouncementDate,
                    CorporateActionsNominalValueType = CorporateActionsNominalValueType.NominalValueDate,
                    CorporateActionsSharesType = CorporateActionsSharesType.SharesAmountDate,
                    CorporateActionsStandardEventsType = CorporateActionsStandardEventsType.None,
                    CorporateActionsVotingRightsType = CorporateActionsVotingRightsType.VotingRightsDate,
                    QueryStartDate = null,
                    NextDays = null,
                    QueryEndDate = null,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    IncludeInstrumentsWithNoEvents = null,
                    IncludeNullDates = null,
                    ExcludeDeletedEvents = true,
                    ExcludeHistoricalEvents = false,
                    IncludeCapitalChangeEvents = true,
                    IncludeDividendEvents = true,
                    IncludeEarningsEvents = true,
                    IncludeMergersAndAcquisitionsEvents = true,
                    IncludeNominalValueEvents = true,
                    IncludePublicEquityOfferingsEvents = true,
                    IncludeSharesOutstandingEvents = true,
                    IncludeVotingRightsEvents = true
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region IPO Events (Corporate Actions)

        [Example("Create: IPO Events (Corporate Actions)")]
        public void CorporateActionsIpo()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetExchangeTypes", MethodType.Operation, Publish.Primary);
            //Retrieve the list of available exchnage types.  You need not retrieve this list every time.  The report condition
            //accepts a list of exchange type codes which can be hard-coded.
            var exchangeTypes = ExtractionsContext.ReportTemplateOperations.GetExchangeTypes();

            //Output
            foreach (var exchangeType in exchangeTypes)
                Status.WriteLine("{0}: {1}", exchangeType.Code, exchangeType.Name);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new CorporateActionsIpoExtractionRequest
            {
                ContentFieldNames = new[] {
                    "CUSIP", "City", "Common Code", "Company Name at Offer", "Corporate Actions Type", "Country", "Currency Code",
                    "Email", "Entitlement", "Exchange Code", "Exchange Offer End Date", "Exchange Offer Start Date",
                    "Expected Pricing Date", "Instrument ID", "Instrument ID Type", "Issuer OrgID", "Market Level", "Nominal Value",
                    "Nominal Value Currency", "Number of Shares", "Offer Exchange", "Offer Exchange Description", "Offer ID",
                    "Offer Status", "Offer Sub-Status", "Offer Type", "Shares Type", "Sicovam", "State Code", "Target Group",
                    "Ticker", "Total Amount Raised", "Total Shares Offered", "Usage Instrument SubType", "Usage Instrument Type",
                    "User Defined Identifier", "Web Site"},
                Condition = new CorporateActionsIpoCondition
                {
                    PreviousDays = 30,
                    //QueryStartDate = null,
                    //NextDays = null,
                    //QueryEndDate = null,
                    IncludeInstrumentsWithNoEvents = true,
                    IncludeNullDates = null,
                    ExchangeTypes = new[] { "NYS", "LSE" },
                    ReportDateRangeType = ReportDateRangeType.Range
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region ISO 15022 (Corporate Actions)

        [Example("Create: ISO 15022 (Corporate Actions)")]
        public void CorporateActionsIso()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var extractionRequest = new CorporateActionsIsoExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                Condition = new CorporateActionsIsoCondition
                {
                    CorporateActionsStandardYearType = CorporateActionsStandardYearType.Current,
                    ExcludeNilPaidFromPaymentEvents = true,
                    GrossAmountOnlyForPaymentEvents = true,
                    IncludeNonPendingEvents = true,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    PendingEventsPreviousDays = null,
                    PendingEventsPreviousMonths = null,
                    ReportDateRangeType = ReportDateRangeType.Init,
                    ReportIsoEvents = new[] { ReportIsoEventType.ACTV, ReportIsoEventType.BIDS }
                },
            };

            //NO FIELDS FOR CORAX ISO - the fields are pre-determined

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                var result = reader.ReadToEnd();

                //Output
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region ISO 15022 (DebtCorporate Actions)

        [Example("Create: ISO 15022 (Debt Corporate Actions)")]
        public void DebtCorporateActionsIso()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.DebtCorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var extractionRequest = new DebtCorporateActionsIsoExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                Condition = new DebtCorporateActionsIsoCondition
                {
                    DebtCorporateActionsStandardYearType = DebtCorporateActionsStandardYearType.Current,
                    IncludeNonPendingEvents = true,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    PendingEventsPreviousDays = null,
                    PendingEventsPreviousMonths = null,
                    ReportDateRangeType = ReportDateRangeType.Init,
                    ReportIsoEvents = new[] { ReportDebtIsoEventType.BRUP }
                },
            };

            //NO FIELDS FOR Debt CORAX ISO - the fields are pre-determined

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                var result = reader.ReadToEnd();

                //Output
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Detail (Entity Data)

        [Example("Create: Detail (Entity Data)")]
        public void LegalEntityDetail()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityDetail);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new LegalEntityDetailExtractionRequest
            {
                IdentifierList = EntityIdentifierList.Create(
                    new[]
                    {
                        new EntityIdentifier { Identifier = "18228", IdentifierType = IdentifierType.OrgId, UserDefinedIdentifier = "(Good!)" },
                        new EntityIdentifier { Identifier = "18236", IdentifierType = IdentifierType.OrgId, UserDefinedIdentifier = "ßÃÐ" },
                    }),
                ContentFieldNames = new[] {
                    "CIK", "Company/Registration Authority", "Company/Registration Number", "Country of Incorporation",
                    "Country of Incorporation Description", "Date of Incorporation", "Domicile", "Domicile Description",
                    "Entity URL", "Event Date", "Event Type", "FATF Jurisdiction", "GICS Industry Code",
                    "ID", "ID Type", "Immediate Parent Official Name", "Immediate Parent RCP ID", "Immediate Parent Reuters Editorial RIC",
                    "Is Issuer", "Jurisdiction of Incorporation", "Official Name", "Primary Regulator", "Primary Regulator ID",
                    "RCP ID", "RCP Reuters Editorial RIC", "Record Active Flag", "Registered City/Town", "Registered Country",
                    "Registered Country Description", "Registered Fax", "Registered Postal/Zip Code", "Registered State/County/Region",
                    "S&P Issuer Long Rating", "S&P Issuer Long Rating - F", "S&P Issuer Long Rating Date", "S&P Issuer Long Rating Date - F",
                    "S&P Issuer Long Rating Rank", "S&P Issuer Long Rating Rank - F", "S&P Issuer Short Rating",
                    "S&P Issuer Short Rating - F", "S&P Issuer Short Rating Date", "S&P Issuer Short Rating Date - F",
                    "S&P Issuer Short Rating Rank", "S&P Issuer Short Rating Rank - F"},
                Condition = new LegalEntityDetailCondition
                {
                    DeltaDays = 100
                },
            };

            //Add fields

            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Hierarchy (Entity Data)

        [Example("Create: Hierarchy (Entity Data)")]
        public void LegalEntityHierarchy()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityHierarchy);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new LegalEntityHierarchyExtractionRequest
            {
                IdentifierList = EntityIdentifierList.Create(
                    new[] { new EntityIdentifier { Identifier = "18228", IdentifierType = IdentifierType.OrgId } }),
                ContentFieldNames = new[] {
                    "ID", "ID Type", "Immediate Parent ID", "Immediate Parent Official Name",
                    "Immediate Parent RCP ID", "Node/Level", "Official Name", "RCP ID",
                    "Source RCP ID", "Ultimate Parent ID", "Ultimate Parent Official Name", "Ultimate Parent RCP ID",
                    "User Defined Identifier"},
                Condition = new LegalEntityHierarchyCondition
                {
                    DeltaDays = 100
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Historical Reference (HRD)

        [Example("Create: Historical Reference")]
        public void HistoricalReference()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.HistoricalReference);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);

            //Create the request
            var extractionRequest = new HistoricalReferenceExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers =
                        new[] { new InstrumentIdentifier { Identifier = "TRI.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new HistoricalReferenceCondition()
                {
                    ReportDateRangeType = ReportDateRangeType.Range,
                    QueryStartDate = new DateTimeOffset(new DateTime(2014, 12, 1)),
                    QueryEndDate = new DateTimeOffset(new DateTime(2015, 12, 2))
                },
                ContentFieldNames = new[]
                {
                    "Asset Category",
                    "Australia Code",
                    "CESR Average Daily Turnover",
                    "CESR Average Daily Turnover Currency Code",
                    "CESR Average Value of Orders Executed",
                    "CESR Average Value of Orders Executed Currency Code",
                    "CESR EEA Regulated",
                    "CESR Free Float",
                    "CESR Free Float Currency Code",
                    "CESR Most Relevant Market",
                    "CESR Standard Market Size",
                    "CESR Standard Market Size Currency Code",
                    "CFI Code"
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Summary Estimates (Estimates Data)

        [Example("Create: Summary Estimates (Estimates Data)")]
        public void EstimatesSummary()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesSummary);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesSummaryExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                "RIC", "Security Description", "Period", "Period Date", "Measure", "Flash Mean Estimate", "Flash Number of Estimates",
                "High Estimate", "High Estimate - 3 Months ago", "High Estimate - 4 Weeks ago", "Low Estimate",
                "Low Estimate - 3 Months ago", "Low Estimate - 4 Weeks ago", "Mean Estimate", "Mean Estimate - 3 Months ago",
                "Mean Estimate - 4 Weeks ago", "Median Estimate", "Median Estimate - 3 Months ago",
                "Median Estimate - 4 Weeks ago", "Number of Estimates", "Number of Estimates - 3 Months ago",
                "Number of Estimates - 4 Weeks ago", "Number of Estimates Lowered - 1 Month",
                "Number of Estimates Lowered - 1 Week", "Number of Estimates Lowered - 4 Weeks", "Number of Estimates Raised - 1 Month",
                "Number of Estimates Raised - 1 Week", "Number of Estimates Raised - 4 Weeks", "Estimate Currency Code"},
                Condition = new EstimatesSummaryCondition
                {
                    DaysAgo = 30,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = RelativeFiscalYearType.FY1,
                    FiscalYearTo = RelativeFiscalYearType.FY3,
                    QuarterFrom = RelativeQuarterType.None,
                    QuarterTo = RelativeQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    SemiAnnualFrom = RelativeSemiAnnualType.None,
                    SemiAnnualTo = RelativeSemiAnnualType.None
                },
            };

            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Actual Estimates (Estimates Data)

        [Example("Create: Actual Estimates (Estimates Data)")]
        public void EstimatesActual()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesActual);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesActualExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Actual Currency", "Actual Date", "Actual Status", "Actual Value", "Mean Estimate", "Measure", "MIC",
                    "Number of Estimates", "OPOL", "Period", "Period Date", "Quote ID", "RCP ID", "Reuters Editorial RIC",
                    "RIC", "Security Description", "SEDOL", "SICC", "Sicovam", "Standard Deviation", "Ticker", "Trading Status",
                    "Update Date", "Update Type", "Usage Instrument SubType", "Usage Instrument Type"},
                Condition = new EstimatesActualCondition
                {
                    DaysAgo = 365,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = HistoricalFiscalYearType.FYNegative1,
                    FiscalYearTo = HistoricalFiscalYearType.FYNegative5,
                    QuarterFrom = HistoricalQuarterType.None,
                    QuarterTo = HistoricalQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Range,
                    SemiAnnualFrom = HistoricalSemiAnnualType.None,
                    SemiAnnualTo = HistoricalSemiAnnualType.None,
                    ExcludeRestatedActuals = true
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Detail Estimates (Estimates Data)

        [Example("Create: Detail Estimates (Estimates Data)")]
        public void EstimatesDetail()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesDetail);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesDetailExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "RIC", "Period", "Period Date", "Previous Broker Recommendation", "Previous Broker Recommendation Description",
                    "Previous Currency", "Previous Estimate", "Previous Horizon", "Previous IBES Entry Date",
                    "Previous IBES Recommendation", "Previous IBES Recommendation Description", "Previous Is Diluted",
                    "Previous Is Excluded", "Previous Status", "Quote ID", "RCP ID", "Reuters Editorial RIC",
                    "Ticker", "Trading Status", "Update Date", "Update Type", "Usage Instrument SubType", "Usage Instrument Type",
                    "User Defined Identifier"},
                Condition = new EstimatesDetailCondition
                {
                    DaysAgo = 90,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = RelativeFiscalYearType.FY1,
                    FiscalYearTo = RelativeFiscalYearType.FY3,
                    QuarterFrom = RelativeQuarterType.None,
                    QuarterTo = RelativeQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Range,
                    SemiAnnualFrom = RelativeSemiAnnualType.None,
                    SemiAnnualTo = RelativeSemiAnnualType.None
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Company Level Footnotes (Estimates Data)

        [Example("Create: Company Level Footnotes (Estimates Data)")]
        public void EstimatesFootnotesCompany()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesCompanyFootnotes);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesFootnotesCompanyExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset ID", "Asset Status", "Asset Status Description", "CUSIP", "Common Code", "Company Name", "Country",
                    "Currency Code", "Currency Code Description", "Dilution Factor", "Exchange", "Exchange Code",
                    "Exchange Description", "Footnote", "Footnote Expiry Date", "Footnote Type", "IBES Code", "IBES Entry Date",
                    "ISIN", "Issuer Name", "RIC", "Security Description", "Ticker", "Trading Status", "Update Date",
                    "Update Type", "Usage Instrument SubType", "Usage Instrument Type"},
                Condition = new EstimatesFootnotesCompanyCondition
                {
                    DaysAgo = 30,
                    ReportDateRangeType = ReportDateRangeType.Delta
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Company Level Footnotes (Estimates Data)

        [Example("Create: Detail Level Footnotes (Estimates Data)")]
        public void EstimatesFootnotesDetail()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.DetailEstimateFootnotes);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesFootnotesDetailExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                "Analyst Name", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type", "Asset Type Description",
                "Broker Name", "Dilution Factor", "Estimate Flag", "Exchange", "Exchange Code", "Exchange Description",
                "File Code", "Footnote", "Footnote Expiry Date", "Footnote Type", "IBES Code", "IBES Estimate Entry Date",
                "IBES Footnote Entry Date", "Issuer Name", "Issuer OrgID", "Market MIC", "Measure", "Period", "Period Date",
                "Quote ID", "RIC", "Trading Status", "Update Date", "Update Type", "Usage Instrument SubType",
                "Usage Instrument Type"},
                Condition = new EstimatesFootnotesDetailCondition
                {
                    DaysAgo = 30,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = RelativeFiscalYearType.FY1,
                    FiscalYearTo = RelativeFiscalYearType.FY3,
                    QuarterFrom = RelativeQuarterType.None,
                    QuarterTo = RelativeQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    SemiAnnualFrom = RelativeSemiAnnualType.None,
                    SemiAnnualTo = RelativeSemiAnnualType.None
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region News Items (News)

        [Example("Create: News Items")]
        public void NewsItems()
        {
            Status.Notify(ExtractionsContext, null, "GetContentFieldTypes", MethodType.Operation, Publish.Primary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.NewsItems);
            Status.WriteLine(String.Join(", ", availableFields.Select(lbp => lbp.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetNewsAttributionTypes", MethodType.Operation, Publish.Primary);
            //Get the list of attributions which can be applied to Condition.NewsAttributionsCodes
            var newsAttributions = ExtractionsContext.ReportTemplateOperations.GetNewsAttributionTypes();
            Status.WriteLine(String.Join(", ", newsAttributions.Select(lbp => lbp.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetNewsTopicTypes", MethodType.Operation, Publish.Primary);
            //Get the list of topics which can be applied to Condition.NewsTopicsCodes
            var newsTopics = ExtractionsContext.ReportTemplateOperations.GetNewsTopicTypes();
            Status.WriteLine(String.Join(", ", newsTopics.Select(lbp => lbp.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetNewsItemsLanguageTypes", MethodType.Operation, Publish.Primary);
            //Get the list of languages which can be applied to Condition.NewsItemsLanguageCodes
            var newsLanguages = ExtractionsContext.ReportTemplateOperations.GetNewsItemsLanguageTypes();
            Status.WriteLine(String.Join(", ", newsLanguages.Select(lbp => lbp.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new NewsItemsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                "Asset ID", "Attribution", "Common Code", "CUSIP", "File Code", "Headline", "Instrument ID", "Instrument ID Type",
                "ISIN", "Language", "OPOL", "PILC", "PNAC", "Products", "Quote ID", "Related RICs", "Reuters Editorial RIC",
                "RIC", "SEDOL", "SICC", "Story Body", "Story Date Time", "Take Date Time", "Ticker", "Topics",
                "Unique Story Index"},
                Condition = new NewsItemsCondition
                {
                    QueryStartDate = null,
                    QueryEndDate = null,
                    DaysAgo = 1,
                    HoursAgo = 20,
                    MinutesAgo = 40,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    NewsAttributionsCodes = new[] { "RTRS", "JSE" }, //from newsAttributionCodes
                    NewsTopicsCodes = new[] { "GDP", "ECI", "MCE" }, //from newsTopics
                    NewsItemsSource = NewsItemsSourceType.AllNews,
                    NewsItemsLanguage = NewsItemsLanguageType.Selected,
                    NewsLanguagesCodes = new[] { "EN" } //from newsLanguages
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Premimum Of Day

        [Example("Create: Premium End Of Day")]
        public void PremiumEndOfDayPricing()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumEndOfDayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new PremiumEndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Instrument ID", "Instrument ID Type", "Trading Indicator", "Trading Indicator Description",
                    "Universal Close Validation Code", "Universal Close Validation Code Description", "Universal Bid Price",
                    "Universal Ask Price", "Universal Bid Ask Date", "Universal Close Price", "Universal Close Price Date",
                    "RIC", "ISIN", "Asset Status", "Asset Status Description", "Asset SubType", "Asset SubType Description",
                    "Asset Type", "Asset Type Description", "Currency Code", "Currency Code Description",
                    "Current Embargo Delay", "Embargo Times", "Embargo Window", "Exchange Requiring Embargo",
                    "File Code", "Instrument Snap Time", "Last Intraday Update Time", "Maximum Embargo Delay",
                    "Real Time Permitted", "PE Code", "Security Description", "Trading Status"},
                Condition = new PremiumEndOfDayPricingCondition
                {
                    IncludeIntradayPricing = true,
                    IntradayFixedIncomePremiumEndOfDay = true,
                    ScalableCurrency = false
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Premium Pricing

        [Example("Create: Premium Pricing")]
        public void CreatePremiumPricingReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new PremiumPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "912810RJ9", IdentifierType = IdentifierType.Cusip } }, null, false),
                ContentFieldNames = new[]
                {
                   "Accrued Interest", "Ask Price", "Asset Status", "Asset Type", "Bid Ask Spread", "Bid Price", "Bid Yield",
                   "Date Moving Convention Description", "Highest Quote", "Issuer Movement", "Issuer Name", "LIBOR Rate",
                   "Lowest Quote", "Maturity Date", "Payment Frequency", "Previous Close Price", "Pricing Date",
                   "Pricing Method Code", "Pricing Method Value", "Pricing Model Code", "Pricing Region Code", "Pricing Time",
                   "Settlement Date"
                },
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingAsia3PM,
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Single Historical Price

        [Example("Create: Single Historical Price (Pricing Data)")]
        public void SingleHistoricalPrice()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TimeseriesPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new SingleHistoricalPriceExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Close Price", "Bid Price", "Ask Price", "Trade Date", "CUSIP", "RIC", "Security Description",
                    "Alternate Close Price", "Low Price", "High Price", "Mid Price", "Open Price", "Instrument ID"},
                Condition = new SingleHistoricalPriceCondition
                {
                    LookBackPeriod = LookbackType.OneMonth,
                    PriceDate = DateTime.UtcNow,
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Single Price History

        [Example("Create: Single Price History")]
        public void SinglePriceHistory()
        {
            //Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            //var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PriceHistory);
            //Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);

            //Create the request
            var extractionRequest = new SinglePriceHistoryExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "TRI.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new SinglePriceHistoryCondition()
                {
                    //QueryStartDate = new DateTimeOffset(new DateTime(2015, 12, 1)),
                    //QueryEndDate = new DateTimeOffset(new DateTime(2015, 12, 2))
                    EndDate = DateTime.UtcNow.AddDays(-1),
                    Lookback = LookbackType.FourMonths,
                    AdjustedPrices = false,
                    LastEntityOnly = false
                },
                ContentFieldNames = new[]
                    {
                        "Ask High", "Ask Low", "Bid High", "Bid Low", "Trade Date"
                    }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Price History

        [Example("Create: Price History")]
        public void PriceHistory()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PriceHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);

            //Create the request
            var extractionRequest = new PriceHistoryExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "TRI.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new PriceHistoryCondition()
                {
                    QueryStartDate = new DateTimeOffset(new DateTime(2015, 12, 1)),
                    QueryEndDate = new DateTimeOffset(new DateTime(2015, 12, 2)),
                    LastEntityOnly = false
                },
                ContentFieldNames = new[]
                    {
                        "Ask High", "Ask Low", "Bid High", "Bid Low", "Trade Date"
                    }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Fund Allocation

        [Example("Create: Fund Allocation (Reference Data)")]
        public void FundAllocation()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.FundAllocation);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new FundAllocationExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset SubType Description", "Asset Type", "Asset Type Description", "CUSIP",
                    "Identifier Segment Source",
                    "Identifier Segment Source Description", "Instrument ID", "Instrument ID Type", "Security Description",
                    "Allocation Asset Type", "Allocation CUSIP", "Allocation Date", "Allocation Item", "Allocation OrgID",
                    "Allocation Percentage", "Allocation Rank", "Market Value Currency", "Market Value Held", "RIC",
                    "FundSERVAC", "FundSERVBE", "FundSERVDM", "FundSERVDO", "FundSERVFE", "FundSERVIS", "FundSERVLL",
                    "FundSERVNL", "FundSERVVS"},
                Condition = new FundAllocationCondition
                {
                    FundAllocationTypes = new[]
                        {
                            FundAllocationTypes.Currency,
                            FundAllocationTypes.FullHoldings
                        }
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Symbol Cross Reference

        [Example("Create: Symbol Cross Reference (Reference Data)")]
        public void SymbolCrossReference()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.SymbolCrossReference);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new SymbolCrossReferenceExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                "Quote ID", "Asset Type", "Asset Type Description",
                "Security Description", "Instrument ID", "Instrument ID Type",
                "Reuters Editorial RIC"},
                Condition = new SymbolCrossReferenceCondition
                {
                    AllXRefIdentifierTypes = true,
                    DaysAgo = 14,
                    IdentifierTypes = new[] {
                        IdentifierType.Ric,
                        IdentifierType.Sedol,
                        IdentifierType.Cusip,
                        IdentifierType.Wertpapier,
                        IdentifierType.Valoren
                    }
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Starmine

        [Example("Create: Starmine")]
        public void Starmine()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.Starmine);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new StarmineExtractionRequest
            {
                ContentFieldNames = new[] {
                    "RIC", "StarMine Calculation Date", "Credit Text Distance to Default", "Credit Text Mining Broker Research Component",
                    "Credit Text Mining Broker Research External Rank", "Credit Text Mining Broker Research Income Rank",
                    "Credit Text Mining Broker Research Legal Rank", "Credit Text Mining Broker Research Structural Rank",
                    "Credit Text Mining Country Rank", "Credit Text Mining External Category Score",
                    "Credit Text Mining Filing Income Rank", "Credit Text Mining Filing Legal Rank"},
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false)
            };

            //Create the report template
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Features

        [Example("Features: Pre-Validate without Notes")]
        public void Features_PreValidateWithoutNotes()
        {

            Status.Notify(ExtractionsContext, "InstrumentList", "ValidateIdentifiers", MethodType.Operation, Publish.Secondary);
            //Pre-Validate the instruments: If the same instruments are to be used for multiple extractions for the same or
            //other different extraction types, there is a significant performance benefit to pre-validating the instruments
            //and invoking the extraction with the pre-validated instruments
            var validatedIdentifiers = ExtractionsContext.InstrumentListOperations.ValidateIdentifiers(
                new[]
                {
                    new InstrumentIdentifier {Identifier = "191216100", IdentifierType = IdentifierType.Cusip}
                }, true);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                //Use pre-validated instruments to reduce the extraction time
                IdentifierList = InstrumentIdentifierList.Create(validatedIdentifiers.ValidatedInstruments.ToArray(), null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                    "Ask Price", "High Price", "Low Price", "Volume", "Net Asset Value", "Official Close Price",
                    "Open Price", "Security Description" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                Console.WriteLine("# Rows with errors: {0}", extractedRows.Count(er => er.Error != null));
                var udf1 = extractedRows.First();
                foreach (var field in udf1.DynamicProperties)
                    Console.WriteLine(field.Key + ": " + field.Value);
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Features: User Defined Identifier without Notes")]
        public void Features_UserDefinedIdentifiersWithoutNotes()
        {
            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip, UserDefinedIdentifier = "UDF-1"},
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol, UserDefinedIdentifier = "UDF-2" },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric, UserDefinedIdentifier = "UDF-3" }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                "CUSIP", "File Code", "Ask Price", "High Price", "Low Price", "Mid Price", "Volume", "Net Asset Value",
                "Offer Price", "Official Close Price", "Open Price", "Previous Close Price", "RIC",
                "Security Description", "SEDOL", "Ticker", "Trade Date", "User Defined Identifier" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                Console.WriteLine("# Rows with errors: {0}", extractedRows.Count(er => er.Error != null));
                var udf1 = extractedRows.First(r => r.UserDefinedIdentifier == "UDF-1");
                foreach (var field in udf1.DynamicProperties)
                    Console.WriteLine(field.Key + ": " + field.Value);
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Features: Pre-Validate with User Defined Identifiers without Notes")]
        public void Features_PreValidateWithUserDefinedIdentifiersWithoutNotes()
        {

            Status.Notify(ExtractionsContext, "InstrumentList", "ValidateIdentifiers", MethodType.Operation, Publish.Secondary);
            //Pre-Validate the instruments
            var validatedIdentifiers = ExtractionsContext.InstrumentListOperations.ValidateIdentifiers(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip, UserDefinedIdentifier = "Group1"},
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol, UserDefinedIdentifier = "Group1" },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric, UserDefinedIdentifier = "Group2" }
                }, true);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                //Only use the instruments in Group1
                IdentifierList = InstrumentIdentifierList.Create(
                    validatedIdentifiers.ValidatedInstruments.ToArray(), null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                    "Ask Price", "High Price", "Low Price", "Volume", "Net Asset Value", "Official Close Price",
                    "Open Price", "Security Description" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                Console.WriteLine("# Rows with errors: {0}", extractedRows.Count(er => er.Error != null));
                foreach (var row in extractedRows)
                    Console.WriteLine("{0} ({1}): {2} / Status = {3}",
                        row.Identifier, row.IdentifierType, row.UserDefinedIdentifier, row.DynamicProperties["Asset Status"]);
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Features: Cancel")]
        public void FeaturesCancel()
        {
            /* Asynchronous requests provides a mechanism for supporting long running requests with status updates and progress
             * messages while circumventing the need to make firewall changes.
             * Many of our long-running requests support async.  For example Schedule.ExtractNow or Extract (on demand).  If async 
             * is enabled (it is enabled by default for toolkit users, but not for HTTP direct users.  Asynchronous responses can 
             * be enabled by Prefer: respond-async in the header.  If respond-async is not included in the header, the request 
             * will be handled synchronously and may fail if the request takes more than 30 minutes due to firewall timeouts.
             * If the asynchronous request can be responded to within the poll time, the request is executed synchronously.  If
             * the request takes longer than the poll time, a 202 (accepted) is returned to the client with a monitor location
             * URL.  Once the results are available the payload will be returned on the next request to the monitor location.
             * 
             * The SDK provides a means for you to take control of the polling process which can be useful for very long running 
             * requests where you may want to cancel the extraction or retrieve the results in a different process or after a 
             * process restart.  All async methods offer a method with a Start suffix (e.g. context.ExtractWithNotesStart) which 
             * immediately returns control to the caller and returns a handle for the job.  This job handle can be converted to 
             * a string (ToString()) and recreated from this string.  This allows the handle to be persisted across client sessions.
             * 
             * This example demonstrates how to start a request, cancel the request and check on it's status.
             */
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Create the request
            var extractionRequest = new TickHistoryIntradaySummariesExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] { "Close Ask", "Close Bid", "High" },
                Condition = new TickHistoryIntradaySummariesCondition
                {
                    DaysAgo = 5,
                    MessageTimeStampIn = TickHistoryTimeOptions.LocalExchangeTime,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                    Preview = PreviewMode.None,
                    SummaryInterval = TickHistorySummaryInterval.FifteenMinutes,
                }
            };

            //Extract using the "Start" method.  Start methods return control to the caller without polling the 
            //monitor url until results are returned.  This may be useful if the job is a very long running job
            //as the case may be with Tick History content depending on the conditions/criteria.  The job can
            //also be persisted to string and re-hydrated.  See the Async help page for additional information.
            var job = ExtractionsContext.ExtractRawStart(extractionRequest);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "CancelJob", MethodType.Operation, Publish.Primary);
            Thread.Sleep(3000);
            //Request cancellation
            job = ExtractionsContext.CancelJob(job);
            //Output
            Status.WriteLine("Job Status: {0}", job.Status);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "MonitorJob", MethodType.Operation, Publish.Primary);
            //Wait a couple of seconds
            Thread.Sleep(2000);

            try
            {
                //Retrieve an updated status of the job
                job = ExtractionsContext.MonitorJob(job);

                //Output
                Status.WriteLine("Job Status: {0}", job.Status);
            }
            catch (Exception)
            {
                Status.WriteLine("Job Status: {0}", job.Status);

            }

            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Error Handling

        [Example("Error Handling: Invalid Identifier without Notes")]
        public void ErrorHandling_InvalidIdentifierWithoutNotes()
        {

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                //Use pre-validated instruments to reduce the extraction time
                IdentifierList = InstrumentIdentifierList.Create(new[]
                {
                    new InstrumentIdentifier {Identifier = "BAD_CUSIP", IdentifierType = IdentifierType.Cusip }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                    "Ask Price", "High Price", "Low Price", "Volume", "Net Asset Value", "Official Close Price",
                    "Open Price", "Security Description" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                Console.WriteLine("# Rows with errors: {0}", extractedRows.Count(er => er.Error != null));
                foreach (var row in extractedRows)
                    Console.WriteLine("{0} ({1}): Error: '{2}'", row.Identifier, row.IdentifierType, row.Error);
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region End of Day without Notes

        [Example("Create: End of Day without Notes")]
        public void EndOfDayWithoutNotes()
        {
            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Primary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price", "Currency Code",
                "CUSIP", "File Code", "Ask Price", "High Price", "Low Price", "Mid Price", "Volume", "Net Asset Value",
                "Offer Price", "Official Close Price", "Open Price", "Previous Close Price", "RIC",
                "Security Description", "SEDOL", "Ticker", "Trade Date" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Intraday without Notes

        [Example("Create: Intraday without Notes")]
        public void IntradayWithoutNotes()
        {
            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);

            //Create the request
            var extractionRequest = new IntradayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                Condition = new IntradayPricingCondition { ScalableCurrency = true },
                ContentFieldNames = new[] {
                    "RIC", "Ask Price", "Asset Type", "Bid Price", "Currency Code",
                    "Exchange Code", "High Price", "Instrument ID", "Instrument ID Type", "Low Price", "Open Price",
                    "Previous Close Date", "Previous Close Price", "Security Description", "Settlement Price",
                    "Trade Date", "User Defined Identifier", "Volume" }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Fixed Income Analytics without Notes

        [Example("Create: Fixed Income Analytics Without Notes")]
        public void FixedIncomeAnalyticsWithoutNotes()
        {
            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the request
            var extractionRequest = new FixedIncomeAnalyticsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[]
                    {
                        new InstrumentIdentifier { Identifier = "912810RJ9", IdentifierType = IdentifierType.Cusip},
                    }, null, false),
                ContentFieldNames = new[] {
                    "Instrument ID", "Econometric Corp Convexity", "Econometric Corp Modified Duration", "Econometric Corp Yield"
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Technical Indicators without Notes

        [Example("Create: Technical Indicators without Notes")]
        public void TechnicalIndicatorsWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TechnicalIndicators);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new TechnicalIndicatorsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Alpha - 3 Year Weekly", "Alpha - 3 Year Weekly - Down", "Alpha - 3 Year Weekly - Up", "Asset Category",
                    "Asset Category Description", "Asset ID", "Asset Status", "Asset Status Description", "Asset SubType",
                    "Asset SubType Description", "Asset Type", "Asset Type Description", "Average Volume - 10 Days", "Average Volume - 100 Days",
                    "Average Volume - 50 Days", "Average Volume - 60 Days", "Average Volume - 80 Days", "Average Volume - 90 Days",
                    "Average Volume - Monthly 13 Weeks", "Benchmark RIC", "Beta - 3 Year Weekly", "CFI Code", "Close Price - 1 Day",
                    "Close Price - 13 Weeks", "Close Price - 2 Days", "Close Price - 52 Week High", "Close Price - 52 Week Low"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Legal Entity Audit without Notes

        [Example("Create: Legal Entity Audit without Notes")]
        public void LegalEntityAuditWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityAudit);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new LegalEntityAuditExtractionRequest
            {
                IdentifierList = EntityIdentifierList.Create(
                    new[] { new EntityIdentifier { Identifier = "18228", IdentifierType = IdentifierType.OrgId } }),
                ContentFieldNames = new[] {
                    "Audit Modified Date", "Audit Source", "Audit Source Reference", "Audited Field Name", "ID", "ID Type",
                    "Official Name", "RCP ID", "User Defined Identifier"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Composite without Notes

        [Example("Create: Composite without Notes")]
        public void CompositeWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.Composite);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new CompositeExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Daily Beta - 180 Days", "Volatility - 30 days", "Market Capitalization", "Average Volume - 30 Days",
                    "Volume", "Total Shares - Listed"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region MBS without Notes

        [Example("Create: MBS without Notes")]
        public void CreateMbsTemplateWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.MBSFactorHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new MBSFactorHistoryExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset ID", "April Factor", "Asset SubType", "Asset SubType Description", "Asset Type", "Asset Type Description",
                    "August Factor", "CUSIP", "December Factor", "Factor Year", "February Factor", "Instrument ID",
                    "Instrument ID Type", "January Factor", "July Factor", "June Factor", "March Factor", "May Factor",
                    "November Factor", "October Factor", "Security Description", "September Factor", "Usage Instrument Type"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Tranche Factor History without Notes

        [Example("Create: Tranche Factor History without Notes")]
        public void CreateTrancheTemplateWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TrancheFactorHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new TrancheFactorHistoryExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "CUSIP", "Factor", "Factor Date", "Asset ID", "Asset SubType", "Asset SubType Description", "Asset Type",
                    "Asset Type Description", "Common Code", "ISIN",
                    "Instrument ID", "Instrument ID Type", "SEDOL", "Security Description", "Usage Instrument Type",
                    "Wertpapier", "User Defined Identifier"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Ownership without Notes

        [Example("Create: Ownership without Notes")]
        public void CreateOwnershipTemplateWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.Owners);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new OwnershipExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset ID", "Asset Status", "Asset Status Description",
                    "Change Sign", "Common Code", "Currency Code", "Currency Code Description", "CUSIP", "File Code",
                    "Issuer ID", "Issuer Name", "Issuer OrgID", "RCP ID", "Issuer Short Name", "Legal Entity OrgID",
                    "Holdings Previous Report Date", "Holdings Report Date", "Owner Country", "Owner ID", "Owner Name", "Owner Type",
                    "Owner Type Description", "Percent of Shares Outstanding", "Primary Issue Flag", "Security Description",
                    "Shares Held", "Value Held", "Shares Changed", "Value Changed"}
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows.Take(10))
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Terms and Conditions without Notes

        [Example("Create: Terms and Conditions without Notes")]
        public void TermsAndConditionsWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TermsAndConditions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create new report template with conditions
            var extractionRequest = new TermsAndConditionsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Conversion Terms End Date", "Index Principal Flag", "Conversion Terms Start Date", "RIC", "Round Lot Size",
                    "Redemption Value", "Original Issue Amount", "Issue Price", "First Coupon Date", "Issue Date", "Par Value",
                    "Accrual Date", "Base Index", "Factor", "Original Issue Discount Flag", "End Of Month Payment Flag", "ISIN",
                    "Index Linked Bond Base Index", "Coupon Frequency Description", "Total Amount Outstanding", "Maturity Date",
                    "Total Amount Issued", "Capitilization Flag", "Currency Code", "Trading Status"},
                Condition = new TermsAndConditionsCondition
                {
                    IssuerAssetClassType = IssuerAssetClassType.AllSupportedAssets,
                    ExcludeWarrants = false,
                    //DaysAgo = 3, //Use either DaysAgo or StartDate
                    StartDate = new DateTimeOffset(new DateTime(2015, 1, 1)),
                    FixedIncomeRatingSources = FixedIncomeRatingSource.StandardAndPoors,
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Time Series without Notes

        [Example("Create: Time Series without Notes")]
        public void TimeSeriesWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TimeseriesPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new TimeSeriesExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] {
                    "File Code", "RIC", "Trade Date", "Close Price", "Alternate Close Price", "Split Factor", "High Price",
                    "Low Price", "Open Price", "VWAP Price", "Turnover", "Volume", "Bid Price", "Asset Type", "Quote ID",
                    "Bid Yield", "Universal Close Price", "Exchange Code", "Currency Code"},
                Condition = new TimeSeriesCondition
                {
                    StartDate = DateTime.UtcNow.Date.AddDays(-7),
                    EndDate = DateTime.UtcNow.Date
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Bond Schedule without Notes

        [Example("Create: Bond Schedule without Notes")]
        public void BondScheduleWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.BondSchedules);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "BondScheduleReportTemplate", "GetBondScheduleTypes");
            //Get the list of bond schedule types
            var bondScheduleTypes = ExtractionsContext.ReportTemplateOperations.GetBondScheduleTypes();

            //Output the list of codes
            Status.WriteLine(String.Join(", ", bondScheduleTypes.Select(bst => bst.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);

            //Create the new report template
            var extractionRequest = new BondScheduleExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Instrument ID", "Instrument ID Type", "Asset ID", "Asset Type", "CUSIP",
                    "Common Code", "Extend Holders Option Flag", "MIC",
                    "RIC", "SEDOL", "SICC", "Schedule Type", "Security Description", "User Defined Identifier"},
                Condition = new BondScheduleCondition
                {
                    BondScheduleTypeCodes = new[] { "CALS", "PUTS" }
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Ratings without Notes

        [Example("Create: Ratings without Notes")]
        public void RatingsWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.BondRatings);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);

            //Create the new report template
            var extractionRequest = new RatingsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset ID", "Asset Status", "Asset Status Description", "Asset Status Effective Date", "Asset SubType",
                    "Asset Subtype Country Code", "Asset SubType Description", "Asset Type", "Asset Type Description",
                    "Bond Type", "Bond Type Description", "Country of Issuance", "Country of Issuance Description",
                    "CUSIP", "Domicile", "Domicile Description", "Entity ID", "FirmID", "Identifier Segment Source",
                    "Identifier Segment Source Description", "Instrument ID", "Instrument ID Type", "Instrument Type Code",
                    "Instrument Type Code Description", "ISIN", "ISO Country Code", "ISO Currency", "Local Code", "Rating Type",
                    "RIC", "Security Description", "Series", "SIC Classification Code", "SIC Description1", "SICC", "Sicovam",
                    "Ticker", "Usage Instrument SubType", "Usage Instrument Type"},
                Condition = new RatingsCondition
                {
                    DaysAgo = 7,
                    ReportDateRangeType = ReportDateRangeType.Range,
                    QueryStartDate = DateTime.Now.AddDays(-30),
                    QueryEndDate = DateTime.Now,
                    IncludeInstrumentsWithNoRatings = false,
                    RatingLevel = new RatingLevel { IsIssue = false, IsIssuer = true, IssuerOutlookScopeCodeFilter = "F" },
                    RatingSourcesCodes = new[] { "ASI", "CAR", "SSU" }
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Standard Events (Corporate Actions) without Notes

        [Example("Create: Standard Events (Corporate Actions) without Notes")]
        public void CorporateActionsStandardWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new CorporateActionsStandardExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                }, null, false),
                ContentFieldNames = new[] {
                    "Capital Change Announcement Date", "Capital Change Ex Date", "Effective Date", "Record Date",
                    "Capital Change Event Type", "Old Shares Terms", "New Shares Terms", "New Shares Currency" },
                Condition = new CorporateActionsStandardCondition
                {
                    PreviousDays = 30,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    CorporateActionsCapitalChangeType = CorporateActionsCapitalChangeType.CapitalChangeExDate,
                    CorporateActionsDividendsType = CorporateActionsDividendsType.DividendPayDate,
                    CorporateActionsEarningsType = CorporateActionsEarningsType.PeriodEndDate,
                    CorporateActionsEquityOfferingsType = CorporateActionsEquityOfferingsType.AllPendingDeals,
                    CorporateActionsMergersAcquisitionsType = CorporateActionsMergersAcquisitionsType.DealAnnouncementDate,
                    CorporateActionsNominalValueType = CorporateActionsNominalValueType.NominalValueDate,
                    CorporateActionsSharesType = CorporateActionsSharesType.SharesAmountDate,
                    CorporateActionsStandardEventsType = CorporateActionsStandardEventsType.None,
                    CorporateActionsVotingRightsType = CorporateActionsVotingRightsType.VotingRightsDate,
                    QueryStartDate = null,
                    NextDays = null,
                    QueryEndDate = null,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    IncludeInstrumentsWithNoEvents = null,
                    IncludeNullDates = null,
                    ExcludeDeletedEvents = true,
                    ExcludeHistoricalEvents = false,
                    IncludeCapitalChangeEvents = true,
                    IncludeDividendEvents = true,
                    IncludeEarningsEvents = true,
                    IncludeMergersAndAcquisitionsEvents = true,
                    IncludeNominalValueEvents = true,
                    IncludePublicEquityOfferingsEvents = true,
                    IncludeSharesOutstandingEvents = true,
                    IncludeVotingRightsEvents = true
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region IPO Events (Corporate Actions) without Notes

        [Example("Create: IPO Events (Corporate Actions) without Notes")]
        public void CorporateActionsIpoWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetExchangeTypes", MethodType.Operation, Publish.Primary);
            //Retrieve the list of available exchnage types.  You need not retrieve this list every time.  The report condition
            //accepts a list of exchange type codes which can be hard-coded.
            var exchangeTypes = ExtractionsContext.ReportTemplateOperations.GetExchangeTypes();

            //Output
            foreach (var exchangeType in exchangeTypes)
                Status.WriteLine("{0}: {1}", exchangeType.Code, exchangeType.Name);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new CorporateActionsIpoExtractionRequest
            {
                ContentFieldNames = new[] {
                    "CUSIP", "City", "Common Code", "Company Name at Offer", "Corporate Actions Type", "Country", "Currency Code",
                    "Email", "Entitlement", "Exchange Code", "Exchange Offer End Date", "Exchange Offer Start Date",
                    "Expected Pricing Date", "Instrument ID", "Instrument ID Type", "Issuer OrgID", "Market Level", "Nominal Value",
                    "Nominal Value Currency", "Number of Shares", "Offer Exchange", "Offer Exchange Description", "Offer ID",
                    "Offer Status", "Offer Sub-Status", "Offer Type", "Shares Type", "Sicovam", "State Code", "Target Group",
                    "Ticker", "Total Amount Raised", "Total Shares Offered", "Usage Instrument SubType", "Usage Instrument Type",
                    "User Defined Identifier", "Web Site"},
                Condition = new CorporateActionsIpoCondition
                {
                    PreviousDays = 30,
                    //QueryStartDate = null,
                    //NextDays = null,
                    //QueryEndDate = null,
                    IncludeInstrumentsWithNoEvents = true,
                    IncludeNullDates = null,
                    ExchangeTypes = new[] { "NYS", "LSE" },
                    ReportDateRangeType = ReportDateRangeType.Range
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region ISO 15022 (Corporate Actions) without Notes

        [Example("Create: ISO 15022 (Corporate Actions) without Notes")]
        public void CorporateActionsIsoWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new CorporateActionsIsoExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                Condition = new CorporateActionsIsoCondition
                {
                    CorporateActionsStandardYearType = CorporateActionsStandardYearType.Current,
                    ExcludeNilPaidFromPaymentEvents = true,
                    GrossAmountOnlyForPaymentEvents = true,
                    IncludeNonPendingEvents = true,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    PendingEventsPreviousDays = null,
                    PendingEventsPreviousMonths = null,
                    ReportDateRangeType = ReportDateRangeType.Init,
                    ReportIsoEvents = new[] { ReportIsoEventType.ACTV, ReportIsoEventType.BIDS }
                },
            };

            //NO FIELDS FOR CORAX ISO - the fields are pre-determined

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                var result = reader.ReadToEnd();

                //Output
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);

        }

        #endregion

        #region ISO 15022 (Debt Corporate Actions) without Notes

        [Example("Create: ISO 15022 (Debt Corporate Actions) without Notes")]
        public void DebtCorporateActionsIsoWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.DebtCorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new DebtCorporateActionsIsoExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                Condition = new DebtCorporateActionsIsoCondition
                {
                    DebtCorporateActionsStandardYearType = DebtCorporateActionsStandardYearType.Current,
                    IncludeNonPendingEvents = true,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    PendingEventsPreviousDays = null,
                    PendingEventsPreviousMonths = null,
                    ReportDateRangeType = ReportDateRangeType.Init,
                    ReportIsoEvents = new[] { ReportDebtIsoEventType.BRUP}
                },
            };

            //NO FIELDS FOR DEBT CORAX ISO - the fields are pre-determined

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                var result = reader.ReadToEnd();

                //Output
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);

        }

        #endregion

        #region Detail (Entity Data) without Notes

        [Example("Create: Detail (Entity Data) without Notes")]
        public void LegalEntityDetailWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityDetail);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new LegalEntityDetailExtractionRequest
            {
                IdentifierList = EntityIdentifierList.Create(
                    new[] { new EntityIdentifier { Identifier = "18228", IdentifierType = IdentifierType.OrgId } }),
                ContentFieldNames = new[] {
                    "CIK", "Company/Registration Authority", "Company/Registration Number", "Country of Incorporation",
                    "Country of Incorporation Description", "Date of Incorporation", "Domicile", "Domicile Description",
                    "Entity URL", "Event Date", "Event Type", "FATF Jurisdiction", "GICS Industry Code",
                    "ID", "ID Type", "Immediate Parent Official Name", "Immediate Parent RCP ID", "Immediate Parent Reuters Editorial RIC",
                    "Is Issuer", "Jurisdiction of Incorporation", "Official Name", "Primary Regulator", "Primary Regulator ID",
                    "RCP ID", "RCP Reuters Editorial RIC", "Record Active Flag", "Registered City/Town", "Registered Country",
                    "Registered Country Description", "Registered Fax", "Registered Postal/Zip Code", "Registered State/County/Region",
                    "S&P Issuer Long Rating", "S&P Issuer Long Rating - F", "S&P Issuer Long Rating Date", "S&P Issuer Long Rating Date - F",
                    "S&P Issuer Long Rating Rank", "S&P Issuer Long Rating Rank - F", "S&P Issuer Short Rating",
                    "S&P Issuer Short Rating - F", "S&P Issuer Short Rating Date", "S&P Issuer Short Rating Date - F",
                    "S&P Issuer Short Rating Rank", "S&P Issuer Short Rating Rank - F"},
                Condition = new LegalEntityDetailCondition
                {
                    DeltaDays = 100
                },
            };

            //Add fields

            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Hierarchy (Entity Data) without Notes

        [Example("Create: Hierarchy (Entity Data) without Notes")]
        public void LegalEntityHierarchyWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityHierarchy);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new LegalEntityHierarchyExtractionRequest
            {
                IdentifierList = EntityIdentifierList.Create(
                    new[] { new EntityIdentifier { Identifier = "18228", IdentifierType = IdentifierType.OrgId } }),
                ContentFieldNames = new[] {
                    "ID", "ID Type", "Immediate Parent ID", "Immediate Parent Official Name",
                    "Immediate Parent RCP ID", "Node/Level", "Official Name", "RCP ID",
                    "Source RCP ID", "Ultimate Parent ID", "Ultimate Parent Official Name", "Ultimate Parent RCP ID",
                    "User Defined Identifier"},
                Condition = new LegalEntityHierarchyCondition
                {
                    DeltaDays = 100
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Hierarchy (Entity Data) without Notes With Options")]
        public void LegalEntityHierarchyWithoutNotesWithValidationOptions()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityHierarchy);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new LegalEntityHierarchyExtractionRequest
            {
                IdentifierList =
                EntityIdentifierListWithValidationOptions.Create(
                    new[] { new EntityIdentifier { Identifier = "18228", IdentifierType = IdentifierType.OrgId } }
                    , new EntityValidationOptions { AllowDuplicateInstruments = true, AllowUnmanagedOrUnverifiedEntities = true, IncludeParentAndUltimateParent = false }
                    , false
                    ),
                ContentFieldNames = new[] {
                    "ID", "ID Type", "Immediate Parent ID", "Immediate Parent Official Name",
                    "Immediate Parent RCP ID", "Node/Level", "Official Name", "RCP ID",
                    "Source RCP ID", "Ultimate Parent ID", "Ultimate Parent Official Name", "Ultimate Parent RCP ID",
                    "User Defined Identifier"},
                Condition = new LegalEntityHierarchyCondition
                {
                    DeltaDays = 100
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }


        #endregion

        #region Summary Estimates (Estimates Data) without Notes

        [Example("Create: Summary Estimates (Estimates Data) without Notes")]
        public void EstimatesSummaryWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesSummary);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesSummaryExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                "RIC", "Security Description", "Period", "Period Date", "Measure", "Flash Mean Estimate", "Flash Number of Estimates",
                "High Estimate", "High Estimate - 3 Months ago", "High Estimate - 4 Weeks ago", "Low Estimate",
                "Low Estimate - 3 Months ago", "Low Estimate - 4 Weeks ago", "Mean Estimate", "Mean Estimate - 3 Months ago",
                "Mean Estimate - 4 Weeks ago", "Median Estimate", "Median Estimate - 3 Months ago",
                "Median Estimate - 4 Weeks ago", "Number of Estimates", "Number of Estimates - 3 Months ago",
                "Number of Estimates - 4 Weeks ago", "Number of Estimates Lowered - 1 Month",
                "Number of Estimates Lowered - 1 Week", "Number of Estimates Lowered - 4 Weeks", "Number of Estimates Raised - 1 Month",
                "Number of Estimates Raised - 1 Week", "Number of Estimates Raised - 4 Weeks", "Estimate Currency Code"},
                Condition = new EstimatesSummaryCondition
                {
                    DaysAgo = 30,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = RelativeFiscalYearType.FY1,
                    FiscalYearTo = RelativeFiscalYearType.FY3,
                    QuarterFrom = RelativeQuarterType.None,
                    QuarterTo = RelativeQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    SemiAnnualFrom = RelativeSemiAnnualType.None,
                    SemiAnnualTo = RelativeSemiAnnualType.None
                },
            };

            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Actual Estimates (Estimates Data) without Notes

        [Example("Create: Actual Estimates (Estimates Data) without Notes")]
        public void EstimatesActualWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesActual);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesActualExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Actual Currency", "Actual Date", "Actual Status", "Actual Value", "Mean Estimate", "Measure", "MIC",
                    "Number of Estimates", "OPOL", "Period", "Period Date", "Quote ID", "RCP ID", "Reuters Editorial RIC",
                    "RIC", "Security Description", "SEDOL", "SICC", "Sicovam", "Standard Deviation", "Ticker", "Trading Status",
                    "Update Date", "Update Type", "Usage Instrument SubType", "Usage Instrument Type"},
                Condition = new EstimatesActualCondition
                {
                    DaysAgo = 365,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = HistoricalFiscalYearType.FYNegative1,
                    FiscalYearTo = HistoricalFiscalYearType.FYNegative5,
                    QuarterFrom = HistoricalQuarterType.None,
                    QuarterTo = HistoricalQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Range,
                    SemiAnnualFrom = HistoricalSemiAnnualType.None,
                    SemiAnnualTo = HistoricalSemiAnnualType.None,
                    ExcludeRestatedActuals = true
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Detail Estimates (Estimates Data) without Notes

        [Example("Create: Detail Estimates (Estimates Data) without Notes")]
        public void EstimatesDetailWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesDetail);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesDetailExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "RIC", "Period", "Period Date", "Previous Broker Recommendation", "Previous Broker Recommendation Description",
                    "Previous Currency", "Previous Estimate", "Previous Horizon", "Previous IBES Entry Date",
                    "Previous IBES Recommendation", "Previous IBES Recommendation Description", "Previous Is Diluted",
                    "Previous Is Excluded", "Previous Status", "Quote ID", "RCP ID", "Reuters Editorial RIC",
                    "Ticker", "Trading Status", "Update Date", "Update Type", "Usage Instrument SubType", "Usage Instrument Type",
                    "User Defined Identifier"},
                Condition = new EstimatesDetailCondition
                {
                    DaysAgo = 90,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = RelativeFiscalYearType.FY1,
                    FiscalYearTo = RelativeFiscalYearType.FY3,
                    QuarterFrom = RelativeQuarterType.None,
                    QuarterTo = RelativeQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Range,
                    SemiAnnualFrom = RelativeSemiAnnualType.None,
                    SemiAnnualTo = RelativeSemiAnnualType.None
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Company Level Footnotes (Estimates Data) without Notes

        [Example("Create: Company Level Footnotes (Estimates Data) without Notes")]
        public void EstimatesFootnotesCompanyWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesCompanyFootnotes);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesFootnotesCompanyExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset ID", "Asset Status", "Asset Status Description", "CUSIP", "Common Code", "Company Name", "Country",
                    "Currency Code", "Currency Code Description", "Dilution Factor", "Exchange", "Exchange Code",
                    "Exchange Description", "Footnote", "Footnote Expiry Date", "Footnote Type", "IBES Code", "IBES Entry Date",
                    "ISIN", "Issuer Name", "RIC", "Security Description", "Ticker", "Trading Status", "Update Date",
                    "Update Type", "Usage Instrument SubType", "Usage Instrument Type"},
                Condition = new EstimatesFootnotesCompanyCondition
                {
                    DaysAgo = 30,
                    ReportDateRangeType = ReportDateRangeType.Delta
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Company Level Footnotes (Estimates Data) without Notes

        [Example("Create: Detail Level Footnotes (Estimates Data) without Notes")]
        public void EstimatesFootnotesDetailWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.DetailEstimateFootnotes);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new EstimatesFootnotesDetailExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                "Analyst Name", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type", "Asset Type Description",
                "Broker Name", "Dilution Factor", "Estimate Flag", "Exchange", "Exchange Code", "Exchange Description",
                "File Code", "Footnote", "Footnote Expiry Date", "Footnote Type", "IBES Code", "IBES Estimate Entry Date",
                "IBES Footnote Entry Date", "Issuer Name", "Issuer OrgID", "Market MIC", "Measure", "Period", "Period Date",
                "Quote ID", "RIC", "Trading Status", "Update Date", "Update Type", "Usage Instrument SubType",
                "Usage Instrument Type"},
                Condition = new EstimatesFootnotesDetailCondition
                {
                    DaysAgo = 30,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = RelativeFiscalYearType.FY1,
                    FiscalYearTo = RelativeFiscalYearType.FY3,
                    QuarterFrom = RelativeQuarterType.None,
                    QuarterTo = RelativeQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    SemiAnnualFrom = RelativeSemiAnnualType.None,
                    SemiAnnualTo = RelativeSemiAnnualType.None
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region News Items (News) without Notes

        [Example("Create: News Items without Notes")]
        public void NewsItemsWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.NewsItems);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "NewsItemsReportTemplate", "GetNewsAttributionTypes");
            //Get the list of attributions which can be applied to Condition.NewsAttributionsCodes
            var newsAttributions = ExtractionsContext.ReportTemplateOperations.GetNewsAttributionTypes();
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "NewsItemsReportTemplate", "GetNewsTopicCodes", MethodType.Operation, Publish.Primary);
            //Get the list of topics which can be applied to Condition.NewsTopicsCodes
            var newsTopics = ExtractionsContext.ReportTemplateOperations.GetNewsTopicTypes();
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "NewsItemsReportTemplate", "GetNewsItemsLanguageTypes");
            //Get the list of languages which can be applied to Condition.NewsItemsLanguageCodes
            var newsLanguages = ExtractionsContext.ReportTemplateOperations.GetNewsItemsLanguageTypes();
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new NewsItemsExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                "Asset ID", "Attribution", "Common Code", "CUSIP", "File Code", "Headline", "Instrument ID", "Instrument ID Type",
                "ISIN", "Language", "OPOL", "PILC", "PNAC", "Products", "Quote ID", "Related RICs", "Reuters Editorial RIC",
                "RIC", "SEDOL", "SICC", "Story Body", "Story Date Time", "Take Date Time", "Ticker", "Topics",
                "Unique Story Index"},
                Condition = new NewsItemsCondition
                {
                    QueryStartDate = null,
                    QueryEndDate = null,
                    DaysAgo = 1,
                    HoursAgo = 20,
                    MinutesAgo = 40,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    NewsAttributionsCodes = new[] { "RTRS", "JSE" }, //from newsAttributionCodes
                    NewsTopicsCodes = new[] { "GDP", "ECI", "MCE" }, //from newsTopics
                    NewsItemsSource = NewsItemsSourceType.AllNews,
                    NewsItemsLanguage = NewsItemsLanguageType.Selected,
                    NewsLanguagesCodes = new[] { "EN" } //from newsLanguages
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Premimum Of Day without Notes

        [Example("Create: Premium End Of Day without Notes")]
        public void PremiumEndOfDayPricingWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumEndOfDayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new PremiumEndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Instrument ID", "Instrument ID Type", "Trading Indicator", "Trading Indicator Description",
                    "Universal Close Validation Code", "Universal Close Validation Code Description", "Universal Bid Price",
                    "Universal Ask Price", "Universal Bid Ask Date", "Universal Close Price", "Universal Close Price Date",
                    "RIC", "ISIN", "Asset Status", "Asset Status Description", "Asset SubType", "Asset SubType Description",
                    "Asset Type", "Asset Type Description", "Currency Code", "Currency Code Description",
                    "Current Embargo Delay", "Embargo Times", "Embargo Window", "Exchange Requiring Embargo",
                    "File Code", "Instrument Snap Time", "Last Intraday Update Time", "Maximum Embargo Delay",
                    "Real Time Permitted", "PE Code", "Security Description", "Trading Status"},
                Condition = new PremiumEndOfDayPricingCondition
                {
                    IncludeIntradayPricing = true,
                    IntradayFixedIncomePremiumEndOfDay = true,
                    ScalableCurrency = false
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Premium Pricing without Notes

        [Example("Create: Premium Pricing without Notes")]
        public void CreatePremiumPricingReportTemplateWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new PremiumPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "912810RJ9", IdentifierType = IdentifierType.Cusip } }, null, false),
                ContentFieldNames = new[]
                {
                   "Accrued Interest", "Ask Price", "Asset Status", "Asset Type", "Bid Ask Spread", "Bid Price", "Bid Yield",
                   "Date Moving Convention Description", "Highest Quote", "Issuer Movement", "Issuer Name", "LIBOR Rate",
                   "Lowest Quote", "Maturity Date", "Payment Frequency", "Previous Close Price", "Pricing Date",
                   "Pricing Method Code", "Pricing Method Value", "Pricing Model Code", "Pricing Region Code", "Pricing Time",
                   "Settlement Date"
                },
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingAsia3PM,
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Single Historical Price without Notes

        [Example("Create: Single Historical Price (Pricing Data) without Notes")]
        public void SingleHistoricalPriceWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TimeseriesPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new SingleHistoricalPriceExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Close Price", "Bid Price", "Ask Price", "Trade Date", "CUSIP", "RIC", "Security Description",
                    "Alternate Close Price", "Low Price", "High Price", "Mid Price", "Open Price", "Instrument ID"},
                Condition = new SingleHistoricalPriceCondition
                {
                    LookBackPeriod = LookbackType.OneMonth,
                    PriceDate = DateTime.UtcNow
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Fund Allocation without Notes

        [Example("Create: Fund Allocation (Reference Data) without Notes")]
        public void FundAllocationWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.FundAllocation);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new FundAllocationExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                    "Asset SubType Description", "Asset Type", "Asset Type Description", "CUSIP",
                    "Identifier Segment Source",
                    "Identifier Segment Source Description", "Instrument ID", "Instrument ID Type", "Security Description",
                    "Allocation Asset Type", "Allocation CUSIP", "Allocation Date", "Allocation Item", "Allocation OrgID",
                    "Allocation Percentage", "Allocation Rank", "Market Value Currency", "Market Value Held", "RIC",
                    "FundSERVAC", "FundSERVBE", "FundSERVDM", "FundSERVDO", "FundSERVFE", "FundSERVIS", "FundSERVLL",
                    "FundSERVNL", "FundSERVVS"},
                Condition = new FundAllocationCondition
                {
                    FundAllocationTypes = new[]
                        {
                            FundAllocationTypes.Currency,
                            FundAllocationTypes.FullHoldings
                        }
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Symbol Cross Reference without Notes

        //[Example("Create: Symbol Cross Reference (Reference Data) without Notes")] 
        public void SymbolCrossReferenceWithoutNotes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.SymbolCrossReference);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "Extract", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var extractionRequest = new SymbolCrossReferenceExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                    new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }, null, false),
                ContentFieldNames = new[] {
                "Quote ID", "Identifier Description", "Asset Type", "Asset Type Description", "Identifier",
                "Identifier Description", "Identifier Type", "Security Description", "Instrument ID", "Instrument ID Type",
                "Reuters Editorial RIC"},
                Condition = new SymbolCrossReferenceCondition
                {
                    AllXRefIdentifierTypes = true,
                    DaysAgo = 14,
                    IdentifierTypes = new[] {
                        IdentifierType.Ric,
                        IdentifierType.Sedol,
                        IdentifierType.Cusip,
                        IdentifierType.Wertpapier,
                        IdentifierType.Valoren
                    }
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractedRows = ExtractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Console.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region TickHistoryIntradaySummaries

        [Example("Create: TickHistoryIntradaySummaries")]
        public void TickHistoryIntradaySummaries()
        {
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            //Create the request
            var extractionRequest = new TickHistoryIntradaySummariesExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new TickHistoryIntradaySummariesCondition()
                {
                    DaysAgo = null,
                    MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                    QueryEndDate = DateTime.UtcNow,
                    QueryStartDate = DateTime.UtcNow - TimeSpan.FromDays(3),
                    ReportDateRangeType = ReportDateRangeType.Range,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                    DisplaySourceRIC = false,
                    TimebarPersistence = true
                },
                ContentFieldNames = new[]
                    {
                        "Close Ask", "Close Bid", "High", "Low", "Open Ask", "Open Bid"
                    }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            Status.EndNotify(ExtractionsContext);
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            StringBuilder responseBody = new StringBuilder();
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                //Output
                var result = reader.ReadLine();
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                    for (var lineCount = 0; lineCount < 100 && !reader.EndOfStream; lineCount++) //Limit output results to 100 reads.
                    {
                        result = reader.ReadLine();
                        responseBody.AppendLine(result);
                        Status.WriteLine(result);
                    }
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext, responseBody.ToString());
        }

        #endregion

        #region TickHistoryMarketDepth

        [Example("Create: TickHistoryMarketDepth")]
        public void TickHistoryMarketDepth()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryMarketDepth);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            //Create the request
            var extractionRequest = new TickHistoryMarketDepthExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new TickHistoryMarketDepthCondition()
                {
                    DaysAgo = null,
                    MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                    QueryEndDate = DateTime.UtcNow,
                    QueryStartDate = DateTime.UtcNow - TimeSpan.FromDays(3),
                    ReportDateRangeType = ReportDateRangeType.Range,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                    NumberOfLevels = 2,
                    View = TickHistoryMarketDepthViewOptions.NormalizedLL2,
                    DisplaySourceRIC = false
                },
                ContentFieldNames = new[]
                {
                    "Ask Price", "Bid Price", "Number of Buyers", "Number of Sellers"
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            Status.EndNotify(ExtractionsContext);
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            StringBuilder responseBody = new StringBuilder();
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                //Output
                var result = reader.ReadLine();
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                    for (var lineCount = 0; lineCount < 100 && !reader.EndOfStream; lineCount++) //Limit output results to 100 reads.
                    {
                        result = reader.ReadLine();
                        responseBody.AppendLine(result);
                        Status.WriteLine(result);
                    }
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext, responseBody.ToString());
        }

        #endregion

        #region TickHistoryRaw

        [Example("Create: TickHistoryRaw")]
        public void TickHistoryRaw()
        {
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            //Create the request
            var extractionRequest = new TickHistoryRawExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new TickHistoryRawCondition()
                {
                    DaysAgo = null,
                    MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                    QueryEndDate = DateTime.UtcNow,
                    QueryStartDate = DateTime.UtcNow - TimeSpan.FromDays(10),
                    ReportDateRangeType = ReportDateRangeType.Range,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                    DomainCode = TickHistoryRawDomain.MarketPrice,
                    DisplaySourceRIC = false,
                    Fids = "25"
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            Status.EndNotify(ExtractionsContext);
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            StringBuilder responseBody = new StringBuilder();
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                //Output
                var result = reader.ReadLine();
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                    for (var lineCount = 0; lineCount < 100 && !reader.EndOfStream; lineCount++) //Limit output results to 100 reads.
                    {
                        result = reader.ReadLine();
                        responseBody.AppendLine(result);
                        Status.WriteLine(result);
                    }
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext, responseBody.ToString());
        }

        #endregion

        #region TickHistoryTimeAndSales

        [Example("Create: TickHistoryTimeAndSales")]
        public void TickHistoryTimeAndSales()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            //Create the request
            var extractionRequest = new TickHistoryTimeAndSalesExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new TickHistoryTimeAndSalesCondition()
                {
                    DaysAgo = null,
                    MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                    QueryEndDate = DateTime.UtcNow,
                    QueryStartDate = DateTime.UtcNow - TimeSpan.FromDays(3),
                    ReportDateRangeType = ReportDateRangeType.Range,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                    ApplyCorrectionsAndCancellations = false,
                    DisplaySourceRIC = false
                },
                ContentFieldNames = new[]
                {
                    "Quote - Bid Size"
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            Status.EndNotify(ExtractionsContext);
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            StringBuilder responseBody = new StringBuilder();
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                //Output
                var result = reader.ReadLine();
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                    for (var lineCount = 0; lineCount < 100 && !reader.EndOfStream; lineCount++) //Limit output results to 100 reads.
                    {
                        result = reader.ReadLine();
                        responseBody.AppendLine(result);
                        Status.WriteLine(result);
                    }
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext, responseBody.ToString());
        }

        #endregion

        #region Elektron Timeseries

        [Example("Create: Elektron Timeseries")]
        public void ElektronTimeseries()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.ElektronTimeseries);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);

            //Create the request
            var extractionRequest = new ElektronTimeseriesExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "TRI.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new ElektronTimeseriesCondition
                {
                    ReportDateRangeType = ReportDateRangeType.Range,
                    QueryStartDate = new DateTimeOffset(new DateTime(2015, 12, 1)),
                    QueryEndDate = new DateTimeOffset(new DateTime(2015, 12, 2)),
                    LastEntityOnly = false
                },
                ContentFieldNames = new[]
                    {
                        "Ask", "Bid", "High", "Last", "Low", "Trade Date"
                    },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region TickHistoryRaw

        [Example("Create: Per-Instrument Time Ranges (Tick History Only)")]
        public void TickHistoryRawWithPerInstrumentTimeRanges()
        {
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            // Note that the time intervals are expected to be small
            var timeRange1Start = DateTimeOffset.UtcNow.AddDays(-2);
            var timeRange1End = DateTimeOffset.UtcNow.AddDays(-2).AddSeconds(15);

            var timeRange2Start = DateTimeOffset.UtcNow.AddDays(-2).AddHours(-8);
            var timeRange2End = DateTimeOffset.UtcNow.AddDays(-2).AddHours(-8).AddSeconds(15);


            //Create the request
            var extractionRequest = new TickHistoryRawExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[]
                    {
                        // Notice the time ranges specified with the instruments
                        new HistoricalInstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric, RangeStart = timeRange1Start, RangeEnd = timeRange1End },
                        new HistoricalInstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric, RangeStart = timeRange2Start, RangeEnd = timeRange2End }
                    }
                },
                Condition = new TickHistoryRawCondition()
                {
                    DaysAgo = null,
                    MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                    QueryEndDate = null,  // Note that since the time ranges are specified in the list, there is no range specified in the condition.
                    QueryStartDate = null,
                    ReportDateRangeType = ReportDateRangeType.PerIdentifier,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                    DomainCode = TickHistoryRawDomain.MarketPrice,
                    DisplaySourceRIC = false,
                    Fids = "25"
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            Status.EndNotify(ExtractionsContext);
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            StringBuilder responseBody = new StringBuilder();
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                //Output
                var result = reader.ReadLine();
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                    for (var lineCount = 0; lineCount < 100 && !reader.EndOfStream; lineCount++) //Limit output results to 100 reads.
                    {
                        result = reader.ReadLine();
                        responseBody.AppendLine(result);
                        Status.WriteLine(result);
                    }
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext, responseBody.ToString());
        }

        #endregion

        #region Elektron End Of Day Pricing

        [Example("Create: Elektron End Of Day Pricing")]
        public void ElektronEndOfDayPricing()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.ElektronEndOfDayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);

            //Create the request
            var extractionRequest = new ElektronEndOfDayPricingExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "TRI.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new ElektronEndOfDayPricingCondition()
                {
                    AdjustedPrices = true
                },
                ContentFieldNames = new[]
                    {
                        "Ask High", "Ask Low", "Bid High", "Bid Low", "Trade Date"
                    }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

    }
}
