﻿using System.Globalization;
using System.Text;
using Dss.Api.Examples.Internal;
using System;
using System.Linq;
using DataScope.Select.Api;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.ReportTemplates;

namespace Dss.Api.Examples.Extractions.ReportTemplates
{

    [Example("Report Template Examples")]
    public class ReportTemplateExamples
    {
        protected ExtractionsContext ExtractionsContext { get; set; }

        public ReportTemplateExamples()
        {
            ExtractionsContext = ContextHelper.CreateExtractionsContext();
        }

        #region Get

        /// <summary>
        /// This example demonstrates how to retrieve all report templates for the logged in user.
        /// </summary>
        [Example("Get All (Paged Results)")]
        public void GetAllReportTemplates()
        {
            //Set the page size to 1 record per page for the purposes of this example
            ExtractionsContext.Preferences.MaxPageSize = 1;
            Status.Notify(ExtractionsContext, "ReportTemplate", "GetAll", MethodType.Operation, Publish.Primary);
            //Fetch the first page (default page size is 250)
            var page1Results = ExtractionsContext.ReportTemplateOperations.GetAll();

            //Output
            foreach (var template in page1Results)
                Status.WriteLine(template.Name);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "GetAll", MethodType.Operation, Publish.Primary);
            if (page1Results.HasNextPage) //Use a while loop if you need to retrieve all instrument lists
            {
                //Fetch the second page of report templates (default page size is 250)
                var allReportTemplates = ExtractionsContext.ReportTemplateOperations.GetAll(page1Results.NextLink);

                //Output
                foreach (var template in allReportTemplates)
                    Status.WriteLine(template.Name);
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Get: By Id")]
        public void GetReportTemplateById()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            //Create a report template so that we can retrieve it
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New() //NOTE: This is the only required field, but there are several other options
            };
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Get", MethodType.Operation, Publish.Primary);
            //Get the report template by it's id.  See also GetByName which is better suited for API retrieval of individual
            //report templates.
            var retrievedTemplate = ExtractionsContext.ReportTemplateOperations.Get(reportTemplate.ReportTemplateId);

            //Output
            Status.WriteLine(retrievedTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(retrievedTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Get: By Name")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void GetReportTemplateByName()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            //Create a report template so that we can retrieve it
            var name = "MyReportTemplate" + Suffix.New();
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = name //NOTE: This is the only required field, but there are several other options
            };
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "GetByName", MethodType.Operation, Publish.Primary);
            //Retrieve report template by name (e.g. "MyReportTemplate")
            var retrievedTemplate = ExtractionsContext.ReportTemplateOperations.GetByName(name);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(retrievedTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Get: Content Fields")]
        public void GetContentFields()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            //Create a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
            };

            //Add fields.  See also GetValidFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "GetContentFields", MethodType.Operation, Publish.Primary);
            //Retrieve the content fields for a report template given the report template id.
            //NOTE that the ReportTemplateId was set during the Create operation
            var actualContentFields = ExtractionsContext.ReportTemplateOperations.GetContentFields(reportTemplate.ReportTemplateId);

            //Output
            Status.WriteLine(actualContentFields.Count());
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Lookups: Get All Report Template Types")]
        public void GetAvailableReportTemplateTypes()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetAll", MethodType.Operation, Publish.Primary);
            //Get the list of report template types.
            var types = ExtractionsContext.ReportTemplateTypeOperations.GetAll();

            //Output
            Status.WriteLine(String.Join(", ", types.Select(type => type.Code)));
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Lookups: Get Available Fields")]
        public void GetFieldsAvailableForReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Primary);
            //Return the list of fields available for end of day extractions.
            var fields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EndOfDayPricing);

            //Output.  FormatType can be Text, Number or Date.  FormatType determines the custom formatting
            //options available for the field.
            foreach (var field in fields)
                Status.WriteLine("{0} ({1}): {2}", field.Code, field.FormatType, field.Description);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Create Update and Delete

        [Example("Create")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void CreateReportTemplateWithFields()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Custom Formatted Fields")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void CreateReportTemplateWithFormattedFields()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
            };

            //Add text formatted field
            reportTemplate.ContentFields.Add(
                new ContentField
                {
                    FieldName = "RIC",
                    Justification = ContentFieldJustification.Center,
                    Label = "RIC",
                    Width = 24,
                    WidthStyle = ContentFieldWidthStyle.FixedWidth,
                    Format = ContentFieldFormat.CreateFieldTextFormat(ContentFieldCapitalizationType.LowerCase)
                });

            //Add numeric formatted field
            reportTemplate.ContentFields.Add(
                new ContentField
                {
                    FieldName = "Universal Close Price",
                    Justification = ContentFieldJustification.Left,
                    Label = "Close Price",
                    Width = 24,
                    WidthStyle = ContentFieldWidthStyle.FixedWidth,
                    Format = ContentFieldFormat.CreateFieldNumberFormat(
                        2 /* Decimal Places */, ContentFieldDecimalSeparator.Period, 10 /* Integer places */,
                        true /* Use leading zeros */, ContentFieldNegativeSignPosition.Parentheses,
                        ContentFieldThousandsSeparator.Comma, true /* Use thousands separator */, false /* Use trailing zeros */)
                });

            //Add date formatted field
            reportTemplate.ContentFields.Add(
                new ContentField
                {
                    FieldName = "Universal Close Price Date",
                    Justification = ContentFieldJustification.Left,
                    Label = "Close Price",
                    Width = 24,
                    WidthStyle = ContentFieldWidthStyle.FixedWidth,
                    Format = ContentFieldFormat.CreateFieldDateFormat("MM/dd/yyyy")
                });

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: User Defined Identifier Fields")]
        public void CreateReportTemplateWithUserDefinedIdentifierFields()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
            };

            //Add fields.  User Defined Identifier's can be defined when adding fields to entity or instrument lists.  At the 
            //time of extraction, these fields can be output along with the other data.  User Defined Identifiers provide a 
            //mechanism whereby you can more easily correlate the requested instrument with the returned extraction row.
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date", "User Defined Identifier", "User Defined Identifier2", "User Defined Identifier3");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Invalid Field (Error)")]
        public void CreateReportTemplateWithInvalidField()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Conditions")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void CreateReportTemplateWithConditions()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                CompressionType = ReportCompressionType.Zip,
                Condition = EndOfDayPricingReportCondition.Create(true) //Scalable currency
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Compressed XML format")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void CreateReportTemplateWithCompressedXml()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create", MethodType.Operation, Publish.Secondary);
            //Create a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.Xml,
                CompressionType = ReportCompressionType.Zip,
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Headers and Trailers")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void CreateWithHeadersAndTrailers()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create", MethodType.Operation, Publish.Secondary);
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
            };

            //Add fields.  User Defined Identifier's can be defined when adding fields to entity or instrument lists.  At the 
            //time of extraction, these fields can be output along with the other data.  User Defined Identifiers provide a 
            //mechanism whereby you can more easily correlate the requested instrument with the returned extraction row.
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Add Headers
            reportTemplate.Headers.Add(ReportHeaderField.Create(
                1 /* Column */,
                ReportHeaderFieldType.StartTime,
                ContentFieldJustification.Left,
                1 /* Line Number */,
                null /* User Data */,
                20 /* Width */,
                ContentFieldWidthStyle.FixedWidth,
                "Start Time" /* Label */,
                true /* Show thousands separator */));

            reportTemplate.Headers.Add(ReportHeaderField.Create(
                2 /* Column */,
                ReportHeaderFieldType.User,
                ContentFieldJustification.Left,
                1 /* Line Number */,
                "Any value I want" /* User Data */,
                20 /* Width */,
                ContentFieldWidthStyle.FixedWidth,
                "Start Time" /* Label */,
                true /* Show thousands separator */));

            //Add Trailers
            reportTemplate.Trailers.Add(ReportHeaderField.Create(
                1 /* Column */,
                ReportHeaderFieldType.RowCount,
                ContentFieldJustification.Right,
                1 /* Line Number */,
                null /* User Data */,
                0 /* Width */,
                ContentFieldWidthStyle.VariableWidth,
                "Row Count" /* Label */,
                true /* Show thousands separator */));

            reportTemplate.Trailers.Add(ReportHeaderField.Create(
                2 /* Column */,
                ReportHeaderFieldType.Timezone,
                ContentFieldJustification.Center,
                1 /* Line Number */,
                null /* User Data */,
                0 /* Width */,
                ContentFieldWidthStyle.VariableWidth,
                "Time zone" /* Label */,
                true /* Show thousands separator */));

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Update")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void UpdateReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            //Create the report template which will be updated below
            var reportTemplateName = "MyReportTemplate" + Suffix.New();
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = reportTemplateName
            };
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "GetByName");
            //Retrieve the report template to be updated.  NOTE: The fact that the report template is being refetched
            //is only to demonstrate that this is the more typical approach.  In reality, since we already have an instance
            //of the report template, we could simply update it without fetching it.
            var updatedTemplate = ExtractionsContext.ReportTemplateOperations.GetByName(reportTemplateName);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Update", MethodType.Operation, Publish.Primary);
            //Update the report template output format to XML (default is CSV)
            reportTemplate.OutputFormat = ReportOutputFormat.Xml;
            reportTemplate.CompressionType = ReportCompressionType.Zip;
            reportTemplate.ShowColumnHeaders = true;
            ExtractionsContext.ReportTemplateOperations.Update(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(updatedTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to delete a report template by ReportTemplateId.
        /// </summary>
        [Example("Delete")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void DeleteReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            //Create the report template so that we can demonstrate delete.
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
            };
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete", MethodType.Operation, Publish.Primary);
            //Delete the report template.  The delete method accepts reportTemplate or ReportTemplate.ReportTemplateId
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Import and Export

        [Example("Export")]
        public void Export()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */

            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            //Create a report template so that we can retrieve it
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(), //NOTE: This is the only required field, but there are several other options
                ContentFields = new[] { "Ask High", "Ask Low", "Ask Price" }
                    .Select(x => new ContentField { FieldName = x }).ToArray()
            };
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Export", MethodType.Operation, Publish.Primary);
            var exportBytes = ExtractionsContext.ReportTemplateOperations.Export(reportTemplate, true /* include header */);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Import")]
        public void Import()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */

            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            //Create a report template so that we can retrieve it
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(), //NOTE: This is the only required field, but there are several other options
                ContentFields = new[] { "Ask High", "Ask Low", "Ask Price" }
                    .Select(x => new ContentField { FieldName = x }).ToArray()
            };
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Export", MethodType.Operation, Publish.Secondary);
            var exportBytes = ExtractionsContext.ReportTemplateOperations.Export(
                reportTemplate, true /* include header */);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Import", MethodType.Operation, Publish.Primary);
            var importBytes = Encoding.UTF8.GetBytes(exportString);
            var importResult = ExtractionsContext.ReportTemplateOperations.Import(
                importBytes, new ReportTemplateImportSettings { Replace = true });

            //Output
            foreach (var error in importResult.ErrorMessages)
                Status.WriteLine("Error: " + error);
            foreach (var warning in importResult.WarningMessages)
                Status.WriteLine("Warning: " + warning);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "GetByName");
            //When a report template is imported, we don't know what the id of the report template is and therefore
            //we need to retrieve the report template by name.
            var retrievedTemplate = ExtractionsContext.ReportTemplateOperations.GetByName(reportTemplate.Name);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(retrievedTemplate);
            Status.EndNotify(ExtractionsContext);

        }

        #endregion

        #region Fixed Income Analytics

        [Example("Create: Fixed Income Analytics")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(FixedIncomeAnalyticsReportTemplate))]
        public void CreateFixedIncomeAnalyticsReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.FixedIncomeAnalytics);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "FixedIncomeAnalyticsReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new FixedIncomeAnalyticsReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Instrument ID", "Instrument ID Type", "Asset Type", "Asset SubType", "Accrued Interest Rounding",
                "Bond Structure", "Calendar Adjustment Code", "Calendar Code", "Look Back Months", "Instrument Type",
                "Rate Structure", "Rate Type", "Bond Type", "Coupon Rate", "Coupon Rate Effective Date",
                "Currency Code", "CUSIP", "ISIN", "Maturity Date", "RIC", "Security Description", "SEDOL", "Ticker",
                "Usage Instrument SubType", "Usage Instrument Type", "User Defined Identifier", "Valoren", "Wertpapier",
                "Exception Code", "Exception Code Description");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Historical Reference

        [Example("Create: Historical Reference")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(HistoricalReferenceReportTemplate))]
        public void CreateHistoricalReferenceReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.HistoricalReference);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "HistoricalReferenceReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new HistoricalReferenceReportTemplate()
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new HistoricalReferenceCondition()
                {
                    ReportDateRangeType = ReportDateRangeType.Range,
                    QueryStartDate = new DateTimeOffset(new DateTime(2015, 12, 1)),
                    QueryEndDate = new DateTimeOffset(new DateTime(2015, 12, 1))
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                    "Asset Category",
                    "Australia Code",
                    "CESR Average Daily Turnover",
                    "CESR Average Daily Turnover Currency Code",
                    "CESR Average Value of Orders Executed",
                    "CESR Average Value of Orders Executed Currency Code",
                    "CESR EEA Regulated",
                    "CESR Free Float",
                    "CESR Free Float Currency Code",
                    "CESR Most Relevant Market",
                    "CESR Standard Market Size",
                    "CESR Standard Market Size Currency Code",
                    "CFI Code");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Technical Indicators

        [Example("Create: Technical Indicators")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(TechnicalIndicatorsReportTemplate))]
        public void CreateTechnicalIndicatorsReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TechnicalIndicators);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TechnicalIndicatorsReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new TechnicalIndicatorsReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Alpha - 3 Year Weekly", "Alpha - 3 Year Weekly - Down", "Alpha - 3 Year Weekly - Up", "Asset Category",
                "Asset Category Description", "Asset ID", "Asset Status", "Asset Status Description", "Asset SubType",
                "Asset SubType Description", "Asset Type", "Asset Type Description", "Average Volume - 10 Days", "Average Volume - 100 Days",
                "Average Volume - 50 Days", "Average Volume - 60 Days", "Average Volume - 80 Days", "Average Volume - 90 Days",
                "Average Volume - Monthly 13 Weeks", "Benchmark RIC", "Beta - 3 Year Weekly", "CFI Code", "Close Price - 1 Day",
                "Close Price - 13 Weeks", "Close Price - 2 Days", "Close Price - 52 Week High", "Close Price - 52 Week Low");
            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Legal Entity Audit

        [Example("Create: Legal Entity Audit")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(LegalEntityAuditReportTemplate))]
        public void CreateLegalEntityAuditReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityAudit);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "LegalEntityAuditReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new LegalEntityAuditReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Audit Modified Date", "Audit Source", "Audit Source Reference", "Audited Field Name", "ID", "ID Type",
                "Official Name", "RCP ID", "User Defined Identifier");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Composite

        [Example("Create: Composite")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CompositeReportTemplate))]
        public void CreateCompositeReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.Composite);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "CompositeReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new CompositeReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new CompositeCondition { ScalableCurrency = false }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Daily Beta - 180 Days", "Volatility - 30 days", "Market Capitalization", "Average Volume - 30 Days",
                "Volume", "Total Shares - Listed");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region MBS

        [Example("Create: MBS")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(MBSFactorHistoryReportTemplate))]
        public void CreateMbsTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.MBSFactorHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "MBSFactorHistoryReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new MBSFactorHistoryReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Asset ID", "April Factor", "Asset SubType", "Asset SubType Description", "Asset Type", "Asset Type Description",
                "August Factor", "CUSIP", "December Factor", "Factor Year", "February Factor", "Instrument ID",
                "Instrument ID Type", "January Factor", "July Factor", "June Factor", "March Factor", "May Factor",
                "November Factor", "October Factor", "Security Description", "September Factor", "Usage Instrument Type");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Tranche Factor History

        [Example("Create: Tranche Factor History")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(TrancheFactorHistoryReportTemplate))]
        public void CreateTrancheTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TrancheFactorHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TrancheFactorHistoryReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new TrancheFactorHistoryReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "CUSIP", "Factor", "Factor Date", "Asset ID", "Asset SubType", "Asset SubType Description", "Asset Type",
                "Asset Type Description", "CUSIP", "Common Code", "Factor", "Factor Date", "ISIN",
                "Instrument ID", "Instrument ID Type", "SEDOL", "Security Description", "Usage Instrument Type",
                "Wertpapier", "User Defined Identifier", "Factor Date", "Factor");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Ownership

        [Example("Create: Ownership")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(OwnershipReportTemplate))]
        public void CreateOwnershipTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.Owners);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "OwnershipReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new OwnershipReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Asset ID", "Asset Status", "Asset Status Description",
                "Change Sign", "Common Code", "Currency Code", "Currency Code Description", "CUSIP", "File Code",
                "Issuer ID", "Issuer Name", "Issuer OrgID", "RCP ID", "Issuer Short Name", "Legal Entity OrgID",
                "Holdings Previous Report Date", "Holdings Report Date", "Owner Country", "Owner ID", "Owner Name", "Owner Type",
                "Owner Type Description", "Percent of Shares Outstanding", "Primary Issue Flag", "Security Description",
                "Shares Held", "Value Held", "Shares Changed", "Value Changed");
            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region End Of Day

        [Example("Create: End Of Day")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EndOfDayPricingReportTemplate))]
        public void CreateEndOfDayReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EndOfDayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new EndOfDayPricingReportCondition { ScalableCurrency = false }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Ask Price", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type",
                "Asset Type Description", "Bid Price", "Block Trades", "CIN Code", "Common Code",
                "Currency Code", "Currency Code Description", "Currency Code Scaled", "Currency Code Scaled Description",
                "CUSIP", "Exchange Code", "Exchange Description", "Exercise Style", "Expiration Date", "File Code",
                "High Price", "Instrument ID", "Instrument ID Type", "ISIN", "Issuer OrgID", "Lot Size", "Low Price",
                "Market MIC", "MIC", "Mid Price", "Net Asset Value", "Offer Price",
                "Official Close Price", "Open Price", "Previous Close Price", "Put Call Indicator", "Quote ID",
                "RCP ID", "RIC", "Security Description", "SEDOL", "Settlement Date", "Settlement Price", "SICC",
                "Sicovam", "Strike Price", "Ticker", "Trade Date", "Trading Status", "Trading Symbol", "Turnover",
                "Underlying RIC", "Universal Ask Price", "Universal Bid Ask Date", "Universal Bid Price",
                "Usage Instrument SubType", "Usage Instrument Type", "Valoren", "Volume", "VWAP Price", "Wertpapier");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Intraday

        [Example("Create: Intraday")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(IntradayPricingReportTemplate))]
        public void CreateIntradayReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.IntradayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "IntradayPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new IntradayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new IntradayPricingCondition { ScalableCurrency = true }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "Trade Date", "Bid Price", "Ask Price", "Ask Price", "Asset Type", "Bid Price", "Currency Code",
                "Exchange Code", "High Price", "Instrument ID", "Instrument ID Type", "Low Price", "Open Price", "Previous Close Date",
                "Previous Close Price", "RIC", "Security Description", "Settlement Price", "Trade Date", "User Defined Identifier", "Volume");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Elektron Timeseries

        [Example("Create: Elektron Timeseries")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ElektronTimeseriesReportTemplate))]
        public void CreateElektronTimeseriesReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.ElektronTimeseries);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ElektronTimeseriesReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new ElektronTimeseriesReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new ElektronTimeseriesCondition
                {
                    ReportDateRangeType = ReportDateRangeType.Range,
                    QueryStartDate = new DateTimeOffset(new DateTime(2015, 12, 1)),
                    QueryEndDate = new DateTimeOffset(new DateTime(2015, 12, 1)),
                    LastEntityOnly = false
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange("Ask", "Bid", "High", "Last", "Low", "Trade Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Terms and Conditions

        [Example("Create: Terms and Conditions")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(TermsAndConditionsReportTemplate))]
        public void CreateTermsAndConditionsReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TermsAndConditions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TermsAndConditionsReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create new report template with conditions
            var reportTemplate = new TermsAndConditionsReportTemplate
            {
                Name = "My Report Template" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new TermsAndConditionsCondition
                {
                    IssuerAssetClassType = IssuerAssetClassType.AllSupportedAssets,
                    ExcludeWarrants = false,
                    DaysAgo = 3, //Use either DaysAgo or StartDate
                    StartDate = null,
                    FixedIncomeRatingSources = FixedIncomeRatingSource.Fitch,
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "Conversion Terms End Date", "Index Principal Flag", "Conversion Terms Start Date", "RIC", "Round Lot Size",
                "Redemption Value", "Original Issue Amount", "Issue Price", "First Coupon Date", "Issue Date", "Par Value",
                "Accrual Date", "Base Index", "Factor", "Original Issue Discount Flag", "End Of Month Payment Flag", "ISIN",
                "Index Linked Bond Base Index", "Coupon Frequency Description", "Total Amount Outstanding", "Maturity Date",
                "Total Amount Issued", "Capitilization Flag", "Currency Code", "Trading Status");

            //Create 
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Time Series

        [Example("Create: Time Series")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(TimeSeriesReportTemplate))]
        public void CreateTimeSeriesReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TimeseriesPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TimeSeriesReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new TimeSeriesReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new TimeSeriesCondition
                {
                    StartDate = DateTime.UtcNow.Date.AddDays(-7),
                    EndDate = DateTime.UtcNow.Date
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "File Code", "RIC", "Trade Date", "Close Price", "Alternate Close Price", "Split Factor", "High Price",
                "Low Price", "Open Price", "VWAP Price", "Turnover", "Volume", "Bid Price", "Asset Type", "Quote ID",
                "Bid Yield", "Universal Close Price", "Exchange Code", "Currency Code");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Bond Schedule

        [Example("Create: Bond Schedule")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(BondScheduleReportTemplate))]
        public void CreateBondScheduleReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.BondSchedules);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "BondScheduleReportTemplate", "GetBondScheduleTypes", MethodType.Operation, Publish.Primary);
            //Get the list of bond schedule types
            var bondScheduleTypes = ExtractionsContext.ReportTemplateOperations.GetBondScheduleTypes();

            //Output the list of codes
            Status.WriteLine(String.Join(", ", bondScheduleTypes.Select(bst => bst.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "BondScheduleReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new BondScheduleReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new BondScheduleCondition
                {
                    BondScheduleTypeCodes = new[] { "CALS", "PUTS" }
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "Instrument ID", "Instrument ID Type", "Asset ID", "Asset Type", "CUSIP",
                "Common Code", "Extend Holders Option Flag", "Instrument ID", "Instrument ID Type", "MIC",
                "RIC", "SEDOL", "SICC", "Schedule Type", "Security Description", "User Defined Identifier");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Ratings

        [Example("Create: Ratings")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(RatingsReportTemplate))]
        public void CreateRatingsReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.BondRatings);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "RatingsReportTemplate", "GetRatingsSourceTypes", MethodType.Operation, Publish.Primary);
            //Get the list of rating source types
            var ratingSourceTypes = ExtractionsContext.ReportTemplateOperations.GetRatingsSourceTypes();

            //Output the list of codes
            Status.WriteLine(String.Join(", ", ratingSourceTypes.Select(bst => bst.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "RatingsReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new RatingsReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new RatingsCondition
                {
                    DaysAgo = 7,
                    IncludeInstrumentsWithNoRatings = false,
                    RatingLevel = new RatingLevel { IsIssue = false, IsIssuer = true, IssuerOutlookScopeCodeFilter = "F" },
                    RatingSourcesCodes = new[] { "ASI", "CAR", "SSU" },
                    ReportDateRangeType = ReportDateRangeType.Delta
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "Asset ID", "Asset Status", "Asset Status Description", "Asset Status Effective Date", "Asset SubType",
                "Asset Subtype Country Code", "Asset SubType Description", "Asset Type", "Asset Type Description",
                "Bond Type", "Bond Type Description", "Country of Issuance", "Country of Issuance Description",
                "CUSIP", "Domicile", "Domicile Description", "Entity ID", "FirmID", "Identifier Segment Source",
                "Identifier Segment Source Description", "Instrument ID", "Instrument ID Type", "Instrument Type Code",
                "Instrument Type Code Description", "ISIN", "ISO Country Code", "ISO Currency", "Local Code", "Rating Type",
                "RIC", "Security Description", "Series", "SIC Classification Code", "SIC Description1", "SICC", "Sicovam",
                "Ticker", "Usage Instrument SubType", "Usage Instrument Type");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);

        }

        #endregion

        #region Standard Events (Corporate Actions)

        [Example("Create: Standard Events (Corporate Actions)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CorporateActionsStandardReportTemplate))]
        public void CreateCorporateActionsStandardReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "CorporateActionsStandardReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new CorporateActionsStandardReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new CorporateActionsStandardCondition
                {
                    PreviousDays = 30,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    CorporateActionsCapitalChangeType = CorporateActionsCapitalChangeType.CapitalChangeExDate,
                    CorporateActionsDividendsType = CorporateActionsDividendsType.DividendPayDate,
                    CorporateActionsEarningsType = CorporateActionsEarningsType.PeriodEndDate,
                    CorporateActionsEquityOfferingsType = CorporateActionsEquityOfferingsType.AllPendingDeals,
                    CorporateActionsMergersAcquisitionsType = CorporateActionsMergersAcquisitionsType.DealAnnouncementDate,
                    CorporateActionsNominalValueType = CorporateActionsNominalValueType.NominalValueDate,
                    CorporateActionsSharesType = CorporateActionsSharesType.SharesAmountDate,
                    CorporateActionsStandardEventsType = CorporateActionsStandardEventsType.None,
                    CorporateActionsVotingRightsType = CorporateActionsVotingRightsType.VotingRightsDate,
                    QueryStartDate = null,
                    NextDays = null,
                    QueryEndDate = null,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    IncludeInstrumentsWithNoEvents = null,
                    IncludeNullDates = null,
                    ExcludeDeletedEvents = true,
                    ExcludeHistoricalEvents = false,
                    IncludeCapitalChangeEvents = true,
                    IncludeDividendEvents = true,
                    IncludeEarningsEvents = true,
                    IncludeMergersAndAcquisitionsEvents = true,
                    IncludeNominalValueEvents = true,
                    IncludePublicEquityOfferingsEvents = true,
                    IncludeSharesOutstandingEvents = true,
                    IncludeVotingRightsEvents = true,
                    LastEntityOnly = true
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "Adjustment Factor", "Capital Change Announcement Date", "Currency Code", "Exchange Code", "New Shares Currency",
                "Nominal Value Currency", "Dividend Rate", "User Defined Identifier", "RIC", "Offer Price", "Nominal Value",
                "Nominal Value Currency", "Nominal Value Date");
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Standard Events (Corporate Actions)

        [Example("Create: IPO Events (Corporate Actions)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CorporateActionsIpoReportTemplate))]
        public void CreateCorporateActionsIpoReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetExchangeTypes", MethodType.Operation, Publish.Primary);
            //Retrieve the list of available exchnage types.  You need not retrieve this list every time.  The report condition
            //accepts a list of exchange type codes which can be hard-coded.
            var exchangeTypes = ExtractionsContext.ReportTemplateOperations.GetExchangeTypes();

            //Output
            foreach (var exchangeType in exchangeTypes)
                Status.WriteLine("{0}: {1}", exchangeType.Code, exchangeType.Name);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "CorporateActionsIpoReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new CorporateActionsIpoReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new CorporateActionsIpoCondition
                {
                    PreviousDays = 30,
                    //QueryStartDate = null,
                    //NextDays = null,
                    //QueryEndDate = null,
                    IncludeInstrumentsWithNoEvents = true,
                    IncludeNullDates = null,
                    ExchangeTypes = new[] { "NYS", "LSE" },
                    ReportDateRangeType = ReportDateRangeType.Range
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "CUSIP", "City", "Common Code", "Company Name at Offer", "Corporate Actions Type", "Country", "Currency Code",
                "Email", "Entitlement", "Exchange Code", "Exchange Offer End Date", "Exchange Offer Start Date",
                "Expected Pricing Date", "Instrument ID", "Instrument ID Type", "Issuer OrgID", "Market Level", "Nominal Value",
                "Nominal Value Currency", "Number of Shares", "Offer Exchange", "Offer Exchange Description", "Offer ID",
                "Offer Status", "Offer Sub-Status", "Offer Type", "Shares Type", "Sicovam", "State Code", "Target Group",
                "Ticker", "Total Amount Raised", "Total Shares Offered", "Usage Instrument SubType", "Usage Instrument Type",
                "User Defined Identifier", "Web Site");
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region ISO 15022 (Corporate Actions)

        [Example("Create: ISO 15022 (Corporate Actions)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CorporateActionsIsoReportTemplate))]
        public void CreateCorporateActionsIsoReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "CorporateActionsIsoReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new CorporateActionsIsoReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.IsoFormat,
                Condition = new CorporateActionsIsoCondition
                {
                    CorporateActionsStandardYearType = CorporateActionsStandardYearType.Current,
                    ExcludeNilPaidFromPaymentEvents = true,
                    GrossAmountOnlyForPaymentEvents = true,
                    IncludeNonPendingEvents = true,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    PendingEventsPreviousDays = null,
                    PendingEventsPreviousMonths = null,
                    ReportDateRangeType = ReportDateRangeType.Init,
                    ReportIsoEvents = new[] { ReportIsoEventType.ACTV, ReportIsoEventType.BIDS }
                },
            };

            //NO FIELDS FOR CORAX ISO - the fields are pre-determined

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region ISO 15022 (Debt Corporate Actions)

        [Example("Create: ISO 15022 (Debt Corporate Actions)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(DebtCorporateActionsIsoReportTemplate))]
        public void CreateDebtCorporateActionsIsoReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.DebtCorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "DebtCorporateActionsIsoReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new DebtCorporateActionsIsoReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.IsoFormat,
                Condition = new DebtCorporateActionsIsoCondition
                {
                    DebtCorporateActionsStandardYearType = DebtCorporateActionsStandardYearType.Current,
                    IncludeNonPendingEvents = true,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    PendingEventsPreviousDays = null,
                    PendingEventsPreviousMonths = null,
                    ReportDateRangeType = ReportDateRangeType.Init,
                    ReportIsoEvents = new[] { ReportDebtIsoEventType.BRUP }
                },
            };

            //NO FIELDS FOR CORAX ISO - the fields are pre-determined

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Detail (Entity Data)

        [Example("Create: Detail (Entity Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(LegalEntityDetailReportTemplate))]
        public void CreateLegalEntityDetailReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityDetail);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "LegalEntityDetailReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new LegalEntityDetailReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new LegalEntityDetailCondition
                {
                    DeltaDays = 100
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "CIK", "Company/Registration Authority", "Company/Registration Number", "Country of Incorporation",
                "Country of Incorporation Description", "Date of Incorporation", "Domicile", "Domicile Description",
                "Entity URL", "Event Date", "Event Type", "FATF Jurisdiction", "GICS Industry Code",
                "ID", "ID Type", "Immediate Parent Official Name", "Immediate Parent RCP ID", "Immediate Parent Reuters Editorial RIC",
                "Is Issuer", "Jurisdiction of Incorporation", "Official Name", "Primary Regulator", "Primary Regulator ID",
                "RCP ID", "RCP Reuters Editorial RIC", "Record Active Flag", "Registered City/Town", "Registered Country",
                "Registered Country Description", "Registered Fax", "Registered Postal/Zip Code", "Registered State/County/Region",
                "S&P Issuer Long Rating", "S&P Issuer Long Rating - F", "S&P Issuer Long Rating Date", "S&P Issuer Long Rating Date - F",
                "S&P Issuer Long Rating Rank", "S&P Issuer Long Rating Rank - F", "S&P Issuer Short Rating",
                "S&P Issuer Short Rating - F", "S&P Issuer Short Rating Date", "S&P Issuer Short Rating Date - F",
                "S&P Issuer Short Rating Rank", "S&P Issuer Short Rating Rank - F");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Hierarchy (Entity Data)

        [Example("Create: Hierarchy (Entity Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(LegalEntityHierarchyReportTemplate))]
        public void CreateLegalEntityHierarchyReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.LegalEntityHierarchy);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "LegalEntityHierarchyReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new LegalEntityHierarchyReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new LegalEntityHierarchyCondition
                {
                    DeltaDays = 100
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "ID", "ID Type", "Immediate Parent ID", "Immediate Parent Official Name",
                "Immediate Parent RCP ID", "Node/Level", "Official Name", "RCP ID",
                "Source RCP ID", "Ultimate Parent ID", "Ultimate Parent Official Name", "Ultimate Parent RCP ID",
                "User Defined Identifier");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Summary Estimates (Estimates Data)

        [Example("Create: Summary Estimates (Estimates Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EstimatesSummaryReportTemplate))]
        public void CreateEstimatesSummaryReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesSummary);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EstimatesSummaryReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new EstimatesSummaryReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new EstimatesSummaryCondition
                {
                    DaysAgo = 30,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = RelativeFiscalYearType.FY1,
                    FiscalYearTo = RelativeFiscalYearType.FY3,
                    QuarterFrom = RelativeQuarterType.None,
                    QuarterTo = RelativeQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    SemiAnnualFrom = RelativeSemiAnnualType.None,
                    SemiAnnualTo = RelativeSemiAnnualType.None
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "RIC", "Security Description", "Period", "Period Date", "Measure", "Flash Mean Estimate", "Flash Number of Estimates",
                "High Estimate", "High Estimate - 3 Months ago", "High Estimate - 4 Weeks ago", "Low Estimate",
                "Low Estimate - 3 Months ago", "Low Estimate - 4 Weeks ago", "Mean Estimate", "Mean Estimate - 3 Months ago",
                "Mean Estimate - 4 Weeks ago", "Median Estimate", "Median Estimate - 3 Months ago",
                "Median Estimate - 4 Weeks ago", "Number of Estimates", "Number of Estimates - 3 Months ago",
                "Number of Estimates - 4 Weeks ago", "Number of Estimates Lowered - 1 Month",
                "Number of Estimates Lowered - 1 Week", "Number of Estimates Lowered - 4 Weeks", "Number of Estimates Raised - 1 Month",
                "Number of Estimates Raised - 1 Week", "Number of Estimates Raised - 4 Weeks", "Estimate Currency Code");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Actual Estimates (Estimates Data)

        [Example("Create: Actual Estimates (Estimates Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EstimatesActualReportTemplate))]
        public void CreateEstimatesActualReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesActual);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EstimatesActualReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new EstimatesActualReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new EstimatesActualCondition
                {
                    DaysAgo = 30,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = HistoricalFiscalYearType.FYNegative1,
                    FiscalYearTo = HistoricalFiscalYearType.FYNegative5,
                    QuarterFrom = HistoricalQuarterType.None,
                    QuarterTo = HistoricalQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    SemiAnnualFrom = HistoricalSemiAnnualType.None,
                    SemiAnnualTo = HistoricalSemiAnnualType.None,
                    ExcludeRestatedActuals = true
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "Actual Currency", "Actual Date", "Actual Status", "Actual Value", "Mean Estimate", "Measure", "MIC",
                "Number of Estimates", "OPOL", "Period", "Period Date", "Quote ID", "RCP ID", "Reuters Editorial RIC",
                "RIC", "Security Description", "SEDOL", "SICC", "Sicovam", "Standard Deviation", "Ticker", "Trading Status",
                "Update Date", "Update Type", "Usage Instrument SubType", "Usage Instrument Type");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Detail Estimates (Estimates Data)

        [Example("Create: Detail Estimates (Estimates Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EstimatesDetailReportTemplate))]
        public void CreateEstimatesDetailReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesDetail);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EstimatesDetailReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new EstimatesDetailReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new EstimatesDetailCondition
                {
                    DaysAgo = 30,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = RelativeFiscalYearType.FY1,
                    FiscalYearTo = RelativeFiscalYearType.FY3,
                    QuarterFrom = RelativeQuarterType.None,
                    QuarterTo = RelativeQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    SemiAnnualFrom = RelativeSemiAnnualType.None,
                    SemiAnnualTo = RelativeSemiAnnualType.None
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "RIC", "Period", "Period Date", "Previous Broker Recommendation", "Previous Broker Recommendation Description",
                "Previous Currency", "Previous Estimate", "Previous Horizon", "Previous IBES Entry Date",
                "Previous IBES Recommendation", "Previous IBES Recommendation Description", "Previous Is Diluted",
                "Previous Is Excluded", "Previous Status", "Quote ID", "RCP ID", "Reuters Editorial RIC",
                "Ticker", "Trading Status", "Update Date", "Update Type", "Usage Instrument SubType", "Usage Instrument Type",
                "User Defined Identifier");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Company Level Footnotes (Estimates Data)

        [Example("Create: Company Level Footnotes (Estimates Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EstimatesFootnotesCompanyReportTemplate))]
        public void CreateEstimatesFootnotesCompanyReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EstimatesCompanyFootnotes);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EstimatesFootnotesCompanyReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new EstimatesFootnotesCompanyReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new EstimatesFootnotesCompanyCondition
                {
                    DaysAgo = 30,
                    ReportDateRangeType = ReportDateRangeType.Delta
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "Asset ID", "Asset Status", "Asset Status Description", "CUSIP", "Common Code", "Company Name", "Country",
                "Currency Code", "Currency Code Description", "Dilution Factor", "Exchange", "Exchange Code",
                "Exchange Description", "Footnote", "Footnote Expiry Date", "Footnote Type", "IBES Code", "IBES Entry Date",
                "ISIN", "Issuer Name", "RIC", "Security Description", "Ticker", "Trading Status", "Update Date",
                "Update Type", "Usage Instrument SubType", "Usage Instrument Type");


            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Company Level Footnotes (Estimates Data)

        [Example("Create: Detail Level Footnotes (Estimates Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EstimatesFootnotesDetailReportTemplate))]
        public void CreateEstimatesFootnotesDetailReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.DetailEstimateFootnotes);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EstimatesFootnotesDetailReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new EstimatesFootnotesDetailReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new EstimatesFootnotesDetailCondition
                {
                    DaysAgo = 30,
                    EstimateTypes = new[] { EstimateType.EPS, EstimateType.CPS },
                    FiscalYearFrom = RelativeFiscalYearType.FY1,
                    FiscalYearTo = RelativeFiscalYearType.FY3,
                    QuarterFrom = RelativeQuarterType.None,
                    QuarterTo = RelativeQuarterType.None,
                    ReportCompanyType = ReportCompanyType.ParentAndConsolidated,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    SemiAnnualFrom = RelativeSemiAnnualType.None,
                    SemiAnnualTo = RelativeSemiAnnualType.None
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "Analyst Name", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type", "Asset Type Description",
                "Broker Name", "Dilution Factor", "Estimate Flag", "Exchange", "Exchange Code", "Exchange Description",
                "File Code", "Footnote", "Footnote Expiry Date", "Footnote Type", "IBES Code", "IBES Estimate Entry Date",
                "IBES Footnote Entry Date", "Issuer Name", "Issuer OrgID", "Market MIC", "Measure", "Period", "Period Date",
                "Quote ID", "RIC", "Trading Status", "Update Date", "Update Type", "Usage Instrument SubType",
                "Usage Instrument Type");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region News Items (News)

        [Example("Create: News Items")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(NewsItemsReportTemplate))]
        public void CreateNewsItemsReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.NewsItems);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "NewsItemsReportTemplate", "GetNewsAttributionTypes", MethodType.Operation, Publish.Primary);
            //Get the list of attributions which can be applied to Condition.NewsAttributionsCodes
            var newsAttributions = ExtractionsContext.ReportTemplateOperations.GetNewsAttributionTypes();
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "NewsItemsReportTemplate", "GetNewsTopicCodes", MethodType.Operation, Publish.Primary);
            //Get the list of topics which can be applied to Condition.NewsTopicsCodes
            var newsTopics = ExtractionsContext.ReportTemplateOperations.GetNewsTopicTypes();
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "NewsItemsReportTemplate", "GetNewsItemsLanguageTypes", MethodType.Operation, Publish.Primary);
            //Get the list of languages which can be applied to Condition.NewsItemsLanguageCodes
            var newsLanguages = ExtractionsContext.ReportTemplateOperations.GetNewsItemsLanguageTypes();
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "NewsItemsReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create the new report template
            var reportTemplate = new NewsItemsReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new NewsItemsCondition
                {
                    QueryStartDate = null,
                    QueryEndDate = null,
                    DaysAgo = 1,
                    HoursAgo = 20,
                    MinutesAgo = 40,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    NewsAttributionsCodes = new[] { "RTRS", "JSE" }, //from newsAttributionCodes
                    NewsTopicsCodes = new[] { "GDP", "ECI", "MCE" }, //from newsTopics
                    NewsItemsSource = NewsItemsSourceType.AllNews,
                    NewsItemsLanguage = NewsItemsLanguageType.Selected,
                    NewsLanguagesCodes = new[] { "EN" } //from newsLanguages
                },
            };

            //Add fields
            reportTemplate.ContentFields.AddRange(
                "Asset ID", "Attribution", "Common Code", "CUSIP", "File Code", "Headline", "Instrument ID", "Instrument ID Type",
                "ISIN", "Language", "OPOL", "PILC", "PNAC", "Products", "Quote ID", "Related RICs", "Reuters Editorial RIC",
                "RIC", "SEDOL", "SICC", "Story Body", "Story Date Time", "Take Date Time", "Ticker", "Topics",
                "Unique Story Index");

            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Premimum Of Day

        [Example("Create: Premium End Of Day")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumEndOfDayPricingReportTemplate))]
        public void CreatePremiumEndOfDayPricingReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumEndOfDayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumEndOfDayPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumEndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumEndOfDayPricingCondition
                {
                    IncludeIntradayPricing = true,
                    IntradayFixedIncomePremiumEndOfDay = true,
                    ScalableCurrency = false
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Instrument ID", "Instrument ID Type", "Trading Indicator", "Trading Indicator Description",
                "Universal Close Validation Code", "Universal Close Validation Code Description", "Universal Bid Price",
                "Universal Ask Price", "Universal Bid Ask Date", "Universal Close Price", "Universal Close Price Date",
                "RIC", "ISIN", "Asset Status", "Asset Status Description", "Asset SubType", "Asset SubType Description",
                "Asset Type", "Asset Type Description", "Currency Code", "Currency Code Description",
                "Current Embargo Delay", "Embargo Times", "Embargo Window", "Exchange Requiring Embargo",
                "File Code", "Instrument Snap Time", "Last Intraday Update Time", "Maximum Embargo Delay",
                "Real Time Permitted", "PE Code", "Security Description", "Trading Status");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Premium Pricing  (the new single template with enum for region/cycle)

        [Example("Create: Premium Pricing - Asia 3PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingAsia3PmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingAsia3PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Premium Pricing - Asia 4PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingAsia4PmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingAsia4PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Premium Pricing - Asia 6PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingAsia6PmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingAsia6PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Premium Pricing - Asia 9:30PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingAsia10PmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingAsia930PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Premium Pricing - Emea 12PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingEmea12PmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingEmea12PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Premium Pricing - Emea 2PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingEmea2PmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingEmea2PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Premium Pricing - Emea 4:15PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingEmea4PmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingEmea415PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Premium Pricing - Emea 6PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingEmea6PmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = false,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingEmea6PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Premium Pricing - US 3PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingUs3pmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = true,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingUS3PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Premium Pricing - US 4PM  (Generic Template)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PremiumPricingReportTemplate))]
        public void PremiumPricingUs4PmReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PremiumPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new PremiumPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new PremiumPricingCondition
                {
                    ApplyFIGSPricing = true,
                    PremiumPricingRegionCycle = PremiumPricingRegionCycle.PremiumPricingUS4PM
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Single Historical Price

        [Example("Create: Single Historical Price (Pricing Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(SingleHistoricalPriceReportTemplate))]
        public void CreateSingleHistoricalPriceReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TimeseriesPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "SingleHistoricalPriceReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new SingleHistoricalPriceReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new SingleHistoricalPriceCondition
                {
                    LookBackPeriod = LookbackType.OneMonth,
                    PriceDate = DateTime.UtcNow
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Close Price", "Bid Price", "Ask Price", "Trade Date", "CUSIP", "RIC", "Security Description",
                "Alternate Close Price", "Low Price", "High Price", "Mid Price", "Open Price", "Instrument ID");
            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Single Price History

        [Example("Create: Single Price History")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(SinglePriceHistoryReportTemplate))]
        public void CreateSinglePriceHistoryReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            // Retrieve the list of available fields.  This step can be skipped if template fields are already known.
            // The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            // adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PriceHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "SinglePriceHistoryReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new SinglePriceHistoryReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new SinglePriceHistoryCondition
                {
                    EndDate = new DateTimeOffset(new DateTime(2017, 4, 30))
                    ,
                    Lookback = LookbackType.OneMonth
                    ,
                    AdjustedPrices = false
                    ,
                    LastEntityOnly = false
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange("Ask High", "Ask Low", "Bid High", "Bid Low", "Trade Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Price History

        [Example("Create: Price History")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(PriceHistoryReportTemplate))]
        public void CreatePriceHistoryReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            // Retrieve the list of available fields.  This step can be skipped if template fields are already known.
            // The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            // adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.PriceHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PriceHistoryReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new PriceHistoryReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,

                // 1# Report Template base on Range Query Type
                Condition = new PriceHistoryCondition
                {
                    ReportDateRangeType = ReportDateRangeType.Range,
                    QueryStartDate = new DateTimeOffset(new DateTime(2018, 10, 10)),
                    QueryEndDate = new DateTimeOffset(new DateTime(2018, 10, 10)),
                    AdjustedPrices = false,
                    LastEntityOnly = false
                }

                //-- OR 2# Report Template base on Range Query Type	            
                //Condition = new PriceHistoryCondition
                //{
                //	ReportDateRangeType = ReportDateRangeType.Delta,
                //	DaysAgo = 4,
                //	AdjustedPrices = true
                //}			

                //-- OR 3# Report Template base on Range Query Type				
                //Condition = new PriceHistoryCondition
                //{
                //	ReportDateRangeType = ReportDateRangeType.Relative,
                //  RelativeStartDaysAgo = 0,
                //	RelativeEndDaysAgo = 5,
                //	AdjustedPrices = true
                //}
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange("Ask High", "Ask Low", "Bid High", "Bid Low", "Trade Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Fund Allocation

        [Example("Create: Fund Allocation (Reference Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(FundAllocationReportTemplate))]
        public void CreateFundAllocationReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.FundAllocation);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "FundAllocationReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new FundAllocationReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new FundAllocationCondition
                {
                    FundAllocationTypes = new[]
                        {
                            FundAllocationTypes.Currency,
                            FundAllocationTypes.FullHoldings
                        }
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Asset SubType Description", "Asset Type", "Asset Type Description", "CUSIP",
                "Identifier Segment Source",
                "Identifier Segment Source Description", "Instrument ID", "Instrument ID Type", "Security Description",
                "Allocation Asset Type", "Allocation CUSIP", "Allocation Date", "Allocation Item", "Allocation OrgID",
                "Allocation Percentage", "Allocation Rank", "Market Value Currency", "Market Value Held", "RIC",
                "FundSERVAC", "FundSERVBE", "FundSERVDM", "FundSERVDO", "FundSERVFE", "FundSERVIS", "FundSERVLL",
                "FundSERVNL", "FundSERVVS");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Symbol Cross Reference

        [Example("Create: Symbol Cross Reference (Reference Data)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(SymbolCrossReferenceReportTemplate))]
        public void CreateSymbolCrossReferenceReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.SymbolCrossReference);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "SymbolCrossReferenceReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new SymbolCrossReferenceReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new SymbolCrossReferenceCondition
                {
                    AllXRefIdentifierTypes = true,
                    DaysAgo = 14,
                    IdentifierTypes = new[] {
                        IdentifierType.Ric,
                        IdentifierType.Sedol,
                        IdentifierType.Cusip,
                        IdentifierType.Wertpapier,
                        IdentifierType.Valoren
                    },
                    ReportDateRangeType = ReportDateRangeType.Delta
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Quote ID", "Asset Type", "Asset Type Description",
                "Security Description", "Instrument ID", "Instrument ID Type",
                "Reuters Editorial RIC");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Starmine

        [Example("Create: Starmine")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(StarmineReportTemplate))]
        public void CreateStarmineReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.Starmine);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "StarmineReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Create a new end of day report template
            var reportTemplate = new StarmineReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "StarMine Calculation Date", "Credit Text Distance to Default", "Credit Text Mining Broker Research Component",
                "Credit Text Mining Broker Research External Rank", "Credit Text Mining Broker Research Income Rank",
                "Credit Text Mining Broker Research Legal Rank", "Credit Text Mining Broker Research Structural Rank",
                "Credit Text Mining Country Rank", "Credit Text Mining External Category Score",
                "Credit Text Mining Filing Income Rank", "Credit Text Mining Filing Legal Rank");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region TickHistory Intraday Summaries

        [Example("Create: TickHistory Intraday Summaries")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(TickHistoryIntradaySummariesReportTemplate))]
        public void CreateTickHistoryIntradayReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryIntradaySummaries);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TickHistoryIntradaySummariesReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new TickHistoryIntradaySummariesReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new TickHistoryIntradaySummariesCondition
                {
                    DaysAgo = null,
                    MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                    QueryStartDate = new DateTimeOffset(2015, 12, 1, 13, 0, 0, TimeSpan.FromHours(0)),
                    QueryEndDate = new DateTimeOffset(2015, 12, 1, 13, 5, 0, TimeSpan.FromHours(0)),
                    ReportDateRangeType = ReportDateRangeType.Range,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                    Preview = PreviewMode.None,
                    DisplaySourceRIC = false,
                    TimebarPersistence = true
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange("Close Ask", "Close Bid", "High", "High Ask", "High Bid", "Last", "Low", "Low Ask", "Low Bid");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region TickHistory MarketDepth

        [Example("Create: TickHistory MarketDepth")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(TickHistoryMarketDepthReportTemplate))]
        public void CreateTickHistoryMarketDepthReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryMarketDepth);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TickHistoryMarketDepthReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new TickHistoryMarketDepthReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                TemplateTypeCode = "THM",
                CreateDate = new DateTimeOffset(DateTime.UtcNow),
                LastChangedDate = new DateTimeOffset(DateTime.UtcNow),
                CompressionType = ReportCompressionType.Zip,
                Delimiter = ReportDelimiter.Pipe,
                DeliveryType = ReportDeliveryType.None,
                Destination = null,
                Condition = new TickHistoryMarketDepthCondition
                {
                    View = TickHistoryMarketDepthViewOptions.NormalizedLL2,
                    NumberOfLevels = 3,
                    SortBy = TickHistorySort.SingleByRic,
                    MessageTimeStampIn = TickHistoryTimeOptions.LocalExchangeTime,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    DaysAgo = 3,
                    DisplaySourceRIC = false
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange("Ask Price", "Bid Price");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region TickHistory Raw

        [Example("Create: TickHistory Raw")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(TickHistoryRawReportTemplate))]
        public void CreateTickHistoryRawReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryRaw);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TickHistoryRawReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new TickHistoryRawReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                TemplateTypeCode = "THR",
                CreateDate = new DateTimeOffset(DateTime.UtcNow),
                LastChangedDate = new DateTimeOffset(DateTime.UtcNow),
                CompressionType = ReportCompressionType.Zip,
                Delimiter = ReportDelimiter.Pipe,
                DeliveryType = ReportDeliveryType.None,
                Destination = null,
                Condition = new TickHistoryRawCondition
                {
                    SortBy = TickHistorySort.SingleByRic,
                    MessageTimeStampIn = TickHistoryTimeOptions.LocalExchangeTime,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    DomainCode = TickHistoryRawDomain.MarketPrice,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    DaysAgo = 3,
                    DisplaySourceRIC = false,
                    Fids = "68"
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region TickHistory TimeAndSales

        [Example("Create: TickHistory TimeAndSales")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(TickHistoryTimeAndSalesReportTemplate))]
        public void CreateTickHistoryTimeAndSalesReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TickHistoryTimeAndSalesReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new TickHistoryTimeAndSalesReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                TemplateTypeCode = "THT",
                CreateDate = new DateTimeOffset(DateTime.UtcNow),
                LastChangedDate = new DateTimeOffset(DateTime.UtcNow),
                CompressionType = ReportCompressionType.Zip,
                Delimiter = ReportDelimiter.Pipe,
                DeliveryType = ReportDeliveryType.None,
                Destination = null,
                Condition = new TickHistoryTimeAndSalesCondition
                {
                    ApplyCorrectionsAndCancellations = true,
                    SortBy = TickHistorySort.SingleByRic,
                    MessageTimeStampIn = TickHistoryTimeOptions.LocalExchangeTime,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    DaysAgo = 3,
                    DisplaySourceRIC = false
                },
                ShowColumnHeaders = true
            };

            reportTemplate.CompressionType = ReportCompressionType.None;
            reportTemplate.OutputFormat = ReportOutputFormat.CommaSeparatedValues;
            reportTemplate.Delimiter = ReportDelimiter.None;
            reportTemplate.DeliveryType = ReportDeliveryType.None;
            reportTemplate.Destination = string.Empty;
            reportTemplate.Headers = new ReportHeaderField[0];
            reportTemplate.Trailers = new ReportHeaderField[0];
            reportTemplate.ContentFields = new[]
                    {
                        new ContentField
                        {
                            FieldName = "Correction - Price",
                        },
                        new ContentField
                        {
                            FieldName = "Correction - Buyer ID",
                        }
                    };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Commodities Corrections History

        [Example("Create: Commodities Corrections History")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CommoditiesCorrectionsHistoryReportTemplate))]
        public void CreateCommoditiesCorrectionsHistoryReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            // Retrieve the list of available fields.  This step can be skipped if template fields are already known.
            // The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            // adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CommoditiesCorrectionsHistory);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "CommoditiesCorrectionsHistoryReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new CommoditiesCorrectionsHistoryReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                Condition = new CommoditiesCorrectionsHistoryCondition
                {
                    ReportDateRangeType = ReportDateRangeType.Range,
                    QueryStartDate = new DateTimeOffset(2017, 1, 1, 0, 0, 0, TimeSpan.FromHours(0)),
                    QueryEndDate = new DateTimeOffset(2017, 4, 30, 0, 0, 0, TimeSpan.FromHours(0))
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange("RIC", "Field Type", "Correction Date", "Corrected Value");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Elektron End Of Day Pricing

        [Example("Create: Elektron End Of Day Pricing")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ElektronEndOfDayPricingReportTemplate))]
        public void CreateElektronEndOfDayPricingReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            // Retrieve the list of available fields.  This step can be skipped if template fields are already known.
            // The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            // adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.ElektronEndOfDayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ElektronEndOfDayPricingReportTemplate", "Create", MethodType.Operation, Publish.Primary);
            //Define the template
            var reportTemplate = new ElektronEndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,

                // 1# Report Template base on Range Query Type
                Condition = new ElektronEndOfDayPricingCondition
                {
                    AdjustedPrices = false
                }
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange("Ask High", "Ask Low", "Bid High", "Bid Low", "Trade Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Other

        [Example("Copy")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void CopyReportTemplate()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            //Create a report template
            var original = new EndOfDayPricingReportTemplate
            {
                Name = "MyReportTemplate" + Suffix.New()
            };
            ExtractionsContext.ReportTemplateOperations.Create(original);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Copy", MethodType.Operation, Publish.Primary);
            //Create a copy of the report template
            var copy = ExtractionsContext.ReportTemplateOperations
                .Copy(original.ReportTemplateId, "Copy of MyReportTemplate" + Suffix.New());

            //Output
            Status.WriteLine(copy.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            //Clean up.  Delete accepts either the id or the instance of the report template.
            ExtractionsContext.ReportTemplateOperations.Delete(original);
            ExtractionsContext.ReportTemplateOperations.Delete(copy);
            Status.EndNotify(ExtractionsContext);
        }

        /// <summary>
        /// This examples demonstrates how to update an existing content field.
        /// </summary>
        //[Example("Update Report Template Fields")] //NOT YET SUPPORTED
        public void UpdateContentFields()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            var reportTemplateName = Guid.NewGuid().ToString();
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = reportTemplateName,
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                TemplateTypeCode = "EOD",
                CompressionType = ReportCompressionType.Zip,
                Delimiter = ReportDelimiter.Pipe,
                DeliveryType = ReportDeliveryType.None,
                Destination = null
            };
            var contentFieldBidPrice = new ContentField
            {
                FieldName = "Bid Price",
                Justification = ContentFieldJustification.Left,
                Label = "Bid Price",
                Width = 10,
                WidthStyle = ContentFieldWidthStyle.FixedWidth,
                Format = new ContentFieldNumberFormat
                {
                    DecimalPlaces = 2,
                    DecimalSeparator = ContentFieldDecimalSeparator.Period,
                    IntegerPlaces = 4,
                    NegativeSignPosition = ContentFieldNegativeSignPosition.Before,
                    ThousandSeparator = ContentFieldThousandsSeparator.Comma,
                    UseLeadingZero = false,
                    UseThousandSeparator = true,
                    UseTrailingZero = true,
                }
            };
            var contentFieldAskPrice = new ContentField
            {
                FieldName = "Ask Price",
                Justification = ContentFieldJustification.Left,
                Label = "Ask Price",
                Width = 10,
                WidthStyle = ContentFieldWidthStyle.FixedWidth,
                Format = new ContentFieldNumberFormat
                {
                    DecimalPlaces = 2,
                    DecimalSeparator = ContentFieldDecimalSeparator.Period,
                    IntegerPlaces = 4,
                    NegativeSignPosition = ContentFieldNegativeSignPosition.Before,
                    ThousandSeparator = ContentFieldThousandsSeparator.Comma,
                    UseLeadingZero = false,
                    UseThousandSeparator = true,
                    UseTrailingZero = true,
                }
            };
            reportTemplate.ContentFields.Add(contentFieldBidPrice);
            reportTemplate.ContentFields.Add(contentFieldAskPrice);
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "UpdateContentField", MethodType.Operation, Publish.Primary);
            var contentFieldBidLow = new ContentField
            {
                FieldName = "Bid Low",
                Justification = ContentFieldJustification.Left,
                Label = "Bid Low",
                Width = 10,
                WidthStyle = ContentFieldWidthStyle.FixedWidth,
                Format = new ContentFieldNumberFormat
                {
                    DecimalPlaces = 2,
                    DecimalSeparator = ContentFieldDecimalSeparator.Period,
                    IntegerPlaces = 4,
                    NegativeSignPosition = ContentFieldNegativeSignPosition.Before,
                    ThousandSeparator = ContentFieldThousandsSeparator.Comma,
                    UseLeadingZero = false,
                    UseThousandSeparator = true,
                    UseTrailingZero = true,
                }
            };
            var contentFieldNumberFormat = (ContentFieldNumberFormat)contentFieldBidLow.Format;
            contentFieldNumberFormat.DecimalPlaces = 5;

            ExtractionsContext.ReportTemplateOperations.UpdateContentField(reportTemplate, contentFieldBidLow);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "GetContentFieldsByReportTemplateId");
            var actualContentFields = ExtractionsContext.ReportTemplateOperations.GetContentFieldsByReportTemplateId(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);

            Status.WriteLine(actualContentFields.Count(x => x.FieldName == "Bid Low") == 1);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate.ReportTemplateId);
            Status.EndNotify(ExtractionsContext);
        }

        /// <summary>
        /// This example demonstrates a validation error by creating two report templates with the same name.
        /// NOTE:  report templates names must be unique.
        /// </summary>
        [Example("Error: Duplicate Name")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(ReportTemplate))]
        public void ValidationErrorDuplicateName()
        {
            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            //Create the first report template
            var name = "MyReportTemplate" + Suffix.New();
            var reportTemplate = new EndOfDayPricingReportTemplate { Name = name };
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplate", "Create");
            try
            {
                //Create another of the same name
                var duplicateNameReportTemplate = new EndOfDayPricingReportTemplate { Name = name };
                ExtractionsContext.ReportTemplateOperations.Create(duplicateNameReportTemplate);
            }
            catch (ValidationException ex)
            {
                //we expect an error with duplicate name
                foreach (var exception in ex.ValidationErrors)
                    Status.WriteLine("Validation Error:  Id = {0}, Message = {1}", exception.Id, exception.Message);
            }
            Status.EndNotify(ExtractionsContext);
        }

        #endregion
    }
}