﻿using Dss.Api.Examples.Internal;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.SubjectLists;

namespace Dss.Api.Examples.Extractions.SubjectLists.EntityLists
{

    [Example("Entity List Examples")]
    public class EntityListExamples
    {

        #region Get

        [Example("Get All (Paged Results)")]
        public void GetAllPagedResults()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();
            //Set the page size to 1 record per page for the purposes of this example
            extractionsContext.Preferences.MaxPageSize = 1;

            Status.Notify(extractionsContext, "EntityList", "Create");
            //Create an instrument list 
            extractionsContext.EntityListOperations.Create(new EntityList { Name = "Entity List Name" + Suffix.New() });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Create");
            //Create another instrument list 
            extractionsContext.EntityListOperations.Create(new EntityList { Name = "Entity List Name" + Suffix.New() });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAll", MethodType.Operation, Publish.Primary);
            //Get the first page of instrument lists
            var page1Results = extractionsContext.EntityListOperations.GetAll();

            //Output
            Status.WriteLine("HasNextPage: {0}", page1Results.HasNextPage);
            Status.WriteLine("NextLink.Url: {0}", page1Results.NextLink.Url);
            Status.WriteLine("Count: {0}", page1Results.Count());
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAll", MethodType.Operation, Publish.Primary);
            if (page1Results.HasNextPage) //Use a while loop if you need to retrieve all instrument lists
            {
                //Get the second page of instrument lists by passing in the NextLink returned from the previous request
                var page2Results = extractionsContext.EntityListOperations.GetAll(page1Results.NextLink);

                //Output
                Status.WriteLine("HasNextPage: {0}", page2Results.HasNextPage);
                Status.WriteLine("NextLink.Url: {0}", page2Results.NextLink.Url);
                Status.WriteLine("Count: {0}", page2Results.Count());
            }
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to retrieve all of the entities associated with an EntityList.
        /// </summary>
        [Example("Get All Entities")]
        public void GetAllEntities()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "Entity List Name" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAllEntities", MethodType.Operation, Publish.Primary);
            extractionsContext.EntityListOperations.GetAllEntities(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to retrieve an existing EntityList by it's name.
        /// </summary>
        [Example("Get By Name")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void GetByName()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var name = "GetByName" + Suffix.New();
            var entityList = new EntityList { Name = name };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetByName", MethodType.Operation, Publish.Primary);
            var createdEntityList = extractionsContext.EntityListOperations.GetByName(name);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        #endregion

        #region CUD

        /// <summary>
        /// This example demonstrates how to create an EntityList.
        /// </summary>
        [Example("Create")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void Create()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create", MethodType.Operation, Publish.Primary);
            var entityList = new EntityList { Name = "Entity List Name" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to update an EntityList.
        /// </summary>
        [Example("Update")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void Update()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "Entity List Name" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Update", MethodType.Operation, Publish.Primary);
            createdEntityList.Name = "Updated Entity List Name";
            extractionsContext.EntityListOperations.Update(createdEntityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to delete an EntityList.
        /// </summary>
        [Example("Delete")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void Delete()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "Entity List Name" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete", MethodType.Operation, Publish.Primary);
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        #endregion

        #region Items

        /// <summary>
        /// This example demonstrates how to append identifiers to an existing EntityList .
        /// </summary>
        [Example("Append Identifiers")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void AppendIdentifiers()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "AppendIdentifiers" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers", MethodType.Operation, Publish.Primary);
            IEnumerable<EntityIdentifier> appendIdentifiers = new[]
            {
                new EntityIdentifier
                {
                    Identifier = "12286",
                    IdentifierType = IdentifierType.OrgId,
                    UserDefinedIdentifier = "UserIdent4A"
                },
                // This next entry will cause a warning message in the AppendIdentifiers results
                // and this UDI will be dropped in the GetAllEntities results
                new EntityIdentifier
                {
                    Identifier = "12286",
                    IdentifierType = IdentifierType.OrgId,
                    UserDefinedIdentifier = "ßÃÐ"
                },
            };
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, true, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAllEntities");
            extractionsContext.EntityListOperations.GetAllEntities(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }


        /// <summary>
        /// This example demonstrates how to append identifiers to an existing EntityList with options.
        /// </summary>
        [Example("Append Identifiers With Options")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void AppendIdentifiersWithValidationOptions()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "AppendIdentifiersWithOptions" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiersWithOptions", MethodType.Operation, Publish.Primary);
            IEnumerable<EntityIdentifier> appendIdentifiers = new[]
            {
                new EntityIdentifier
                {
                    Identifier = "12286",
                    IdentifierType = IdentifierType.OrgId,
                    UserDefinedIdentifier = "UserIdent4A"
                },
                // This next entry will cause a warning message in the AppendIdentifiers results
                // and this UDI will be dropped in the GetAllEntities results
                new EntityIdentifier
                {
                    Identifier = "12286",
                    IdentifierType = IdentifierType.OrgId,
                    UserDefinedIdentifier = "ßÃÐ"
                }
            };

            EntityValidationOptions options = new EntityValidationOptions { AllowDuplicateInstruments = true, AllowUnmanagedOrUnverifiedEntities = true, IncludeParentAndUltimateParent = false };

            extractionsContext.EntityListOperations.AppendIdentifiersWithOptions(entityList, appendIdentifiers, options);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAllEntities");
            extractionsContext.EntityListOperations.GetAllEntities(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to revalidate the entities already contained in  list.
        /// </summary>
        [Example("Revalidate")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void Revalidate()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "AppendIdentifiers" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiers = new[] {new EntityIdentifier
				{
					Identifier = "12286",
					IdentifierType = IdentifierType.OrgId,
					UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Revalidate", MethodType.Operation, Publish.Primary);
            extractionsContext.EntityListOperations.Revalidate(entityList, true);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }


        /// <summary>
        /// This example demonstrates how to revalidate the entities already contained in  list with options.
        /// </summary>
        [Example("Revalidate with options")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void RevalidateWithOptions()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "AppendIdentifiers" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiers = new[] {new EntityIdentifier
                {
                    Identifier = "12286",
                    IdentifierType = IdentifierType.OrgId,
                    UserDefinedIdentifier = "UserIdent4A"
                }};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "RevalidateWithOptions", MethodType.Operation, Publish.Primary);

            EntityValidationOptions options = new EntityValidationOptions { AllowDuplicateInstruments = true, AllowUnmanagedOrUnverifiedEntities = true, IncludeParentAndUltimateParent = false };

            extractionsContext.EntityListOperations.RevalidateWithOptions(entityList, options);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }


        /// <summary>
        /// This example demonstrates how to validates and replace the contents of the EntityList with the described entities.
        /// </summary>
        [Example("ReplaceAllWithIdentifiers")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void ReplaceAllWithIdentifiers()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var listName = "AppendIdentifiers" + Suffix.New();
            var entityList = new EntityList { Name = listName };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiersOriginal = new[] {new EntityIdentifier
				{
					Identifier = "12286",
					IdentifierType = IdentifierType.OrgId,
					UserDefinedIdentifier = "UserIdentOriginal"
				}};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiersOriginal, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAllEntities");
            var itemsFromServer = extractionsContext.EntityListOperations.GetAllEntities(createdEntityList).ToArray();
            Status.WriteLine("Identifier: {0}", itemsFromServer[0].Identifier);
            Status.WriteLine("IdentifierType: {0}", itemsFromServer[0].IdentifierType);
            Status.WriteLine("UserDefinedIdentifier: {0}", itemsFromServer[0].UserDefinedIdentifier);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "ReplaceAllWithIdentifiers", MethodType.Operation, Publish.Primary);
            IEnumerable<EntityIdentifier> appendIdentifiersReplace = new[] {new EntityIdentifier
				{
                    Identifier = "27602", // MSFT.BA
                    IdentifierType = IdentifierType.OrgId,
                    UserDefinedIdentifier = "UserIdentReplace"
				}};
            extractionsContext.EntityListOperations.ReplaceAllWithIdentifiers(entityList, appendIdentifiersReplace, false, false);
            Status.EndNotify(extractionsContext);
            
            Status.Notify(extractionsContext, "EntityList", "Get");
            createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAllEntities");
            itemsFromServer = extractionsContext.EntityListOperations.GetAllEntities(createdEntityList).ToArray();
            Status.WriteLine("Identifier: {0}", itemsFromServer[0].Identifier);
            Status.WriteLine("IdentifierType: {0}", itemsFromServer[0].IdentifierType);
            Status.WriteLine("UserDefinedIdentifier: {0}", itemsFromServer[0].UserDefinedIdentifier);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to validates and replace the contents of the EntityList with the described entities.
        /// </summary>
        [Example("ReplaceAllWithIdentifiersWithOptions")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void ReplaceAllWithIdentifiersWithOptions()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var listName = "AppendIdentifiers" + Suffix.New();
            var entityList = new EntityList { Name = listName };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiersOriginal = new[] {new EntityIdentifier
                {
                    Identifier = "12286",
                    IdentifierType = IdentifierType.OrgId,
                    UserDefinedIdentifier = "UserIdentOriginal"
                }};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiersOriginal, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAllEntities");
            var itemsFromServer = extractionsContext.EntityListOperations.GetAllEntities(createdEntityList).ToArray();
            Status.WriteLine("Identifier: {0}", itemsFromServer[0].Identifier);
            Status.WriteLine("IdentifierType: {0}", itemsFromServer[0].IdentifierType);
            Status.WriteLine("UserDefinedIdentifier: {0}", itemsFromServer[0].UserDefinedIdentifier);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "ReplaceAllWithIdentifiersWithOptions", MethodType.Operation, Publish.Primary);
            IEnumerable<EntityIdentifier> appendIdentifiersReplace = new[] {new EntityIdentifier
                {
                    Identifier = "27602", // MSFT.BA
                    IdentifierType = IdentifierType.OrgId,
                    UserDefinedIdentifier = "UserIdentReplace"
                }};
            EntityValidationOptions options = new EntityValidationOptions { AllowDuplicateInstruments = true, AllowUnmanagedOrUnverifiedEntities = true, IncludeParentAndUltimateParent = false };
            extractionsContext.EntityListOperations.ReplaceAllWithIdentifiersWithOptions(entityList, appendIdentifiersReplace, options);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAllEntities");
            itemsFromServer = extractionsContext.EntityListOperations.GetAllEntities(createdEntityList).ToArray();
            Status.WriteLine("Identifier: {0}", itemsFromServer[0].Identifier);
            Status.WriteLine("IdentifierType: {0}", itemsFromServer[0].IdentifierType);
            Status.WriteLine("UserDefinedIdentifier: {0}", itemsFromServer[0].UserDefinedIdentifier);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to update a specific entity in an existing EntityList.
        /// </summary>
        [Example("Update Item")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void UpdateItem()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "Entity List Name" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiers = new[] {new EntityIdentifier
                {
                    Identifier = "12286",
                    IdentifierType = IdentifierType.OrgId,
                    UserDefinedIdentifier = "UserIdent4A"
                }};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAllEntities");
            var entities = extractionsContext.EntityListOperations.GetAllEntities(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "UpdateItem", MethodType.Operation, Publish.Primary);
            var entityListItem = entities.First();
            entityListItem.UserDefinedIdentifier = "1";
            entityListItem.IdentifierType = IdentifierType.Cusip;
            entityListItem.Identifier = "XYZ";
            extractionsContext.EntityListOperations.UpdateItem(entityListItem);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to delete a specific entity in an existing EntityList.
        /// </summary>
        [Example("Delete Item")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void DeleteItem()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "Entity List Name" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiers = new[] {new EntityIdentifier
				{
					Identifier = "12286",
					IdentifierType = IdentifierType.OrgId,
					UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetAllEntities");
            var entities = extractionsContext.EntityListOperations.GetAllEntities(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "DeleteItem", MethodType.Operation, Publish.Primary);
            var entityListItem = entities.First();
            extractionsContext.EntityListOperations.DeleteItem(entityListItem);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        #endregion

        #region Validation

        /// <summary>
        /// This example demonstrates possible validation errors.
        /// </summary>
        [Example("Append Identifiers: Validation Errors")]
        public void ValidationErrors()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "Entity List Name" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers", MethodType.Operation, Publish.Secondary);
            IEnumerable<EntityIdentifier> appendIdentifiers = new[] {new EntityIdentifier
				{
					Identifier = "@@@",
					IdentifierType = IdentifierType.OrgId,
					UserDefinedIdentifier = "UserIdent4A"
				}};
            var result = extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, false, false);
            if (result.ValidationResult.Messages.Any())
            {
                // Do something…
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Get");
            var createdEntityList = extractionsContext.EntityListOperations.Get(entityList.ListId);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(createdEntityList);
            Status.EndNotify(extractionsContext);
        }

        /// <summary>
        /// This example demonstrates how to Validate Entity Identifiers.
        /// </summary>
        [Example("ValidateIdentifiers")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void ValidateIdentifiers()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();
            Status.Notify(extractionsContext, "EntityList", "ValidateIdentifiers", MethodType.Operation, Publish.Primary);
            extractionsContext.EntityListOperations.ValidateIdentifiers(
                new[]
                {
                    new EntityIdentifier {Identifier = "12286", IdentifierType = IdentifierType.OrgId}
                }
                , true
                , false
                );
            Status.EndNotify(extractionsContext);
        }


        /// <summary>
        /// This example demonstrates how to Validate Entity Identifiers with Options.
        /// </summary>
        [Example("ValidateIdentifiersWithOptions")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(EntityList))]
        public void ValidateIdentifiersWithOptions()
        {

            var extractionsContext = ContextHelper.CreateExtractionsContext();
            EntityValidationOptions options = new EntityValidationOptions { AllowDuplicateInstruments = true, AllowUnmanagedOrUnverifiedEntities = true, IncludeParentAndUltimateParent = false };

            Status.Notify(extractionsContext, "EntityList", "ValidateIdentifiersWitOptions", MethodType.Operation, Publish.Primary);
            extractionsContext.EntityListOperations.ValidateIdentifiersWitOptions(
                new[]
                {
                    new EntityIdentifier {Identifier = "12286", IdentifierType = IdentifierType.OrgId}
                }
                , options
                );
            Status.EndNotify(extractionsContext);

        }

        #endregion

        #region Export / Import

        [Example("Export: Xml")]
        public void ExportXml()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "ExportXml" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiers = new[] {new EntityIdentifier
				{
					Identifier = "12286",
					IdentifierType = IdentifierType.OrgId,
					UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Export", MethodType.Operation, Publish.Primary);
            var exportBytes = extractionsContext.EntityListOperations.Export(
                entityList.ListId, EntityListExportFormat.Xml);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(entityList);
            Status.EndNotify(extractionsContext);
            
        }

        [Example("Import: Xml")]
        public void ImportXml()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "ImportXml" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiers = new[] {new EntityIdentifier
				{
					Identifier = "12286",
					IdentifierType = IdentifierType.OrgId,
					UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Export", MethodType.Operation, Publish.Secondary);
            var exportBytes = extractionsContext.EntityListOperations.Export(
                entityList.ListId, EntityListExportFormat.Xml);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "ImportFromBytes", MethodType.Operation, Publish.Primary);
            var importBytes = Encoding.UTF8.GetBytes(exportString);
            var listName = "ImportXmlNew" + Suffix.New() + ".xml";
            extractionsContext.EntityListOperations.ImportFromBytes(
                listName, 
                importBytes, 
                new EntityListImportSettings
                {
                    AllowDuplicateEntities = false, 
                    ImportUserEntities = false,
                    PredefinedAction = ImportActionEnum.Append,
                    PredefinedName = listName
                });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetByName");
            //When a list is imported, we don't know what the id of the report template is and therefore
            //we need to retrieve the report template by name.
            var newList = extractionsContext.EntityListOperations.GetByName(listName);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(newList);
            Status.EndNotify(extractionsContext);
        }

        [Example("Export: Csv")]
        public void ExportCsv()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "Export" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiers = new[] {new EntityIdentifier
				{
					Identifier = "12286",
					IdentifierType = IdentifierType.OrgId,
					UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Export", MethodType.Operation, Publish.Primary);
            var exportBytes = extractionsContext.EntityListOperations.Export(
                entityList.ListId, EntityListExportFormat.Csv);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(entityList);
            Status.EndNotify(extractionsContext);

        }

        [Example("Import: Csv")]
        public void ImportCsv()
        {
            /* It is typically not necessary to import and export as XML for an API. This capability is provided only to support the GUI as
             * well as backward compatability. */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "EntityList", "Create");
            var entityList = new EntityList { Name = "ImportXml" + Suffix.New() };
            extractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "AppendIdentifiers");
            IEnumerable<EntityIdentifier> appendIdentifiers = new[] {new EntityIdentifier
				{
					Identifier = "12286",
					IdentifierType = IdentifierType.OrgId,
					UserDefinedIdentifier = "UserIdent4A"
				}};
            extractionsContext.EntityListOperations.AppendIdentifiers(entityList, appendIdentifiers, false, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Export", MethodType.Operation, Publish.Secondary);
            var exportBytes = extractionsContext.EntityListOperations.Export(
                entityList.ListId, EntityListExportFormat.Csv);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(entityList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "ImportFromBytes", MethodType.Operation, Publish.Primary);
            var importBytes = Encoding.UTF8.GetBytes(exportString);
            var listName = "ImportXmlNew" + Suffix.New() + ".csv";
            extractionsContext.EntityListOperations.ImportFromBytes(
                listName,
                importBytes,
                new EntityListImportSettings
                {
                    AllowDuplicateEntities = false,
                    ImportUserEntities = false,
                    PredefinedAction = ImportActionEnum.Append,
                    PredefinedName = listName
                });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "GetByName");
            //When a list is imported, we don't know what the id of the report template is and therefore
            //we need to retrieve the report template by name.
            var newList = extractionsContext.EntityListOperations.GetByName(listName);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "EntityList", "Delete");
            extractionsContext.EntityListOperations.Delete(newList);
            Status.EndNotify(extractionsContext);
        }

        #endregion

    }

}
