﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using Dss.Api.Examples.Properties;

namespace Dss.Api.Examples.Internal
{
    public class ExampleExtractor
    {
        private readonly CodeSettings _codeSettings;
        private static readonly List<string> HeadersToExclude = new List<string>
        {
            "X-Client-Session-Id",
            "X-Client-Version",
            "Pragma",
            "OData-Version",
            "X-App-Id",
            "X-Request-Execution-Correlation-Id",
            "X-App-Version",
            "Cache-Control",
            "Set-Cookie",
            "Date",
            "Server",
            "X-Powered-By",
            "Set-Cookie",
            "X-Sample-Mode",
            "OData-MaxVersion",
            "Accept",
            "User-Agent",
            "Accept-Charset"
        };

        private static readonly Regex SuffixRegex = new Regex(@"\.suffix\.[A-Z0-9]{3,10}");
        private static readonly Regex UsernameRegex = new Regex(@"\""Username\""\s*:\s*""[^""]*""");
        private static readonly Regex PasswordRegex = new Regex(@"\""Password\""\s*:\s*""[^""]*""");

        private static readonly Action EmptyDelegate = delegate { };
        private const string IndentString = "    ";
        private readonly ProgressBar _exportProgressControl;
        private readonly TextBlock _exampleNameControl;
        private readonly bool _truncateResults;
        private readonly Image _redImageControl;
        private readonly TextBlock _failedCountControl;

        #region Ctors

        public ExampleExtractor(ProgressBar progressControl, TextBlock exampleNameControl, Image redImageControl, TextBlock failedCountControl, bool sanitizeUrls, bool truncateResults)
        {
            _exportProgressControl = progressControl;
            _exampleNameControl = exampleNameControl;
            _truncateResults = truncateResults;
            _redImageControl = redImageControl;
            _failedCountControl = failedCountControl;
             
            _codeSettings = new CodeSettings 
            {
                ProductionUrl = "https://selectapi.datascope.refinitiv.com/restapi/v1/",
                SanitizeUrls = sanitizeUrls
            };
        }

        #endregion

        #region HTTP Request / Response

        public string GetHttpRequestSnippet(ExampleStep step)
        {
            if (step.Result == null)
                return string.Empty;

            var builder = new StringBuilder();

            AppendUrl(builder, step);
            AppendHeaders(builder, step.Result.RequestHeaders);
            AppendBody(builder, step.Result.RequestBody, step.Result.IsText);

            return builder.ToString();
        }

        public string GetHttpResponseSnippet(ExampleStep step)
        {
            if (step.Result == null)
                return string.Empty;

            var builder = new StringBuilder();

            builder.AppendFormat("HTTP/1.1 {0} {1}", (int)step.Result.Status, step.Result.Status);
            builder.AppendLine();

            AppendHeaders(builder, step.Result.ResponseHeaders);
            AppendBody(builder, step.Result.ResponseBody, step.Result.IsText);

            return builder.ToString();
        }

        #endregion

        #region C# Snippet

        public CodeSnippet GetCSharpSnippet(CodeSnippet fullExample, ExampleStep step)
        {
            if (fullExample == null || fullExample.Code == null)
                return null;
            //Split the code into multiple lines
            var lines = Regex.Split(fullExample.Code, "\r\n|\r|\n");

            //Find the start and end line of the snippet
            var startLine = GetStartLine(lines, Convert.ToInt32(FixStepNumber(step.StepNumber)));
            if (startLine == null)
                return null;

            var endLine = Math.Min(lines.Length - 1, GetEndLine(lines, startLine.Value) ?? lines.Length);

            //Get the snippet
            var code = new StringBuilder();
            for (var i = startLine.Value; i <= endLine; i++)
                code.AppendLine(lines[i]);

            //Create the snippet
            var result = new CodeSnippet(_codeSettings, code.ToString().Trim('\r', '\n'));
            var cleanedCode = result.CleanCode(false, true);

            //Add the output to the snippet
            if (step.Result != null && !String.IsNullOrWhiteSpace(step.Result.Output))
            {
                cleanedCode += "\r\n\r\n/* === OUTPUT === *\\\r\n";
                cleanedCode += step.Result.Output.Trim() + "\r\n";
                cleanedCode += "\\* ============== */ \r\n";
            }

            //Add the line numbers
            return new CodeSnippet(
                _codeSettings,
                cleanedCode,
                AdjustLineNumber(lines, startLine.Value, true) + fullExample.PrependendLineCount,
                AdjustLineNumber(lines, endLine, true) + fullExample.PrependendLineCount);
        }

        // Remove any characters fro step number...
        private string FixStepNumber(string stepNumber)
        {
            return stepNumber.Where(individualChar => !Char.IsLetter(individualChar)).Aggregate(string.Empty, (current, individualChar) => current + individualChar);
        }

        #endregion

        #region C# Full

        public CodeSnippet GetCSharpFull(Example example)
        {
            //Parse the example name
            var exampleName = example.MethodName;
            var exampleParts = exampleName.Split('.');
            var exampleMethod = exampleParts.Last();
            //var exampleClass = exampleParts.Reverse().Skip(1).Reverse().Last();
            var exampleNamespace = String.Join(".", exampleParts.Reverse().Skip(2).Reverse());

            //Examine the files in all directories (but ignore the \bin and \obj directories)
            var files = Directory.GetFiles("..\\..\\", "*.cs", SearchOption.AllDirectories).Where(x => !x.Contains("\\obj\\") && !x.Contains("\\bin\\")).ToArray();

            var nsRegex = new Regex(@"namespace\s*(?<namespace>[\w.]*)");
            foreach (var file in files)
            {
                //Get the file contents
                var fileContents = File.ReadAllText(file);

                //Get the namespace
                var nsMatch = nsRegex.Match(fileContents);
                var ns = nsMatch.Success ? nsMatch.Groups["namespace"].Value : "";
                ns = ns.Replace("Dss.Api.Examples.", ""); //Example name has this part removed

                //Namespace must match
                if (exampleNamespace != ns)
                    continue;

                //Find the method
                var location = fileContents.IndexOf("public void " + exampleMethod + "(", StringComparison.InvariantCultureIgnoreCase);
                if (location >= 0)
                    return new CodeSnippet(_codeSettings, GetCodeBetweenBraces(fileContents.Substring(location)));
            }

            throw new ApplicationException(string.Format("Unable to locate method '{0}'", "public void " + exampleMethod));
        }

        #endregion

        #region Export All

        public async Task ExportAllExamples()
        {
            // DSS-19252:  Delete all previous example files for Export All
            DeleteExportTarget(GetExportTargetPath());

            var failureCount = 0;

            // Establish the log file...
            using (var streamWriter = EstablishLogFile())
            {
                await CaptureEventsForExport(async () =>
                {
                    var areas = ExampleListings.GetExamples().ToList();

                    foreach (var area in areas)
                    {
                        _exportProgressControl.Value = 0;
                        _exportProgressControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

                        var stepSize = 100/area.Examples.Count();

                        foreach (var example in area.Examples)
                        {
                            _exampleNameControl.Text = string.Format("Running and Exporting - {0}", example.Name);
                            _exampleNameControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

                            var exampleSucceeded = await example.Run(streamWriter);

                            _exportProgressControl.Value += stepSize;
                            _exportProgressControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

                            if (!exampleSucceeded)
                                failureCount++;
                        }
                    }
                });
            }

            _exportProgressControl.Value = 100;
            _exportProgressControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

            _exampleNameControl.Text = "Export complete...";
            _exampleNameControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

            if (failureCount > 0)
            {
                ShowFailureInformation(failureCount);
            }
        }

        #endregion

        #region Export Area

        public async Task ExportAreaExamples(ExampleArea area)
        {
            _exportProgressControl.Value = 0;
            _exportProgressControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

            var targetpath = GetExportTargetPath();

            // DSS-19252:
            // Create a unique list of example names and delete all
            // associated files for each example only one time before
            // we create any new example files.

            var exampleNameList = area.Examples.Select(example => example.Name).ToList();

            var stepSize = 100 / area.Examples.Count();

            foreach (var name in exampleNameList.Distinct().ToList())
            {
                DeleteOldExampleFiles(targetpath, name);
            }

            var failureCount = 0;

            // Establish the log file...
            using (var streamWriter = EstablishLogFile())
            {
                await CaptureEventsForExport(async () =>
                {
                    foreach (var example in area.Examples)
                    {
                        _exampleNameControl.Text = string.Format("Running and Exporting - {0}", example.Name);
                        _exampleNameControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

                        var exampleSucceeded = await example.Run(streamWriter);

                        _exportProgressControl.Value += stepSize;
                        _exportProgressControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

                        if (!exampleSucceeded)
                            failureCount++;
                    }
                });
            }

            _exportProgressControl.Value = 100;
            _exportProgressControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

            _exampleNameControl.Text = "Export complete...";
            _exampleNameControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

            if (failureCount > 0)
            {
                ShowFailureInformation(failureCount);
            }
        }

        #endregion

        #region Export Single Example

        public async Task ExportExample(Example example)
        {
            _exportProgressControl.Value = 0;
            _exportProgressControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

            // Delete all previous files that has a matching example  
            // name within the full filename of the target directory
            DeleteOldExampleFiles(GetExportTargetPath(), example.Name);

            var exampleSucceeded = false;

            // Establish the log file...
            using (var streamWriter = EstablishLogFile())
            {
                await CaptureEventsForExport(async () =>
                {
                    _exampleNameControl.Text = string.Format("Running and Exporting - {0}", example.Name);
                    _exampleNameControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

                    exampleSucceeded = await example.Run(streamWriter);
                });
            }

            _exportProgressControl.Value = 100;
            _exportProgressControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

            _exampleNameControl.Text = "Export complete...";
            _exampleNameControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

            if (!exampleSucceeded)
            {
                ShowFailureInformation(1);
            }
        }

        #endregion

        #region Private Helpers

        private void ShowFailureInformation(int numberOfFailures)
        {
            _redImageControl.Visibility = Visibility.Visible;
            _redImageControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);

            _failedCountControl.Visibility = Visibility.Visible;
            _failedCountControl.Text = string.Format("{0} {1} failed", numberOfFailures, numberOfFailures > 1 ? "examples" : "example");
            _failedCountControl.Dispatcher.Invoke(DispatcherPriority.Render, EmptyDelegate);
        }

        private static StreamWriter EstablishLogFile()
        {
            var appDirectory = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            Debug.Assert(appDirectory != null, "appDirectory != null");

            return new StreamWriter(Path.Combine(appDirectory, "ExportLog.txt"));
        }

        private void AppendUrl(StringBuilder builder, ExampleStep step)
        {
            if (step.Result == null)
                return;

            //The URI can be null if the older OData methods are used
            if (step.Result.Uri == null)
                return;

            var url = CleanHttp(step.Result.Uri.ToString());

            builder.AppendFormat("{0} {1} HTTP/1.1", step.Result.Verb, url);
            builder.AppendLine();
        }

        private void AppendHeaders(StringBuilder builder, Dictionary<string, string> headers)
        {
            if (headers == null)
                return;
            foreach (var header in headers)
            {
                if (HeadersToExclude.Contains(header.Key))
                    continue;
                var headerValue = String.Join("; ", header.Value);
                if (header.Key == "Authorization")
                    headerValue = "Token <your_auth_token_goes_here>";
                //Remove the "wait" from the header since customers are actually using the values in the samples.
                if (headerValue.StartsWith("Prefer"))
                {
                    var regex = new Regex(@"Prefer: .*((?-imsnx:,|)\s*wait=[0-9]*)");
                    headerValue = regex.Replace(headerValue, "");
                    if (headerValue.Trim() == "Prefer:")
                        continue;
                }
                builder.AppendFormat("{0}: {1}", header.Key, CleanHttp(headerValue));
                builder.AppendLine();
            }
        }

        private void AppendBody(StringBuilder builder, string responseBody, bool isText)
        {
            if (responseBody == null)
                return;
            //Remove suffix
            responseBody = CleanHttp(responseBody);
            //Format
            builder.AppendLine((isText ? responseBody : FormatJson(responseBody)).TrimEnd('\r', '\n'));
        }

        private static bool IsJson(string str)
        {
            if (String.IsNullOrWhiteSpace(str))
                return false;
            str = str.TrimStart();
            return (str.StartsWith("{") || str.StartsWith("["));
        }

        private string CleanHttp(string input)
        {
            input = SuffixRegex.Replace(input, "");
            if (_codeSettings.SanitizeUrls)
                input = Regex.Replace(input, Settings.Default.ServiceUri, _codeSettings.ProductionUrl, RegexOptions.IgnoreCase);
            input = UsernameRegex.Replace(input, "\"Username\": \"<Your Username>\"");
            input = PasswordRegex.Replace(input, "\"Password\": \"<Your Password>\"");
            return input;
        }

        private string FormatJson(string str)
        {
            if (!IsJson(str))
                return str;
            var indent = 0;
            var quoted = false;
            var sb = new StringBuilder();
            for (var i = 0; i < str.Length; i++)
            {
                var ch = str[i];
                switch (ch)
                {
                    case '{':
                    case '[':
                        sb.Append(ch);
                        if (!quoted)
                        {
                            sb.AppendLine();
                            Enumerable.Range(0, ++indent).ForEach(item => sb.Append(IndentString));
                        }
                        break;
                    case '}':
                    case ']':
                        if (!quoted)
                        {
                            sb.AppendLine();
                            Enumerable.Range(0, --indent).ForEach(item => sb.Append(IndentString));
                        }
                        sb.Append(ch);
                        break;
                    case '"':
                        sb.Append(ch);
                        bool escaped = false;
                        var index = i;
                        while (index > 0 && str[--index] == '\\')
                            escaped = !escaped;
                        if (!escaped)
                            quoted = !quoted;
                        break;
                    case ',':
                        sb.Append(ch);
                        if (!quoted)
                        {
                            sb.AppendLine();
                            Enumerable.Range(0, indent).ForEach(item => sb.Append(IndentString));
                        }
                        break;
                    case ':':
                        sb.Append(ch);
                        if (!quoted)
                            sb.Append(" ");
                        break;
                    default:
                        sb.Append(ch);
                        break;
                }
                //Special handling of new-line characters (\r\n)
                if (ch == 'n' && i > 3 && str[i - 1] == '\\' && str[i - 2] == 'r' && str[i - 3] == '\\')
                {
                    if (i == str.Length - 1 || str[i + 1] != '"') //If the next char is a quote, don't bother with a new line.
                    {
                        sb.AppendLine();
                        Enumerable.Range(0, indent).ForEach(item => sb.Append(IndentString));
                        sb.Append(" "); //Keep the alignment to the quote char.
                    }
                }
            }

            return _truncateResults ? TruncateResults(sb.ToString()) : sb.ToString();
        }

        private static string TruncateResults(string jsonResuls)
        {
            var truncatedResults = string.Empty;

            var insideBracket = false;
            var bracketCount = 0;
            var singleValueCount = 0;
            const string leftBracket = "[";
            const string rightBracket = "]";

            // Split into individual lines...
            var lines = jsonResuls.Split('\n');

            // Process the collection of lines...
            foreach (var line in lines)
            {
                // Did we find a bracket?
                // There can be multiple depths (collections within collections)...
                if (line.Contains(leftBracket))
                {
                    bracketCount++;

                    insideBracket = true;
                }

                // Are we inside of a bracket?
                if (insideBracket)
                {
                    if (line.Contains("{\r") && bracketCount == 1)
                    {
                        singleValueCount++; // Truncate OUTER bracket collections only...

                        // Do we have more than 4 values?
                        // If yes, we need to fix the last brace...
                        if (singleValueCount == 5)
                        {
                            // Looking for this sequence at the end of the truncated results "},\r\n"
                            var position = truncatedResults.IndexOf(',', truncatedResults.Length - 3);

                            if (position > 0)
                            {
                                // We found it so make it look like this "}\r\n"
                                // Remove the comma...
                                truncatedResults = truncatedResults.Remove(position, 1);

                                // Output the truncated verbage...
                                truncatedResults += IndentString + IndentString + "•••" + "\r\n";
                            }
                        }
                    }

                    // Continue to output until count is greater than 4, then truncate...
                    if (singleValueCount < 5)
                        truncatedResults += line;

                    // Are we at a right bracket?
                    if (line.Contains(rightBracket))
                    {
                        bracketCount--;

                        // Are we back to an OUTER bracket?
                        if (bracketCount > 1)
                            continue; // No

                        if (bracketCount == 0)
                        {
                            if (singleValueCount > 4)
                            {
                                truncatedResults += line; // Capture the bracket line...
                                singleValueCount = 0;
                            }

                            insideBracket = false;
                        }
                    }

                    continue;
                }

                // Handles everything outside of brackets...
                truncatedResults += line;
            }

            return truncatedResults;
        }

        private static int? GetStartLine(IEnumerable<string> lines, int stepNumber)
        {
            const string startPattern = @"Status\.Notify";
            var regex = new Regex(startPattern, RegexOptions.Compiled | RegexOptions.CultureInvariant);
            var lineNo = 0;
            var foundMatchCounter = 0;

            foreach (var line in lines)
            {
                lineNo++;

                //Find the corresponding Status.Notify line based on step number
                if (!regex.IsMatch(line))
                {
                    continue;
                }

                foundMatchCounter++;

                if (foundMatchCounter < stepNumber)
                    continue;

                return lineNo;
            }

            return null;
        }

        private static int? GetEndLine(string[] lines, int startLine)
        {
            const string endPattern = @"Status\.EndNotify";
            var regex = new Regex(endPattern, RegexOptions.Compiled | RegexOptions.CultureInvariant);
            for (var lineNo = startLine; lineNo < lines.Length; lineNo++)
            {
                var line = lines[lineNo];
                if (regex.IsMatch(line))
                    return lineNo - 1;
            }
            return null;
        }

        private static int AdjustLineNumber(string[] lines, int value, bool removeDescription)
        {
            var trueLineNo = 0;
            var inComment = false;
            for (var i = 0; i <= value; i++)
                if (CodeSnippet.ShouldIncludeLine(lines[i], removeDescription, ref inComment))
                    trueLineNo++;
            return trueLineNo - 1;
        }

        private static string GetCodeBetweenBraces(string code)
        {
            var snippetStart = 0;
            var snippetEnd = 0;
            var braceCount = -1;
            while (snippetEnd++ < code.Length && (braceCount != 0))
            {
                if (code[snippetEnd] == '{')
                {
                    if (braceCount == -1)
                    {
                        snippetStart = snippetEnd + 1;
                        braceCount = 0;
                    }
                    braceCount++;
                }
                if (code[snippetEnd] == '}')
                {
                    if (braceCount == -1)
                    {
                        braceCount = 0;
                        snippetStart = snippetEnd + 1;
                    }
                    braceCount--;
                }
            }
            if (braceCount == 0)
                return code.Substring(snippetStart, snippetEnd - snippetStart - 1);
            return null;
        }

        private string GetExportTargetPath()
        {
            // Return the directory to export files to
            var path = Directory.GetCurrentDirectory() + @"\..\..\..\..\..\Api\RestApi.Help\Content\help-files";

            // The condition of the directory not existing will always occur if 
            // ExportAll is the selection since the directory will be removed
            // in that case.

            if (!Directory.Exists(path))
            {
                try
                {
                    Directory.CreateDirectory(path);
                }
                catch (Exception)
                {
                    path = "help-files";   // This was the original default target path passed in
                    // TODO: Determine: Handle this here, when removing files, or when writing files
                }
            }

            return (path);
        }

        private void DeleteExportTarget(string pathname)
        {
            if ((File.GetAttributes(pathname) & FileAttributes.Directory) > 0)
            {
                // Nuke the entire directory, removing all prior example data files.
                Directory.Delete(pathname, true);
            }
            else
            {
                File.Delete(pathname);
            }
        }

        private string RepairExampleFileName(string origName)
        {
            return(origName.Replace(": ", " - "));
        }

        private void DeleteOldExampleFiles(string pathname, string exampleName)
        {
            // We have multiple patterns here because of different naming practices.
            // Also, some examples do not have hyphen or a space before the hyphen
            // Order is important...
            var pattern1 = "." + exampleName.Replace(": ", " - ") + ".";
            var pattern2 = "." + exampleName.Replace(": ", "- ") + ".";

            string[] listFiles = Directory.GetFiles(pathname, "ctx.*");

            foreach (var f in listFiles)
            {
                var fn = Path.GetFileName(f);

                if (fn == null) continue;
                    
                if (fn.Contains(pattern1) || fn.Contains(pattern2))
                {
                    DeleteExportTarget(f);

                    Debug.WriteLine(fn);
                }
            }
        }

        private async Task CaptureEventsForExport(Func<Task> method)
        {
            Status.StatusUpdated += StatusOnStatusUpdated;

            try
            {
                await method();
            }
            finally
            {
                Status.StatusUpdated -= StatusOnStatusUpdated;
            }
        }

        private void StatusOnStatusUpdated(object sender, StatusUpdateEventArgs statusUpdate)
        {
            var step = statusUpdate.Step;
            var targetPath = GetExportTargetPath();

            if (step.Publish != Publish.None)
            {
                //Step Examples: Determine the base filename
                var fileName = String.Format("ctx.{0}.ent.{1}.{2}.{3}.sce.{4}.{5}{6}",
                                             step.ContextName,
                                             step.EntityName,
                                             GetExampleType(step.MethodType),
                                             step.MethodName,
                                             step.Example.Name,
                                             step.StepNumber,
                                             step.Publish == Publish.Primary ? ".primary" : "");

                var fullPath = Path.Combine(targetPath, RepairExampleFileName(fileName));

                //Step Examples: Output
                WriteExampleFiles(fullPath, statusUpdate.Step);
            }

            //End-to-End Examples: Filename
            if (step.Example.EndToEndExamples == null) return;

            foreach (var endToEndExample in step.Example.EndToEndExamples)
            {
                //Step Examples: Determine the base filename
                var fileName = String.Format("ctx.{0}{1}.sce.{2}.{3}",
                    endToEndExample.ContextType == null ? "Unknown" : endToEndExample.ContextType.Name.Replace("Context", ""),
                    endToEndExample.EntityType == null ? "" : ".ent." + endToEndExample.EntityType.Name,
                    step.Example.Name,
                    step.StepNumber);

                var fullPath = Path.Combine(targetPath, RepairExampleFileName(fileName));

                //Step Examples: Output
                WriteExampleFiles(fullPath, step);
            }
        }

        private void WriteExampleFiles(string fullPath, ExampleStep step)
        {
            File.WriteAllText(fullPath + ".httpreq", GetHttpRequestSnippet(step));
            File.WriteAllText(fullPath + ".httpresp", GetHttpResponseSnippet(step));

            var fullCode = GetCSharpFull(step.Example);
            File.WriteAllText(fullPath + ".csfull", fullCode.CleanCode(true, false));

            if (!String.IsNullOrWhiteSpace(fullCode.Description))
                File.WriteAllText(fullPath + ".description", fullCode.Description);
            
            File.WriteAllText(fullPath + ".cssnippet", GetCSharpSnippet(fullCode, step).Code);
        }

        private static string GetExampleType(MethodType type)
        {
            switch (type)
            {
                case MethodType.Association:
                    return "assoc";
                case MethodType.Stream:
                    return "strm";
                default:
                    return "op";
            }
        }

        #endregion
    }

    internal static class Extensions
    {
        public static void ForEach<T>(this IEnumerable<T> ie, Action<T> action)
        {
            foreach (var i in ie)
            {
                action(i);
            }
        }
    }
}
