﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Runtime.ExceptionServices;
using System.Text;
using DataScope.Select.Api;
using DataScope.Select.Api.Core;

namespace Dss.Api.Examples.Internal
{
    public static class Status
    {
        public static volatile Example CurrentExample;
        public static volatile ExampleArea CurrentArea;
        public static volatile Example RunningExample;

        public static event StatusUpdateEventHandler StatusUpdated;
        private static IDssDataServiceContext _lastContext;
        private static Example _lastExample;
        private static int _stepNumber;
        private static string _contextName;
        private static string _entityName;
        private static string _methodName;
        private static MethodType _methodType;
        private static Publish _publish;
        private static StringBuilder _output;
        private static readonly List<ResponseCapture> _requestEvents = new List<ResponseCapture>();
        private static Exception _error;
        private static StackTrace _stackTrace;
        private static Stopwatch _stopwatch;

        public static void Notify(
            IDssDataServiceContext context, string entity, string method, MethodType methodType = MethodType.Operation, Publish publish = Publish.None)
        {
            _contextName = context.GetType().Name.Replace("Context", "");
            _entityName = entity;
            _methodName = method;
            _methodType = methodType;
            _publish = publish;
            //If this is a new example, then reset the step number which will clear the existing steps
            _stepNumber = RunningExample == _lastExample ? ++_stepNumber : 1;
            _lastExample = RunningExample;
            _lastContext = context; //Required for situations where authentication fails
            _output = new StringBuilder();
            _stackTrace = new StackTrace();
            if (_stepNumber == 1)
            {
                RunningExample.ClearSteps();
                _stopwatch = new Stopwatch();
                _stopwatch.Start();
            }
            context.ResponseReceived += context_ResponseReceived;
        }

        public static void EndNotify(IDssDataServiceContext context, string responseBody = null)
        {
            context.ResponseReceived -= context_ResponseReceived;
            //Don't output more than two sub steps because all the steps except for the last step are the same.
            if (_requestEvents.Count > 2)
            {
                var first = _requestEvents.First();
                var last = _requestEvents.Last();
                _requestEvents.Clear();
                _requestEvents.Add(first);
                _requestEvents.Add(last);
            }
            var eventCount = _requestEvents.Count;
            var requestEvents = _requestEvents.ToArray();
            var substep = 0;
            foreach (var requestEvent in requestEvents)
            {
                var step = ExampleStep.Create(
                    RunningExample, requestEvent, _error, _stackTrace,
                    _stopwatch.Elapsed.ToString(),
                    _stepNumber.ToString(CultureInfo.InvariantCulture) + (eventCount == 1 ? "" : Convert.ToString((Char)('a' + substep++))),
                    _contextName, _entityName, _methodName, _methodType, _publish,
                    _output == null ? null : _output.ToString(), responseBody);
                if (StatusUpdated != null)
                    StatusUpdated.Invoke(null, new StatusUpdateEventArgs(null, step, false));
                RunningExample.AddStep(step);
            }
            _requestEvents.Clear();
            _output = null;
            _error = null;
        }

        public static void ResetStepNumber()
        {
            _stepNumber = 0;
        }

        public static void Error(Exception error)
        {
            _error = error;
            //It is possible for the _last context to be null if there was an invalid username/password problem and context
            //could not be created
            if (_lastExample != null)
                EndNotify(_lastContext);
            else
                ExceptionDispatchInfo.Capture(error).Throw();
        }

        public static void WriteLine(object value)
        {
            WriteLine(value == null ? "" : value.ToString());
        }

        public static void WriteLine(string text)
        {
            if (_output != null)          // if null, no way the output will ever find its way to the app window
                _output.AppendLine(text);
            
            Debug.WriteLine(text);
        }

        public static void WriteLine(string format, params object[] args)
        {
            var text = string.Format(format, args);
            if (_output != null)
                _output.AppendLine(text);
            
            Debug.WriteLine(text);
        }

        static void context_ResponseReceived(object sender, HttpRequestEventArgs args)
        {
            _requestEvents.Add(new ResponseCapture { EventArgs = args });
        }

    }

    public delegate void StatusUpdateEventHandler(object sender, StatusUpdateEventArgs args);

    public class StatusUpdateEventArgs : EventArgs
    {
        public bool Cleanup { get; private set; }
        public object Sender { get; private set; }
        public ExampleStep Step { get; private set; }

        public StatusUpdateEventArgs(object sender, ExampleStep step, bool cleanup)
        {
            Cleanup = cleanup;
            Sender = sender;
            Step = step;
        }
    }

    public class ResponseCapture
    {
        public HttpRequestEventArgs EventArgs { get; set; }
    }
}
