﻿using Dss.Api.Examples.Internal;
using System;
using System.IO;
using System.Linq;
using DataScope.Select.Api;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.ExtractionRequests;
using DataScope.Select.Api.Extractions.ReportExtractions;
using DataScope.Select.Api.Extractions.ReportTemplates;
using DataScope.Select.Api.Extractions.Schedules;
using DataScope.Select.Api.Extractions.SubjectLists;
using DataScope.Select.Api.Search;
using DataScope.Select.Core.RestApi;

namespace Dss.Api.Examples.TickHistory
{

    [Example("Tick History")]
    public class TickHistoryExamples
    {

        protected ExtractionsContext ExtractionsContext { get; set; }
        protected SearchContext SearchContext { get; set; }

        public TickHistoryExamples()
        {
            ExtractionsContext = ContextHelper.CreateExtractionsContext();
            SearchContext = ContextHelper.CreateSearchContext();
        }

        [Example("Expand a Chain RIC Over Date Range")]
        public void Number2_ExpandAChainRic()
        {
            // Setup
            Status.Notify(SearchContext, null, "HistoricalChainResolution", MethodType.Operation, Publish.Primary);

            var dateRange = DateTimeRange.Create(
                new DateTimeOffset(2008, 01, 01, 0, 0, 0, TimeSpan.Zero),
                new DateTimeOffset(2016, 01, 01, 0, 0, 0, TimeSpan.Zero));
            var request = new HistoricalChainResolutionRequest
            {
                ChainRics = new[] { "0#HO:" },
                Range = dateRange
            };

            // Resolve constituents
            var results = SearchContext.HistoricalChainResolution(request);

            //Output
            var chain = results.First();
            Status.WriteLine("Chain Identifier: {0} {1}, Constituents: {2}", chain.IdentifierType, chain.Identifier, chain.Constituents.Count());
            foreach (var constituent in chain.Constituents)
                Status.WriteLine("{0} {1}", constituent.IdentifierType, constituent.Identifier);

            Status.EndNotify(SearchContext);
        }

        [Example("Import a list of ISINs, CUSIPs or SEDOLs")]
        //Albert [8/26] There are four components to this one. 1. Importing a list (.csv or whatever format) into the DSS platform for use 
        //through the API, 2. performing a historical instrument look up to translate these secondary identifiers to RICs. 3. Filtering the
        //results by a criteria like NYS exchange, then 4. showing how to apply  the results to a search care of InstrumentList or OnDemand
        //extraction.
        public void Number4_ImportAList()
        {
            Status.Notify(SearchContext, null, "HistoricalSearch", MethodType.Operation, Publish.Secondary);
            var dateRange = DateTimeRange.Create(
                new DateTimeOffset(2008, 01, 01, 0, 0, 0, TimeSpan.Zero),
                new DateTimeOffset(2008, 02, 01, 0, 0, 0, TimeSpan.Zero));

            //Import a list of instruments
            //Simulate loading the contents from a file. You can have multiple rows separated by carriage return and a line feed in the file.
            //To load from a file use: csvContents = File.ReadAllLines(<your_file_path>);
            var csvContents = new []
            {
                "US0378331005,ISIN", //Apple
                "17275R102,CUSIP", //Cisco
                "2090571,SEDOL", //Verizon 
                "US5801351017,ISIN", //mc donalds 
                "594918104,CUSIP", //Microsoft
                "2270726,SEDOL" //Disney 
            };

            //Create a list of instruments from the imported lines
            var instruments = csvContents
                .Select(x => x.Trim().Split(',')) //Split by comma
                .Where(x => x.Count() == 2) //Ignore bad lines
                .Select(x => InstrumentIdentifier.Create((IdentifierType)Enum.Parse(typeof(IdentifierType), x[1], true), x[0]));

            //Search: This will perform a separate search for each instrument. If you have some familiarity with threading
            //and asynchronous programming, you can submit these requests in parallel.
            var searchResults = instruments
                .SelectMany(instrument=>
                    SearchContext.HistoricalSearch(
                        HistoricalSearchRequest.Create(
                            instrument.Identifier, instrument.IdentifierType, dateRange, HistoricalResultsBy.Ric, null)))
                .ToArray(); //Force the searches to run now

            //Output
            foreach (var result in searchResults.Take(10))
                Status.WriteLine("{0}, {1}, {2}, {3}", result.Identifier, result.IdentifierType, result.FirstDate, result.LastDate);
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "ReferenceHistory", MethodType.Operation, Publish.Secondary);
            var rics = searchResults
                .Where(x => x.IdentifierType == IdentifierType.Ric) //All results will be by RIC
                .Select(x => x.Identifier)
                .ToArray();
            
            //Get the history
            var history = SearchContext.ReferenceHistory(ReferenceHistoryRequest.Create(rics, dateRange, HistoricalResultsBy.Ric));

            //Output
            foreach (var result in history.Take(10))
                Status.WriteLine("{0}, {1}", result.Ric, result.HistoryEvents.Count());
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "GetHistoricalExchanges", MethodType.Operation, Publish.Secondary);
            //Get the exchange code for NYS
            var exchanges = SearchContext.GetHistoricalExchanges();

            //Output
            foreach (var result in exchanges.Where(x=>x.Name.Contains("New York")).Take(10))
                Status.WriteLine("{0}, {1}", result.Code, result.Name);
            Status.EndNotify(SearchContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "Create", MethodType.Operation, Publish.Secondary);
            //Filter the list to only those in NYS
            var nysRics = history
                .Where(x => x.HistoryEvents.Any(he => he.Field == "Exchange" && he.Value == "2")) //2, [NYS] New York Stock Exchange
                .Select(x => InstrumentIdentifier.Create(IdentifierType.Ric, x.Ric));

            //Create the instrument list
            var instrumentList = ExtractionsContext.InstrumentListOperations.Create("sample-list" + Suffix.New());
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers", MethodType.Operation, Publish.Secondary);
            //Add the search results to the list
            ExtractionsContext.InstrumentListOperations.ReplaceAllWithIdentifiers(instrumentList, nysRics, false);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request Time and Sales data for RICs")]
        public void RequestTimeAndSalesDataForRics()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Find fields that are trade or quote related
            var fields = availableFields
                .Where(x => x.FieldGroup == "Quote" || x.FieldGroup == "Trade")
                .Select(x => x.Name).ToArray();

            //Request an extraction
            //Reduce the poll time for the purposes of the example (to trigger async processing and exhibit a polling step).
            ExtractionsContext.Preferences.WaitSeconds = 5;
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var startDate = new DateTimeOffset(2016, 9, 29, 0, 0, 0, TimeSpan.FromHours(-5)); //Central Time Zone
            var endDate = startDate.AddHours(12);
            var result = ExtractionsContext.ExtractRaw(
                new TickHistoryTimeAndSalesExtractionRequest
                {
                    Condition = new TickHistoryTimeAndSalesCondition
                    {
                        ReportDateRangeType = ReportDateRangeType.Range,
                        QueryStartDate = startDate,
                        QueryEndDate = endDate,
                        ApplyCorrectionsAndCancellations = false,
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        SortBy = TickHistorySort.SingleByRic,
                        DisplaySourceRIC = false
                    },
                    ContentFieldNames = fields,
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, "IBM.N"),
                        }
                    },
                });

            //Download the results
            using (var response = ExtractionsContext.RawExtractionResultOperations.GetReadStream(result))
            using (var stream = response.Stream)
            using (var reader = new StreamReader(stream))
            {
                var lineCount = 0;
                while (!reader.EndOfStream && lineCount++ < 10) //Limit to ten rows for the example
                    Status.WriteLine(reader.ReadLine());
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request Market Depth Data for RICs")]
        public void RequestMarketDepthForRics()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryMarketDepth);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Request an extraction
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var startDate = new DateTimeOffset(2008, 10, 13, 13, 0, 0, TimeSpan.FromHours(-5)); //Central Time Zone
            var endDate = startDate.AddDays(30);
            var result = ExtractionsContext.ExtractRaw(
                new TickHistoryMarketDepthExtractionRequest
                {
                    Condition = new TickHistoryMarketDepthCondition
                    {
                        ReportDateRangeType = ReportDateRangeType.Range,
                        QueryStartDate = startDate,
                        QueryEndDate = endDate,
                        NumberOfLevels = 10,
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        SortBy = TickHistorySort.SingleByRic,
                        View = TickHistoryMarketDepthViewOptions.NormalizedLL2,
                        DisplaySourceRIC = true
                    },
                    ContentFieldNames = availableFields.Select(f=>f.Name).ToArray(),
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, "VOD.L")
                        }
                    },
                });
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetReadStream", MethodType.Operation, Publish.Secondary);
            //Download the results
            using (var response = ExtractionsContext.RawExtractionResultOperations.GetReadStream(result))
            using (var stream = response.Stream)
            using (var reader = new StreamReader(stream))
            {
                var lineCount = 0;
                while (!reader.EndOfStream && lineCount++ < 10) //Limit to ten rows for the example
                    Status.WriteLine(reader.ReadLine());
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request Intra-day bars data for RICs")]
        public void RequestIntradayBarsDataForRics()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryIntradaySummaries);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Request an extraction
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var startDate = new DateTimeOffset(2008, 10, 13, 13, 0, 0, TimeSpan.FromHours(-5)); //Central Time Zone
            var endDate = startDate.AddHours(1);
            var result = ExtractionsContext.ExtractRaw(
                new TickHistoryIntradaySummariesExtractionRequest
                {
                    Condition = new TickHistoryIntradaySummariesCondition
                    {
                        //Possible values: OneSecond, FiveSeconds, OneMinute, FiveMinutes, TenMinutes, FifteenMinutes, OneHour
                        SummaryInterval = TickHistorySummaryInterval.OneMinute,
                        //Possible values: Ric, Entity
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        ReportDateRangeType = ReportDateRangeType.Range,
                        QueryStartDate = startDate,
                        QueryEndDate = endDate,
                        //Possible values: SingleByRic, SingleByTimestamp
                        SortBy = TickHistorySort.SingleByRic,
                        TimebarPersistence = true,
                        DisplaySourceRIC = true
                    },
                    ContentFieldNames = availableFields.Select(x => x.Name).ToArray(),
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, "EUR=")
                        }
                    },
                });
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "GetReadStream", MethodType.Operation, Publish.Secondary);
            //Download the results
            using (var response = ExtractionsContext.RawExtractionResultOperations.GetReadStream(result))
            using (var stream = response.Stream)
            using (var reader = new StreamReader(stream))
            {
                var lineCount = 0;
                while (!reader.EndOfStream && lineCount++ < 10) //Limit to ten rows for the example
                    Status.WriteLine(reader.ReadLine());
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request End of Day data for RICs")]
        public void RequestEndOfDayDataForRics()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.ElektronTimeseries);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Primary);
            //Reduce the poll time for the purposes of the example (to trigger async processing and exhibit a polling step).
            ExtractionsContext.Preferences.WaitSeconds = 5;
            //Request an extraction
            var extractionRequest = new ElektronTimeseriesExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "TRI.N", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new ElektronTimeseriesCondition
                {
                    ReportDateRangeType = ReportDateRangeType.Range,
                    QueryStartDate = new DateTimeOffset(new DateTime(2015, 12, 1)),
                    QueryEndDate = new DateTimeOffset(new DateTime(2015, 12, 2))
                },
                ContentFieldNames = new[]
                {
                    "Ask", "Bid", "High", "Last", "Low", "Trade Date"
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }
            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request Corporate Actions Data")]
        public void RequestCorporateActionsData()
        {
            ExtractionsContext.Preferences.WaitSeconds = 5; //DO NOT INCLUDE: Only for the purposes of the example.
            Status.Notify(ExtractionsContext, null, "GetValidContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.WriteLine(String.Join(", ", availableFields.Select(lbp => lbp.Code)));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the new report template
            var extractionRequest = new CorporateActionsStandardExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] {
                    "Accounting Standard",	"Annualised Dividend Gross Amount",	"Capital Change Market Event ID",	
                    "Capital Change Stock Different Flag",	"Company Role Description",	"Corporate Action Notes",	
                    "Corporate Actions ID",	"Corporate Actions Type",	"Currency Code",	"CUSIP",	
                    "Delete Marker",	"Fitch Issuer ID",	"GICS Industry Code",	"GICS Industry Code Description",	
                    "Instrument ID",	"Instrument ID Type",	"ISIN",	"Issue Level Event ID",	"Mandatory/Voluntary Indicator",	
                    "MIC",	"Moodys Issuer ID",	"Nominal Value",	"Nominal Value Currency",	"OPOL",	"Payment Status",	
                    "Period End Date",	"Period Length",	"Period Units",	"Period Units Description",	"PILC",	"QDI Eligibility Code",	
                    "QDI Eligibility Code Description",	"QDI Percent",	"RIC",	"S&P Issuer ID",	"Security Description",	
                    "Security Long Description",	"SEDOL",	"Shares Amount",	"Shares Amount Date",	
                    "Shares Amount In Thousands",	"Shares Amount Type",	"Shares Amount Type Default Flag",	
                    "Shares Amount Type Description",	"Shares Type",	"Shares Type Description",
                },
                Condition = new CorporateActionsStandardCondition
                {
                    QueryStartDate = new DateTimeOffset(2016, 06, 01, 0, 0, 0, TimeSpan.Zero),
                    QueryEndDate = new DateTimeOffset(2016, 09, 30, 0, 0, 0, TimeSpan.Zero),
                    ReportDateRangeType = ReportDateRangeType.Range,
                    CorporateActionsCapitalChangeType = CorporateActionsCapitalChangeType.CapitalChangeAnnouncementDate,
                    CorporateActionsDividendsType = CorporateActionsDividendsType.DividendPayDate,
                    CorporateActionsEarningsType = CorporateActionsEarningsType.PeriodEndDate,
                    CorporateActionsEquityOfferingsType = CorporateActionsEquityOfferingsType.AllPendingDeals,
                    CorporateActionsMergersAcquisitionsType = CorporateActionsMergersAcquisitionsType.DealAnnouncementDate,
                    CorporateActionsNominalValueType = CorporateActionsNominalValueType.NominalValueDate,
                    CorporateActionsSharesType = CorporateActionsSharesType.SharesAmountDate,
                    CorporateActionsStandardEventsType = CorporateActionsStandardEventsType.SHO,
                    CorporateActionsVotingRightsType = CorporateActionsVotingRightsType.VotingRightsDate,
                    NextDays = null,
                    PreviousDays = null,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    IncludeInstrumentsWithNoEvents = true,
                    IncludeNullDates = false,
                    ExcludeDeletedEvents = true,
                    IncludeCapitalChangeEvents = true,
                    IncludeDividendEvents = true,
                    IncludeEarningsEvents = true,
                    IncludeMergersAndAcquisitionsEvents = true,
                    IncludeNominalValueEvents = true,
                    IncludePublicEquityOfferingsEvents = true,
                    IncludeSharesOutstandingEvents = true,
                    IncludeVotingRightsEvents = true,
                    ShareAmountChoice = ShareAmountChoice.All,
                },
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            var extractionResult = ExtractionsContext.ExtractWithNotes(extractionRequest);
            var extractedRows = extractionResult.Contents;

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var row in extractedRows)
                    Status.WriteLine(
                        row.Identifier + " (" + row.IdentifierType + ") " +
                        String.Join(", ", row.DynamicProperties.Select(dp => dp.Key + "=" + dp.Value)));
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);

            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request Raw Transactions Data")]
        public void RequestRawTransactionsData()
        {
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Request an extraction
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var result = ExtractionsContext.ExtractRaw(
                new TickHistoryRawExtractionRequest
                {
                    Condition = new TickHistoryRawCondition
                    {
                        ReportDateRangeType = ReportDateRangeType.Range,
                        QueryStartDate = new DateTimeOffset(2016, 07, 25, 20, 0, 0, TimeSpan.Zero),
                        QueryEndDate = new DateTimeOffset(2016, 08, 02, 19, 59, 59, TimeSpan.Zero),
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        SortBy = TickHistorySort.SingleByRic,
                        DomainCode = TickHistoryRawDomain.MarketPrice,
                        DisplaySourceRIC = true
                    },
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, "ICBK.NS")
                        }
                    },
                });
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "RawExtractionResult", "GetReadStream", MethodType.Operation, Publish.Secondary);
            //Download the results
            using (var response = ExtractionsContext.RawExtractionResultOperations.GetReadStream(result))
            using (var stream = response.Stream)
            using (var reader = new StreamReader(stream))
            {
                var lineCount = 0;
                while (!reader.EndOfStream && lineCount++ < 10) //Limit to ten rows for the example
                    Status.WriteLine(reader.ReadLine());
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request Raw Transactions Data - Save to File")]
        public void RequestRawTransactionsDataAndSaveToGzipFile()
        {
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Request an extraction
            //ExtractionsContext.Options.AutomaticDecompression = true; //No decompression because we want to save the compressed file
            var result = ExtractionsContext.ExtractRaw(
                new TickHistoryRawExtractionRequest
                {
                    Condition = new TickHistoryRawCondition
                    {
                        ReportDateRangeType = ReportDateRangeType.Range,
                        QueryStartDate = new DateTimeOffset(2016, 07, 25, 20, 0, 0, TimeSpan.Zero),
                        QueryEndDate = new DateTimeOffset(2016, 08, 02, 19, 59, 59, TimeSpan.Zero),
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        SortBy = TickHistorySort.SingleByRic,
                        DomainCode = TickHistoryRawDomain.MarketPrice,
                        DisplaySourceRIC = true
                    },
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, "ICBK.NS")
                        }
                    },
                });
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "RawExtractionResult", "GetReadStream", MethodType.Operation, Publish.Secondary);
            //Download the results
            using (var response = ExtractionsContext.RawExtractionResultOperations.GetReadStream(result))
                using (var fileStream = File.Create("savedExtraction.gzip"))
                    response.Stream.CopyTo(fileStream);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request Auctions messages in the TRTH Time and Sales message type")]
        public void RequestActionsMessageInTimeAndSales()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Request an extraction
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var startDate = new DateTimeOffset(2008, 10, 13, 13, 0, 0, TimeSpan.FromHours(-5)); //Central Time Zone
            var endDate = startDate.AddDays(30);
            var result = ExtractionsContext.ExtractRaw(
                new TickHistoryTimeAndSalesExtractionRequest
                {
                    Condition = new TickHistoryTimeAndSalesCondition
                    {
                        ReportDateRangeType = ReportDateRangeType.Range,
                        QueryStartDate = startDate,
                        QueryEndDate = endDate,
                        ApplyCorrectionsAndCancellations = false,
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        SortBy = TickHistorySort.SingleByRic,
                        DisplaySourceRIC = true
                    },
                    ContentFieldNames = new []
                    {
                        "Auction - Exchange Time", 
                        "Auction - Price", 
                        "Auction - Volume", 
                        "Auction - Qualifiers", 
                        "Quote - Price", 
                        "Quote - Volume", 
                        "Quote - Bid Price", 
                        "Quote - Bid Size", 
                        "Quote - Ask Price", 
                        "Quote - Ask Size", 
                        "Quote - Qualifiers"
                    },
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, "BP.L")
                        }
                    },
                });
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "RawExtractionResult", "GetReadStream", MethodType.Operation, Publish.Secondary);
            //Download the results
            using (var response = ExtractionsContext.RawExtractionResultOperations.GetReadStream(result))
            using (var stream = response.Stream)
            using (var reader = new StreamReader(stream))
            {
                var lineCount = 0;
                const int typeIndex = 3; //The 4th field (0 based) is the "Type" field.
                while (!reader.EndOfStream && lineCount < 10)
                {
                    var line = reader.ReadLine();
                    if (String.IsNullOrEmpty(line))
                        continue;
                    var columns = line.ToLowerInvariant().Split(',');
                    if (columns.Length > typeIndex && columns[typeIndex] == "auction")
                    {
                        Status.WriteLine(line);
                        lineCount++;
                    }
                }
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request Market Statistics messages in the TRTH Time and Sales message type")]
        public void RequestMarketStatisticsMessagesInTimeAndSales()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Reduce the poll time for the purposes of the example (to trigger async processing and exhibit a polling step).
            ExtractionsContext.Preferences.WaitSeconds = 5;
            //Request an extraction
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var result = ExtractionsContext.ExtractRaw(
                new TickHistoryTimeAndSalesExtractionRequest
                {
                    Condition = new TickHistoryTimeAndSalesCondition
                    {
                        ReportDateRangeType = ReportDateRangeType.Range,
                        QueryStartDate = new DateTimeOffset(1996, 01, 02, 0, 0, 0, TimeSpan.Zero),
                        QueryEndDate = new DateTimeOffset(2007, 03, 03, 0, 0, 0, TimeSpan.Zero),
                        ApplyCorrectionsAndCancellations = true,
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        SortBy = TickHistorySort.SingleByRic,
                        DisplaySourceRIC = true,
                    },
                    ContentFieldNames = new DssCollection<string>()
                    {
                        "Trade - Dealing Code",	"Trade - Trade Sequence Number",	"Trade - Trading Status",	"Trade - Exchange Time",	"Trade - Date",	"Trade - Advancing Issues",	"Trade - Declining Issues",	
                        "Trade - Unchanged Issues",	"Trade - Total Issues",	"Trade - Advancing Volume",	"Trade - Declining Volume",	"Trade - Unchanged Volume",	
                        "Trade - Total Volume",	"Trade - New Highs",	"Trade - New Lows",	"Trade - Total Moves",	"Trade - Percent Change",	
                        "Trade - Advancing Moves",	"Trade - Declining Moves",
                    },
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, ".AD.N")
                        }
                    },
                });

            //Download the results
            using (var response = ExtractionsContext.RawExtractionResultOperations.GetReadStream(result))
            using (var stream = response.Stream)
            using (var reader = new StreamReader(stream))
            {
                var lineCount = 0;
                //const int typeIndex = 3; //The 4th field (0 based) is the "Type" field.
                while (!reader.EndOfStream && lineCount < 200)
                {
                    var line = reader.ReadLine();
                    if (String.IsNullOrEmpty(line))
                        continue;
                    var columns = line.ToLowerInvariant().Split(',');
                    //if (columns.Length > typeIndex && columns[typeIndex] == "auction")
                    {
                        Status.WriteLine(line);
                        lineCount++;
                    }
                }
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request NYSE Order Imbalance data")]
        public void RequestNyseOrderImbalanceData()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Request an extraction
            //Reduce the poll time for the purposes of the example (to trigger async processing and exhibit a polling step).
            ExtractionsContext.Preferences.WaitSeconds = 5;
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var result = ExtractionsContext.ExtractRaw(
                new TickHistoryTimeAndSalesExtractionRequest
                {
                    Condition = new TickHistoryTimeAndSalesCondition
                    {
                        ApplyCorrectionsAndCancellations = true,
                        QueryStartDate = new DateTimeOffset(2010, 04, 15, 13, 30, 0, TimeSpan.Zero),
                        QueryEndDate = new DateTimeOffset(2010, 04, 15, 14, 0, 0, TimeSpan.Zero),
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        ReportDateRangeType = ReportDateRangeType.Range,
                        SortBy = TickHistorySort.SingleByRic,
                        DisplaySourceRIC = true
                    },
                    ContentFieldNames = new [] {
                        "Quote - Imbalance Activity Type",
                        "Quote - Imbalance Quantity",
                        "Quote - Imbalance Side",
                        "Quote - Imbalance Variation Indicator",
                        "Quote - Volume",
                        "Quote - Far Clearing Price",
                        "Quote - Near Clearing Price",
                        "Quote - Reference Price",
                        "Quote - Qualifiers",
                        "Quote - Paired Quantity"
                    },
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, "IBM.N")
                        }
                    },
                });

            //Download the results
            using (var response = ExtractionsContext.RawExtractionResultOperations.GetReadStream(result))
            using (var stream = response.Stream)
            using (var reader = new StreamReader(stream))
            {
                var lineCount = 0;
                //const int typeIndex = 3; //The 4th field (0 based) is the "Type" field.
                while (!reader.EndOfStream)// && lineCount < 10)
                {
                    var line = reader.ReadLine();
                    if (String.IsNullOrEmpty(line))
                        continue;
                    var columns = line.ToLowerInvariant().Split(',');
                    //if (columns.Length > typeIndex && columns[typeIndex] == "auction")
                    {
                        Status.WriteLine(line);
                        lineCount++;
                    }
                }
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Request Corrections messages in the TRTH Time and Sales message type")]
        public void RequestCorrectionsMessagesInTimeAndSales()
        {
            /*Request Corrections messages in the TRTH Time and Sales message type (primarily to identify trades that were 
              cancelled/corrected, outside the market hours corrections may include Admin Closing run Correction messages) 
             */
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Request an extraction
            //Reduce the poll time for the purposes of the example (to trigger async processing and exhibit a polling step).
            ExtractionsContext.Preferences.WaitSeconds = 5;
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var result = ExtractionsContext.ExtractRaw(
                new TickHistoryTimeAndSalesExtractionRequest
                {
                    Condition = new TickHistoryTimeAndSalesCondition
                    {
                        ReportDateRangeType = ReportDateRangeType.Range,
                        QueryStartDate = new DateTimeOffset(2012, 09, 01, 08, 0, 0, TimeSpan.Zero),
                        QueryEndDate = new DateTimeOffset(2012, 09, 08, 23, 59, 59, TimeSpan.Zero),
                        ApplyCorrectionsAndCancellations = true,
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        SortBy = TickHistorySort.SingleByRic,
                        DisplaySourceRIC = true,
                    },
                    ContentFieldNames = availableFields.Where(x=>x.FieldGroup=="Correction").Select(x => x.Name).ToArray(),
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, "BP.L")
                        }
                    },
                });

            //Download the results
            using (var response = ExtractionsContext.RawExtractionResultOperations.GetReadStream(result))
            using (var stream = response.Stream)
            using (var reader = new StreamReader(stream))
            {
                var lineCount = 0;
                //const int typeIndex = 3; //The 4th field (0 based) is the "Type" field.
                while (!reader.EndOfStream)// && lineCount < 10)
                {
                    var line = reader.ReadLine();
                    if (String.IsNullOrEmpty(line))
                        continue;
                    var columns = line.ToLowerInvariant().Split(',');
                    //if (columns.Length > typeIndex && columns[typeIndex] == "auction")
                    {
                        Status.WriteLine(line);
                        lineCount++;
                    }
                }
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Find RIC renames using Historical Search By Entity")]
        public void FindRicRenames()
        {
            var start = new DateTimeOffset(2008, 04, 14, 00, 00, 00, TimeSpan.Zero);
            var end = start.AddDays(4);

            Status.Notify(SearchContext, null, "HistoricalSearch", MethodType.Operation, Publish.Secondary);
            //Perform a historical search by Entity. Because this is a 'by entity' search the historical
            //RICs will be returned.
            SearchContext.HistoricalSearch(HistoricalSearchRequest.Create(
                "TRIL.L", IdentifierType.Ric, DateTimeRange.Create(start, end), HistoricalResultsBy.Entity, null));
            Status.EndNotify(SearchContext);

            Status.Notify(SearchContext, null, "HistoricalSearch", MethodType.Operation, Publish.Secondary);
            //Perform a historical search by RIC. Because this is a 'by RIC' search the historical
            //RICs will be not be returned.
            SearchContext.HistoricalSearch(HistoricalSearchRequest.Create(
                "TRIL.L", IdentifierType.Ric, DateTimeRange.Create(start, end), HistoricalResultsBy.Ric, null));
            Status.EndNotify(SearchContext);
        }

        [Example("Identify the RIC Symbology changes for RTR.L")]
        public void IdentifyRicSymbologyChanges()
        {
            Status.Notify(SearchContext, null, "HistoricalSearch", MethodType.Operation, Publish.Secondary);
            var start = new DateTimeOffset(2008, 04, 14, 00, 00, 00, TimeSpan.Zero);
            var end = start.AddDays(4);
            SearchContext.HistoricalSearch(HistoricalSearchRequest.Create(
                "RTR.L", IdentifierType.Ric, DateTimeRange.Create(start, end), HistoricalResultsBy.Entity, null));
            Status.EndNotify(SearchContext);
        }

        [Example("Message type to field list")]
        public void MessageTypeToFieldList()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            var fields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            //Output
            Status.WriteLine("=== Time and Sales Fields ===");
            foreach (var field in fields.Take(10))
                Status.WriteLine("{0} :: {1} :: {2}", field.FieldGroup, field.Name, field.Description);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            fields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryIntradaySummaries);
            //Output
            Status.WriteLine("=== Intraday Summaries Fields ===");
            foreach (var field in fields.Take(10))
                Status.WriteLine("{0} :: {1} :: {2}", field.FieldGroup, field.Name, field.Description);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            fields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryMarketDepth);
            //Output
            Status.WriteLine("=== Market Depth Fields ===");
            foreach (var field in fields.Take(10))
                Status.WriteLine("{0} :: {1} :: {2}", field.FieldGroup, field.Name, field.Description);
            Status.EndNotify(ExtractionsContext);

        }

        [Example("Cancel a request which was incorrectly set up or is too large")]
        public void CancelARequest()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            //Request an extraction. Use the "start" version of the method to return control to the code with a job handle.
            var startDate = new DateTimeOffset(2008, 10, 13, 13, 0, 0, TimeSpan.FromHours(-5)); //Central Time Zone
            var endDate = startDate.AddDays(30);
            var job = ExtractionsContext.ExtractRawStart(
                new TickHistoryTimeAndSalesExtractionRequest
                {
                    Condition = new TickHistoryTimeAndSalesCondition
                    {
                        ReportDateRangeType = ReportDateRangeType.Range,
                        QueryStartDate = startDate,
                        QueryEndDate = endDate,
                        ApplyCorrectionsAndCancellations = true,
                        ExtractBy = TickHistoryExtractByMode.Ric,
                        MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                        SortBy = TickHistorySort.SingleByRic,
                        DisplaySourceRIC = true
                    },
                    ContentFieldNames = availableFields.Select(x => x.Name).Take(10).ToArray(),
                    IdentifierList = new InstrumentIdentifierList
                    {
                        InstrumentIdentifiers = new[]
                        {
                            InstrumentIdentifier.Create(IdentifierType.Ric, "SPY.P")
                        }
                    },
                });

            //Sleep for 6 seconds
            System.Threading.Thread.Sleep(TimeSpan.FromSeconds(6));
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "CancelJob", MethodType.Operation, Publish.Secondary);
            //Attempt cancellation of the job
            ExtractionsContext.CancelJob(job);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, null, "MonitorJob", MethodType.Operation, Publish.Secondary);
            //Retrieve an updated status of the job.  Note: It may take a few seconds, perhaps a minute for larger
            //extractions for the cancellation to take effect. It is also possible that the cancellation may not
            //be accepted.
            try
            {
                job = ExtractionsContext.MonitorJob(job);
                Status.WriteLine("Status: {0}", job.Status);
            }
            catch (Exception)
            {
                Status.WriteLine("Status: {0}", job.Status);
            }
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Scheduled Extraction of Tick History content")]
        public void ScheduledExtractionForTh()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.TickHistoryTimeAndSales);
            Status.EndNotify(ExtractionsContext);

            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "ScheduledExtractionForTH" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");

            var startDate = DateTime.Now.AddDays(-7);
            var endDate = DateTime.Now;

            //Define a new end of day report template
            var reportTemplate = new TickHistoryTimeAndSalesReportTemplate
            {
                Name = "ScheduledExtractionForTH" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                 Condition = new TickHistoryTimeAndSalesCondition
                 {
                     ReportDateRangeType = ReportDateRangeType.Range,
                     QueryStartDate = startDate,
                     QueryEndDate = endDate
                 }
                     //NOTE: There are many other fields which can be use to customize the report template
                 };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(availableFields
                .Where(x => x.FieldGroup == "Quote" || x.FieldGroup == "Trade")
                .Select(x => x.Name).ToArray());


            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "ScheduledExtractionForTH" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow.AddDays(1), isImmediate: false),
                Trigger = new TimeTrigger
                {
                    LimitReportToTodaysData = false,
                    At = new[] { new HourMinute { Hour = 5, Minute = 0 } }
                },
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "ExtractNow");
            //Force an extraction which does not take the schedule or triggering into consideration.  You can use ExtractNow
            //to preview the results of an extraction.
            var extraction = ExtractionsContext.ScheduleOperations.ExtractNow(schedule);

            //Output some extraction information
            Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
            extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
            extraction.Files.Count);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association);
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                ExtractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(ExtractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream", MethodType.Stream);
                    //Output the contents of the extracted file
                    ExtractionsContext.Options.AutomaticDecompression = true;
                    var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
					string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
	                {
		                Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
	                }
	                Status.EndNotify(ExtractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        /// <summary>
        /// This use case illustrates using historical search by specifying certain criteria such as exchange (by using RIC suffix) 
        /// to retrieve all instruments trading on the NYSE.
        /// </summary>
        [Example("Historical Search - Criteria Based Search - Search by Exchange")]
        public void HistoricalSearchByExchange()
        {
            // limit the search to a single day to speed up the result for this example.
            var end = new DateTimeOffset(2010, 11, 08, 00, 00, 00, TimeSpan.Zero);
            var start = end.AddDays(-1);
            Status.Notify(SearchContext, null, "HistoricalCriteriaSearch", MethodType.Operation, Publish.Secondary);

            var results = SearchContext.HistoricalCriteriaSearch(HistoricalCriteriaSearchRequest.Create(
                "\\.N$",        // regex pattern, this pattern finds all instruments that end with ".N".
                null, null, null, null, null, null, null, null, null, null, null, null, null, null, null,
                DateTimeRange.Create(start, end),
                HistoricalResultsBy.Ric));

            var counter = 0;
            foreach (var result in results)
            {
                Status.WriteLine("Identifier={0},IdentifierType={1}", result.Identifier, result.IdentifierType.ToString());
                counter++;
                if (counter > 20)
                    break;
            }

            Status.EndNotify(SearchContext);
        }

        /// <summary>
        /// This use case illustrates using a criteria-based historical search to retrieve all fixed income instruments 
        /// that have a maturity date in the month of September 2016 and a coupon rate greater than or equal to 30.
        /// </summary>
        [Example("Historical Search - Criteria Based Search - Fixed Income Instruments")]
        public void HistoricalSearchFixedIncome()
        {
            var rangeEnd = new DateTimeOffset(2016, 09, 30, 00, 00, 00, TimeSpan.Zero);
            var rangeStart = rangeEnd.AddYears(-1);

            var maturityDateStartDate = new DateTimeOffset(2016, 09, 01, 00, 00, 00, TimeSpan.Zero);
            var maturityDateEndDate = new DateTimeOffset(2016, 09, 30, 23, 59, 59, TimeSpan.Zero);

            Status.Notify(SearchContext, null, "HistoricalCriteriaSearch", MethodType.Operation, Publish.Secondary);
            var results = SearchContext.HistoricalCriteriaSearch(HistoricalCriteriaSearchRequest.Create(
                null, null, null, null, null, null, null, null, null, null, null, null,
                DecimalValueRange.Create(null, 30, null), 
                null, null,
                DateTimeRange.Create(maturityDateStartDate, maturityDateEndDate),
                DateTimeRange.Create(rangeStart, rangeEnd),
                HistoricalResultsBy.Ric));

            var counter = 0;
            foreach (var result in results)
            {
                Status.WriteLine("Identifier={0},IdentifierType={1}", result.Identifier, result.IdentifierType.ToString());
                counter++;
                if (counter > 20)
                    break;
            }
            Status.EndNotify(SearchContext);
        }

        /// <summary>
        /// This use case illustrates extracting reference data for RIC G.N (Gillette) back to 1996.
        /// </summary>
        [Example("Reference History - Gillette-Genpact")]
        public void HistoricalReferenceHistoryGilletteGenpact()
        {
            Status.Notify(SearchContext, null, "ReferenceHistory", MethodType.Operation, Publish.Secondary);
            var rangeStartDate = new DateTimeOffset(1996, 01, 01, 00, 00, 00, TimeSpan.Zero);
            var rangeEndDate = new DateTimeOffset(2016, 09, 30, 00, 00, 00, TimeSpan.Zero);

            // get the reference history.
            var history = SearchContext.ReferenceHistory
            (
                ReferenceHistoryRequest.Create(new DssCollection<string>("G.N"), 
                DateTimeRange.Create(rangeStartDate, rangeEndDate), 
                HistoricalResultsBy.Ric)
            );
            Status.EndNotify(SearchContext);
        }

        /// <summary>
        /// This use case illustrates filtering on specific FIDs for a US Treasury in a Tick History Raw extraction. 
        /// </summary>
        [Example("Request Raw Extraction - FID Filtering")]
        public void TickHistoryRawExtractionFidFiltering()
        {
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);

            //Create the request
            var extractionRequest = new TickHistoryRawExtractionRequest()
            {
                IdentifierList = new InstrumentIdentifierList()
                {
                    InstrumentIdentifiers = new[] { new InstrumentIdentifier { Identifier = "US10YT=RRPS", IdentifierType = IdentifierType.Ric } }
                },
                Condition = new TickHistoryRawCondition()
                {
                    DaysAgo = null,
                    MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                    QueryStartDate = new DateTimeOffset(2016, 10, 01, 00, 00, 00, TimeSpan.Zero),
                    QueryEndDate = new DateTimeOffset(2016, 11, 09, 00, 00, 00, TimeSpan.Zero),
                    ReportDateRangeType = ReportDateRangeType.Range,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                    DomainCode = TickHistoryRawDomain.MarketPrice,
                    DisplaySourceRIC = false,
                    Fids = "68",
                    Preview = PreviewMode.None,
                }
            };

            //Extract - NOTE: If the extraction request takes more than 30 seconds the async mechansim will be used.  See Key Mechanisms 
            ExtractionsContext.Options.AutomaticDecompression = true; //Decompress gzip to plain text
            var extractionResult = ExtractionsContext.ExtractRaw(extractionRequest);
            Status.EndNotify(ExtractionsContext);
            Status.Notify(ExtractionsContext, null, "ExtractRaw", MethodType.Operation, Publish.Secondary);
            var streamResponse = ExtractionsContext.GetReadStream(extractionResult);
            using (var reader = new StreamReader(streamResponse.Stream))
            {
                //Output
                var result = reader.ReadLine();
                if (string.IsNullOrEmpty(result))
                    Status.WriteLine("No raw results returned");
                else
                {
                    Status.WriteLine(result);
                    for (var lineCount = 0; lineCount < 100 && (result = reader.ReadLine()) != null; lineCount++) //Limit output results to 100 reads.
                    {
                        //result = reader.ReadLine();
                        Status.WriteLine(result);
                    }
                }
            }

            //Output Notes
            Status.WriteLine("NOTES:");
            foreach (var note in extractionResult.Notes)
                Status.WriteLine(note);
            Status.EndNotify(ExtractionsContext);
        }

        #region Cleanup Helper Method

        // This cleanup code serves as an example of how to do a get-all for the various entity types.
        // The cleanup button is useful if you abort debug sessions while exploring these samples.  
        // Cleanup is by name prefix.  It removes all lists, reports, schedules, and file content that matches the prefix.

        public void Cleanup(Schedule schedule, InstrumentList instrumentList, ReportTemplate reportTemplate)
        {
            Cleanup(schedule, reportTemplate);

            //Delete the instrument list
            if (instrumentList != null && instrumentList.ListId != null)
                ExtractionsContext.InstrumentListOperations.Delete(instrumentList);
        }

        public void Cleanup(Schedule schedule, EntityList entityList, ReportTemplate reportTemplate)
        {
            Cleanup(schedule, reportTemplate);

            //Delete the entity list
            if (entityList != null && entityList.ListId != null)
                ExtractionsContext.EntityListOperations.Delete(entityList);
        }

        public void Cleanup(Schedule schedule, ReportTemplate reportTemplate)
        {
            // The order of cleanup is intentional.  Minimizes the potential impact of orphaned entities (if any).

            // ReportExtractions and ExtractedFiles are immutable, but the file content needs to be cleaned up.
            // The ReportExtractions and ExtractedFiles will still be retrievable, but the file content will be gone.

            if (schedule != null && schedule.ScheduleId != null)
            {
                var scheduleExtractions = ExtractionsContext.ReportExtractionOperations.GetByScheduleId(schedule.ScheduleId);
                foreach (var reportExtraction in scheduleExtractions)
                {
                    ExtractionsContext.LoadProperty(reportExtraction, "Files");
                    var fileIds = reportExtraction.Files
                                                  .Where(f => f.ContentsExists)
                                                  .Select(f => f.ExtractedFileId)
                                                  .ToList();
                    if (fileIds.Count > 0)
                        ExtractionsContext.ExtractedFileOperations.DeleteMultiple(fileIds);
                }
                ExtractionsContext.ScheduleOperations.Delete(schedule);
            }

            if (reportTemplate != null && reportTemplate.ReportTemplateId != null)
                ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate);
        }

        #endregion


    }

}
